@@ -13,12 +13,13 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKCLIENT_H
-#define GKCLIENT_H "@(#) $Id: GkClient.h,v 1.43 2011/09/23 09:28:52 willamowius Exp $"
+#define GKCLIENT_H "@(#) $Id: GkClient.h,v 1.47 2012/12/10 19:22:22 shorne Exp $"
 
 #include "gk_const.h"
 #include "Toolkit.h"
 #include "Routing.h"
 #include "config.h"
+#include <map>
 
 class Q931;
 class H225_AliasAddress;
@@ -68,6 +69,14 @@ public:
 	bool IsNATed() const { return m_natClient != NULL; }
 	PString GetParent() const;
 
+	bool UsesAdditiveRegistration();
+	bool AdditiveRegister(H225_ArrayOf_AliasAddress & aliases, int & rejectReason, 
+						H225_ArrayOf_ClearToken * tokens, H225_ArrayOf_CryptoH323Token * cryptotokens);
+	bool AdditiveUnRegister(const H225_ArrayOf_AliasAddress & aliases);
+	void AppendLocalAlias(const H225_ArrayOf_AliasAddress & aliases);
+	void RemoveLocalAlias(const H225_ArrayOf_AliasAddress & aliases);
+
+	bool OnSendingGRQ(H225_GatekeeperRequest &grq);
 	bool OnSendingRRQ(H225_RegistrationRequest &rrq);
 	bool OnSendingARQ(H225_AdmissionRequest &arq, Routing::AdmissionRequest &req);
 	bool OnSendingLRQ(H225_LocationRequest &lrq, Routing::LocationRequest &req);
@@ -121,7 +130,7 @@ public:
 		for (PINDEX i = 0; i < m_h235Authenticators->GetSize();  i++) {
 			H235Authenticator * authenticator = (H235Authenticator *)(*m_h235Authenticators)[i].Clone();
 
-			if (authenticator->IsSecuredPDU(rasmsg.GetTag(), FALSE)) {
+			if (authenticator && authenticator->IsSecuredPDU(rasmsg.GetTag(), FALSE)) {
 				authenticator->SetLocalId(id);
 				authenticator->SetPassword(m_password);
 
@@ -195,6 +204,9 @@ private:
 	H225_GatekeeperIdentifier m_gatekeeperId;
 	PMutex m_rrqMutex;
 
+	/// Use Additive Registration
+	PBoolean m_useAdditiveRegistration;
+
 	/// reregistration timeout (seconds)
 	long m_ttl;
 	/// timeout to send a next RRQ (milliseconds)
@@ -247,9 +259,13 @@ private:
 
 #ifdef HAS_H46023
 	// Handle H46023 RCF
-	void HandleP2P_RCF(H460_FeatureStd * feat);
+	void H46023_RCF(H460_FeatureStd * feat);
     // Handle H46023 ACF
-	void HandleP2P_ACF(callptr m_call, H460_FeatureStd * feat);
+	void H46023_ACF(callptr m_call, H460_FeatureStd * feat);
+    // Run STUN test
+	void RunSTUNTest(const H323TransportAddress & addr);
+    // Force Reregistration
+	void H46023_ForceReregistration();
     // Notify NAT type
     bool H46023_TypeNotify(int & nattype);
 	// detected NAT type
@@ -260,13 +276,21 @@ private:
 	bool gk_H460_23;
 	// STUN Client
 	STUNClient * m_stunClient;
+	// ALG Detected
+	bool m_algDetected;
+    // Call NAT Strategy Map
+	PMutex m_strategyMutex;
+	std::map<H225_CallIdentifier, unsigned> m_natstrategy;
 
 public:
 	// NAT type detected
 	void H46023_TypeDetected(int nattype);
     // Create socket pair
-    bool H46023_CreateSocketPair(const H225_CallIdentifier & id, UDPProxySocket * & rtp, UDPProxySocket * & rtcp, bool & nated);
-
+    bool H46023_CreateSocketPair(const H225_CallIdentifier & id, WORD flcn, UDPProxySocket * & rtp, UDPProxySocket * & rtcp, bool & nated);
+	// Set the NAT Strategy
+	void H46023_SetNATStategy(const H225_CallIdentifier & id, unsigned nat);
+	// Find the NAT Strategy
+	CallRec::NatStrategy H46023_GetNATStategy(const H225_CallIdentifier & id);
 #endif
 };
 
