@@ -2,7 +2,7 @@
 //
 // GkStatus.cxx
 //
-// Copyright (c) 2000-2011, Jan Willamowius
+// Copyright (c) 2000-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -107,7 +107,7 @@ public:
 		int instanceNo
 		);
 		
-	virtual ~StatusClient() {}
+	virtual ~StatusClient();
 
 	virtual bool ReadCommand(
 		/// command that has been read (if ReadCommand succeeded)
@@ -369,6 +369,7 @@ SSHStatusClient::~SSHStatusClient()
 		ssh_bind_free(sshbind);
 	}
 	ssh_finalize();
+	GkStatus::Instance()->StatusClientDeleted();
 }
 
 #ifdef LARGE_FDSET
@@ -398,22 +399,26 @@ PBoolean SSHStatusClient::Accept(PSocket & socket)
 	}
 	if (!keyAvailable) {
 		PTRACE(1, "No DSA or RSA key file found");
+		SNMP_TRAP(7, SNMPError, Network, "No SSH keys");
 		socket.Close();
 		return false;
 	}
 
     if(ssh_init() < 0) {
 		PTRACE(1, "ssh_init() failed");
+		SNMP_TRAP(7, SNMPError, Network, "SSH init failed");
 		return false;
     }
 	ssh_bind_set_fd(sshbind, socket.GetHandle());
 	if(ssh_bind_accept(sshbind, session) == SSH_ERROR) {
 		PTRACE(1, "ssh_bind_accept() failed: " << ssh_get_error(sshbind));
+		SNMP_TRAP(7, SNMPError, Network, "SSH bind failed");
 		return false;
     }
 
     if(ssh_handle_key_exchange(session)) {
 		PTRACE(1, "ssh_handle_key_exchange failed: " << ssh_get_error(session));
+		SNMP_TRAP(7, SNMPError, Network, "SSH key exchange failed");
 		return false;
 	}
 	
@@ -434,6 +439,7 @@ bool SSHStatusClient::Authenticate()
         message = ssh_message_get(session);
         if (!message) {
 			PTRACE(1, "ssh read error: " << ssh_get_error(session));
+			SNMP_TRAP(7, SNMPError, Network, PString("SSH read error: ") + ssh_get_error(session));
             break;
 		}
 		switch(ssh_message_type(message)) {
@@ -483,6 +489,7 @@ bool SSHStatusClient::Authenticate()
     } while(message && !chan);
     if (!chan) {
         PTRACE(1, "Error establishing SSH channel: " << ssh_get_error(session));
+		SNMP_TRAP(7, SNMPError, Network, PString("SSH error: ") + ssh_get_error(session));    
         return false;
     }
 
@@ -817,12 +824,14 @@ GkStatus::GkStatus() : Singleton<GkStatus>("GkStatus"), SocketsReader(500)
 #endif
 
 	SetName("GkStatus");
+	m_statusClients = 0;
+	m_maxStatusClients = GkConfig()->GetInteger("MaxStatusClients", 20);
 	Execute();
 }
 
 void GkStatus::AuthenticateClient(StatusClient* newClient)
 {
-	if (newClient->Authenticate()) {
+	if ((m_statusClients++ < m_maxStatusClients) && (newClient->Authenticate())) {
 		newClient->SetTraceLevel(GkConfig()->GetInteger("StatusTraceLevel", MAX_STATUS_TRACE_LEVEL));
 		PTRACE(1, "STATUS\tNew client authenticated successfully: " << newClient->WhoAmI()
 			<< ", login: " << newClient->GetUser()
@@ -832,9 +841,7 @@ void GkStatus::AuthenticateClient(StatusClient* newClient)
 		newClient->Flush();
 		AddSocket(newClient);
 	} else {
-		PTRACE(3, "STATUS\tNew client rejected: " << newClient->WhoAmI()
-			<< ", login: " << newClient->GetUser()
-			);
+		PTRACE(3, "STATUS\tNew client rejected: " << newClient->WhoAmI() << ", login: " << newClient->GetUser());
 		newClient->WriteString("\r\nAccess forbidden!\r\n");
 		// newClient->Flush();	// dont' flush when access is denied to avoid blocking
 		delete newClient;
@@ -1046,10 +1053,8 @@ void GkStatus::OnStart()
 	m_commands["quit"] = e_Exit;
 	m_commands["q"] = e_Exit;
 	m_commands["trace"] = e_Trace;
-#if PTRACING
 	m_commands["rotatelog"] = e_RotateLog;
 	m_commands["setlog"] = e_SetLogFilename;
-#endif
 	m_commands["addincludefilter"] = e_AddIncludeFilter;
 	m_commands["removeincludefilter"] = e_RemoveIncludeFilter;
 	m_commands["addexcludefilter"] = e_AddExcludeFilter;
@@ -1067,6 +1072,7 @@ void GkStatus::OnStart()
 	m_commands["gci"] = e_GetAcctInfo;
 	m_commands["resetcallcounters"] = e_ResetCallCounters;
 	m_commands["printendpointqos"] = e_PrintEndpointQoS;
+	m_commands["printallconfigswitches"] = e_PrintAllConfigSwitches;
 }
 
 void GkStatus::ReadSocket(
@@ -1087,7 +1093,7 @@ void GkStatus::CleanUp()
 		iterator iter = m_removed.begin();
 		while (iter != m_removed.end()) {
 			StatusClient *client = static_cast<StatusClient *>(*iter);
-			if (!client->IsBusy()) {
+			if (client && !client->IsBusy()) {
 				iter = m_removed.erase(iter);
 				--m_rmsize;
 				delete client;
@@ -1096,7 +1102,7 @@ void GkStatus::CleanUp()
 			else ++iter;
 		}
 	}
-	RemoveClosed(false);
+	RemoveClosed(false);	// TODO: this runs every second, do we really need this here ? other SocketsReader don't do this
 }
 
 
@@ -1129,6 +1135,11 @@ StatusClient::StatusClient(
 	m_isFilteringActive = Toolkit::AsBool(GkConfig()->GetString(filteringsec, "Enable", "0"));
 }
 
+StatusClient::~StatusClient()
+{
+	GkStatus::Instance()->StatusClientDeleted();
+}
+
 bool StatusClient::ReadCommand(
 	/// command that has been read (if ReadCommand succeeded)
 	PString& cmd,
@@ -1286,8 +1297,7 @@ void StatusClient::Dispatch()
 
 void StatusClient::DoDebug(
 	/// tokenized debug command
-	const PStringArray& args
-	)
+	const PStringArray & args)
 {
 	bool tmp = m_isFilteringActive;
 	m_isFilteringActive = false;
@@ -1330,7 +1340,11 @@ void StatusClient::DoDebug(
 				WriteString(result + ";\r\n");
 			}
 		} else if ((args[1] *= "set") && (args.GetSize() >= 5)) {
-			Toolkit::Instance()->SetConfig(1, args[2], args[3], args[4]);
+			// re-assemble config value
+			PString value = args[4];
+			for (PINDEX i = 5; i < args.GetSize(); ++i)
+				value += " " + args[i];
+			Toolkit::Instance()->SetConfig(1, args[2], args[3], value);
 			WriteString(GkConfig()->GetString(args[2],args[3],"") + "\r\n");
 		} else if (args[1] *= "remove") {
 			if (args.GetSize() >= 4) {
@@ -1390,11 +1404,11 @@ bool StatusClient::CheckAuthRule(
 		result = AuthenticateUser();
 	} else {
 		PTRACE(1, "STATUS\tERROR: Unrecognized [GkStatus::Auth] rule (" << rule << ')');
+		SNMP_TRAP(7, SNMPError, Configuration, "Invalid [GkStatus::Auth] rule: " + rule);    
 	}
 	
 	PTRACE(4, "STATUS\tAuthentication rule '" << rule 
-		<< (result?"' accepted":"' rejected") << " the client " << Name() 
-		);
+		<< (result ? "' accepted" : "' rejected") << " the client " << Name());
 	return result;
 }
 
@@ -1759,7 +1773,6 @@ void StatusClient::ExecCommand(
 		WriteString("Output trace level is " + PString(m_traceLevel) + "\r\n");
 		break;
 
-#if PTRACING
 	case GkStatus::e_RotateLog:
 	    if (Gatekeeper::RotateLogFile())
 			WriteString("Log file rotation succeeded\r\n");
@@ -1776,7 +1789,6 @@ void StatusClient::ExecCommand(
 		} else
 			CommandError("Syntax Error: setlog <logfilepath>");
 		break;				
-#endif
 
 	case GkStatus::e_AddIncludeFilter:
 	    if (args.GetSize() == 2) {
@@ -1859,6 +1871,17 @@ void StatusClient::ExecCommand(
 		// print QoS values for all endpoints
 		SoftPBX::PrintEndpointQoS(this);
 		break;
+	case GkStatus::e_PrintAllConfigSwitches:
+		{
+			const char * sect = NULL;
+			unsigned j = 0;
+			while ((sect = KnownConfigEntries[j][0])) {
+				WriteString(PString(sect) + "," + PString(KnownConfigEntries[j][1]) + "\r\n");
+				j++;
+			}
+			WriteString(";\r\n");
+		}
+		break;
 	default:
 		// commmand not recognized
 		CommandError("Error: Unknown command '" + cmd + "'");
@@ -1965,12 +1988,14 @@ StatusListener::StatusListener(const Address & addr, WORD lport)
 	}
 	SetName(AsString(addr, GetPort()));
 	m_addr = addr;
-	Toolkit::Instance()->PortNotification(StatusPort, PortOpen, "tcp", m_addr, lport);
+	if (Toolkit::Instance()->IsPortNotificationActive())
+		Toolkit::Instance()->PortNotification(StatusPort, PortOpen, "tcp", m_addr, lport);
 }
 
 StatusListener::~StatusListener()
 {
-	Toolkit::Instance()->PortNotification(StatusPort, PortClose, "tcp", m_addr, GetPort());
+	if (Toolkit::Instance()->IsPortNotificationActive())
+		Toolkit::Instance()->PortNotification(StatusPort, PortClose, "tcp", m_addr, GetPort());
 }
 
 ServerSocket * StatusListener::CreateAcceptor() const
