@@ -3,7 +3,7 @@
 // Neighboring System for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2002-2003
-// Copyright (c) 2004-2011, Jan Willamowius
+// Copyright (c) 2004-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -72,7 +72,7 @@ void SetCryptoGkTokens(H225_ArrayOf_CryptoH323Token & cryptoTokens, const PStrin
 	cryptoGKPwdHash.m_timeStamp = cryptoEPPwdHash.m_timeStamp;
 	cryptoGKPwdHash.m_token.m_algorithmOID = OID_MD5;
 	cryptoGKPwdHash.m_token.m_hash = cryptoEPPwdHash.m_token.m_hash;
-	
+
 	cryptoTokens.Append(finalCryptoToken);
 }
 
@@ -123,7 +123,7 @@ namespace { // anonymous namespace
 	SimpleCreator<GlonetGK> GlonetGKCreator("GlonetGK");
 
 	int challenge;
-	const char OID_T[] = "0.0.8.235.0.2.5";
+	const char * const OID_T = "0.0.8.235.0.2.5";
 }
 
 // if we put nomatch into anonymous namespace,
@@ -147,6 +147,7 @@ private:
 template<class R>
 PrefixInfo LRQSender<R>::operator()(Neighbor *nb, WORD seqnum) const
 {
+	// select neighbor based on dialed alias
 	if (const H225_ArrayOf_AliasAddress *dest = m_r.GetAliases()) {
 		H225_ArrayOf_AliasAddress aliases;
 		if (PrefixInfo info = nb->GetPrefixInfo(*dest, aliases)) {
@@ -156,6 +157,16 @@ PrefixInfo LRQSender<R>::operator()(Neighbor *nb, WORD seqnum) const
 				return info;
 		}
 	}
+	// select neighbor based on dialed IP
+	if (const H225_TransportAddress *dest = m_r.GetDestIP()) {
+		H225_ArrayOf_AliasAddress aliases;
+		if (PrefixInfo info = nb->GetIPInfo(*dest, aliases)) {
+			H225_RasMessage lrq_ras;
+			H225_LocationRequest & lrq = nb->BuildLRQ(lrq_ras, seqnum, aliases);
+			if (nb->OnSendingLRQ(lrq, m_r) && nb->SendLRQ(lrq_ras))
+				return PrefixInfo(100, 1);
+		}
+	}
 	return nomatch;
 }
 
@@ -194,6 +205,13 @@ PrefixInfo LRQForwarder::operator()(Neighbor *nb, WORD /*seqnum*/) const
 Neighbor::Neighbor()
 {
 	m_rasSrv = RasServer::Instance();
+	m_port = 0;
+	m_forwardHopCount = 0;
+	m_dynamic = false;
+	m_acceptForwarded = true;
+	m_forwardResponse = false;
+	m_forwardto = 0;
+	m_externalGK = false;
 	m_keepAliveTimer = GkTimerManager::INVALID_HANDLE;
 	m_keepAliveTimerInterval = 0;
 	m_H46018Server = false;
@@ -294,15 +312,16 @@ bool Neighbor::SetProfile(const PString & id, const PString & type)
 	if (!m_dynamic && !GetTransportAddress(m_name, GK_DEF_UNICAST_RAS_PORT, m_ip, m_port))
 		return false;
 
-	PINDEX i;
 	m_sendPrefixes.clear();
 	PString sprefix(config->GetString(section, "SendPrefixes", ""));
 	PStringArray sprefixes(sprefix.Tokenise(",", false));
-	for (i = 0; i < sprefixes.GetSize(); ++i) {
+	for (PINDEX i = 0; i < sprefixes.GetSize(); ++i) {
 		PStringArray p(sprefixes[i].Tokenise(":=", false));
 		m_sendPrefixes[p[0]] = (p.GetSize() > 1) ? p[1].AsInteger() : 1;
 	}
- 
+
+	m_sendIPs = config->GetString(section, "SendIPs", "");
+
 	PString salias(config->GetString(section, "SendAliases", ""));
 	PStringArray defs(salias.Tokenise(",", FALSE));
 	m_sendAliases.SetSize(0);
@@ -334,7 +353,7 @@ bool Neighbor::SetProfile(const PString & id, const PString & type)
 			m_sendAliases.AppendString(defs[i]);
 		}
 	}
- 
+
 	PString aprefix(config->GetString(section, "AcceptPrefixes", "*"));
 	m_acceptPrefixes = PStringArray(aprefix.Tokenise(",", false));
 	if (m_keepAliveTimer != GkTimerManager::INVALID_HANDLE)
@@ -406,19 +425,18 @@ void Neighbor::SetH46018GkKeepAliveInterval(int interval)
 // initialize neighbor object created by SRV policy
 bool Neighbor::SetProfile(const PString & name, const H323TransportAddress & addr)
 {
-  addr.GetIpAndPort(m_ip,m_port);
-  m_id = "SRVrec";
-  m_name = name;
-  m_dynamic = false;
-  m_externalGK = true;
+	addr.GetIpAndPort(m_ip,m_port);
+	m_id = "SRVrec";
+	m_name = name;
+	m_dynamic = false;
+	m_externalGK = true;
 
-  m_sendPrefixes.clear();
-  m_sendPrefixes["*"] = 1;
- 
-  SetForwardedInfo(LRQFeaturesSection);
+	m_sendPrefixes.clear();
+	m_sendPrefixes["*"] = 1;
 
-  return true;
+	SetForwardedInfo(LRQFeaturesSection);
 
+	return true;
 }
 
 PrefixInfo Neighbor::GetPrefixInfo(const H225_ArrayOf_AliasAddress & aliases, H225_ArrayOf_AliasAddress & dest)
@@ -464,6 +482,33 @@ PrefixInfo Neighbor::GetPrefixInfo(const H225_ArrayOf_AliasAddress & aliases, H2
 	return PrefixInfo(0, (short)iter->second);
 }
 
+PrefixInfo Neighbor::GetIPInfo(const H225_TransportAddress & ip, H225_ArrayOf_AliasAddress & dest) const
+{
+	PStringArray sendNet(m_sendIPs.Tokenise(","));
+	for (PINDEX i=0; i < sendNet.GetSize(); ++i) {
+		bool noMatch = false;
+		if (sendNet[i].Left(1) == "!") {
+			noMatch = true;
+			sendNet[i] = sendNet[i].Mid(1); // cut away first char ("!")
+		}
+		NetworkAddress network = NetworkAddress(sendNet[i]);
+		PIPSocket::Address addr;
+		bool validNetwork = GetIPFromTransportAddr(ip, addr);
+		if ((sendNet[i] == "*")
+			|| ((sendNet[i] == "public") && !addr.IsRFC1918())
+			|| ((sendNet[i] == "private") && addr.IsRFC1918())
+			|| (validNetwork && !noMatch && (NetworkAddress(addr) << network))
+			|| (validNetwork && noMatch && !(NetworkAddress(addr) << network)) )
+		{
+			dest.SetSize(1);
+			H323SetAliasAddress(AsDotString(ip), dest[0], H225_AliasAddress::e_transportID);
+			return PrefixInfo(100, 1);
+		}
+	}
+
+	return nomatch;	// don't send to this neighbor
+}
+
 bool Neighbor::OnSendingLRQ(H225_LocationRequest & lrq)
 {
 	return true;
@@ -474,7 +519,7 @@ bool Neighbor::OnSendingLRQ(H225_LocationRequest & lrq, const AdmissionRequest &
 	return OnSendingLRQ(lrq);
 }
 
-bool Neighbor::OnSendingLRQ(H225_LocationRequest & lrq, const LocationRequest &orig_lrq)
+bool Neighbor::OnSendingLRQ(H225_LocationRequest & lrq, const LocationRequest & orig_lrq)
 {
 	// adjust hopCount to be lesser or equal to the original value
 	if (orig_lrq.GetRequest().HasOptionalField(H225_LocationRequest::e_hopCount)) {
@@ -508,7 +553,7 @@ bool Neighbor::OnSendingLRQ(H225_LocationRequest & lrq, const FacilityRequest &)
 
 bool Neighbor::CheckReply(RasMsg *ras) const
 {
-	if( ras->IsFrom(GetIP(), 0 /*m_port*/) )
+	if( ras->IsFrom(GetIP(), 0) )
 		return true;
 
 	const H225_NonStandardParameter *param = ras->GetNonStandardParam();
@@ -523,7 +568,7 @@ bool Neighbor::CheckReply(RasMsg *ras) const
 		if (oid.GetDataLength() == 0)
 			iec = Toolkit::iecNeighborId;
 	}
-	
+
 	return iec == Toolkit::iecNeighborId
 		? strncmp(m_id, param->m_data.AsString(), m_id.GetLength()) == 0
 		: false;
@@ -584,10 +629,10 @@ bool Neighbor::Authenticate(RasMsg *ras) const
 		return false;	// no token found that allows access
 	}
 }
- 
+
 bool Neighbor::IsAcceptable(RasMsg *ras) const
 {
-	if (ras->IsFrom(GetIP(), 0 /*m_port*/)) {
+	if (ras->IsFrom(GetIP(), 0)) {
 		// ras must be an LRQ
 		H225_LocationRequest & lrq = (*ras)->m_recvRAS;
 		PINDEX i, j, sz = m_acceptPrefixes.GetSize();
@@ -733,7 +778,7 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const SetupRequest & reques
 		lrq.IncludeOptionalField(H225_LocationRequest::e_sourceInfo);
 		lrq.m_sourceInfo = setup.m_sourceAddress;
 	}
-	
+
 	lrq.IncludeOptionalField(H225_LocationRequest::e_canMapAlias);
 	lrq.m_canMapAlias = TRUE;
 
@@ -757,7 +802,7 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const FacilityRequest & /*r
 	t35.m_manufacturerCode = Toolkit::t35mGnuGk;
 	t35.m_t35Extension = Toolkit::t35eNeighborId;
 	lrq.m_nonStandardData.m_data.SetValue(m_id);
-	
+
 	lrq.IncludeOptionalField(H225_LocationRequest::e_canMapAlias);
 	lrq.m_canMapAlias = TRUE;
 
@@ -782,7 +827,7 @@ bool CiscoGK::OnSendingLRQ(H225_LocationRequest &lrq, const AdmissionRequest &re
 {
 	const H225_AdmissionRequest &arq = req.GetRequest();
 	Cisco_LRQnonStandardInfo nonStandardData;
-	
+
 	nonStandardData.m_ttl = m_forwardHopCount >= 1 ? m_forwardHopCount : 5;
 
 	nonStandardData.IncludeOptionalField(Cisco_LRQnonStandardInfo::e_gatewaySrcInfo);
@@ -794,7 +839,7 @@ bool CiscoGK::OnSendingLRQ(H225_LocationRequest &lrq, const AdmissionRequest &re
 		nonStandardData.IncludeOptionalField(Cisco_LRQnonStandardInfo::e_callIdentifier);
 		nonStandardData.m_callIdentifier = arq.m_callIdentifier;
 	}
-		
+
 	// Cisco GK needs these
 	lrq.IncludeOptionalField(H225_LocationRequest::e_nonStandardData);
 	lrq.m_nonStandardData.m_nonStandardIdentifier.SetTag(H225_NonStandardIdentifier::e_h221NonStandard);
@@ -802,12 +847,12 @@ bool CiscoGK::OnSendingLRQ(H225_LocationRequest &lrq, const AdmissionRequest &re
 	h221.m_manufacturerCode = Toolkit::t35mCisco;
 	h221.m_t35CountryCode = Toolkit::t35cUSA;
 	h221.m_t35Extension = 0;
-	
+
 	PPER_Stream buff;
 	nonStandardData.Encode(buff);
 	buff.CompleteEncoding();
 	lrq.m_nonStandardData.m_data = buff;
-	
+
 	lrq.IncludeOptionalField(H225_LocationRequest::e_canMapAlias);
 	lrq.m_canMapAlias = TRUE;
 	return true;
@@ -821,10 +866,10 @@ bool CiscoGK::OnSendingLRQ(H225_LocationRequest &lrq, const LocationRequest & /*
 		const H225_H221NonStandard &h221 = lrq.m_nonStandardData.m_nonStandardIdentifier;
 		if (h221.m_manufacturerCode == Toolkit::t35mCisco && h221.m_t35CountryCode == Toolkit::t35cUSA)
 			return true;
-			
+
 		lrq.RemoveOptionalField(H225_LocationRequest::e_nonStandardData);
 	}
-	
+
 	Cisco_LRQnonStandardInfo nonStandardData;
 	nonStandardData.m_ttl = m_forwardHopCount >= 1 ? m_forwardHopCount : 5;
 
@@ -834,19 +879,19 @@ bool CiscoGK::OnSendingLRQ(H225_LocationRequest &lrq, const LocationRequest & /*
 		for (PINDEX i = 0; i < lrq.m_sourceInfo.GetSize(); i++)
 			nonStandardData.m_gatewaySrcInfo[i] = lrq.m_sourceInfo[i];
 	}
-	
+
 	lrq.IncludeOptionalField(H225_LocationRequest::e_nonStandardData);
 	lrq.m_nonStandardData.m_nonStandardIdentifier.SetTag(H225_NonStandardIdentifier::e_h221NonStandard);
 	H225_H221NonStandard & h221 = lrq.m_nonStandardData.m_nonStandardIdentifier;
 	h221.m_manufacturerCode = Toolkit::t35mCisco;
 	h221.m_t35CountryCode = Toolkit::t35cUSA;
 	h221.m_t35Extension = 0;
-	
+
 	PPER_Stream buff;
 	nonStandardData.Encode(buff);
 	buff.CompleteEncoding();
 	lrq.m_nonStandardData.m_data = buff;
-	
+
 	lrq.IncludeOptionalField(H225_LocationRequest::e_canMapAlias);
 	lrq.m_canMapAlias = TRUE;
 	return true;
@@ -858,7 +903,7 @@ bool CiscoGK::OnSendingLRQ(H225_LocationRequest &lrq, const SetupRequest &req)
 	const Q931 &setup = req.GetWrapper()->GetQ931();
 	const H225_Setup_UUIE &setupBody = req.GetRequest();
 	Cisco_LRQnonStandardInfo nonStandardData;
-	
+
 	nonStandardData.m_ttl = m_forwardHopCount >= 1 ? m_forwardHopCount : 5;
 
 	if (setupBody.HasOptionalField(H225_Setup_UUIE::e_sourceAddress)) {
@@ -871,7 +916,7 @@ bool CiscoGK::OnSendingLRQ(H225_LocationRequest &lrq, const SetupRequest &req)
 		nonStandardData.IncludeOptionalField(Cisco_LRQnonStandardInfo::e_callIdentifier);
 		nonStandardData.m_callIdentifier = setupBody.m_callIdentifier;
 	}
-	
+
 	if (setup.HasIE(Q931::CallingPartyNumberIE)) {
 		PBYTEArray data = setup.GetIE(Q931::CallingPartyNumberIE);
 		if ((data[0] & 0x80) == 0x80 && data.GetSize() >= 2) {
@@ -879,7 +924,7 @@ bool CiscoGK::OnSendingLRQ(H225_LocationRequest &lrq, const SetupRequest &req)
 			nonStandardData.m_callingOctet3a = data[1];
 		}
 	}
-		
+
 	// Cisco GK needs these
 	lrq.IncludeOptionalField(H225_LocationRequest::e_nonStandardData);
 	lrq.m_nonStandardData.m_nonStandardIdentifier.SetTag(H225_NonStandardIdentifier::e_h221NonStandard);
@@ -887,12 +932,12 @@ bool CiscoGK::OnSendingLRQ(H225_LocationRequest &lrq, const SetupRequest &req)
 	h221.m_manufacturerCode = Toolkit::t35mCisco;
 	h221.m_t35CountryCode = Toolkit::t35cUSA;
 	h221.m_t35Extension = 0;
-	
+
 	PPER_Stream buff;
 	nonStandardData.Encode(buff);
 	buff.CompleteEncoding();
 	lrq.m_nonStandardData.m_data = buff;
-	
+
 	lrq.IncludeOptionalField(H225_LocationRequest::e_canMapAlias);
 	lrq.m_canMapAlias = TRUE;
 	return true;
@@ -902,7 +947,7 @@ bool CiscoGK::CheckReply(RasMsg *msg) const
 {
 	if (msg->IsFrom(GetIP(), 0))
 		return true;
-		
+
 	if (msg->GetTag() != H225_RasMessage::e_locationConfirm
 			&& msg->GetTag() != H225_RasMessage::e_locationReject)
 		return false;
@@ -910,21 +955,22 @@ bool CiscoGK::CheckReply(RasMsg *msg) const
 	H225_NonStandardParameter *nonStandardData = msg->GetNonStandardParam();
 	if (nonStandardData == NULL)
 		return false;
-	
+
 	if (nonStandardData->m_nonStandardIdentifier.GetTag() != H225_NonStandardIdentifier::e_h221NonStandard)
 		return false;
-	
-	H225_H221NonStandard &h221 = nonStandardData->m_nonStandardIdentifier;
+
+	H225_H221NonStandard & h221 = nonStandardData->m_nonStandardIdentifier;
 	if (h221.m_manufacturerCode != Toolkit::t35mCisco || h221.m_t35CountryCode != Toolkit::t35cUSA)
 		return false;
-	
+
 	PPER_Stream strm(nonStandardData->m_data);
 	Cisco_LRQnonStandardInfo ciscoNonStandardData;
 	if (ciscoNonStandardData.Decode(strm)) {
 		// here should go additional checks to match callIdentifier, for example
-	} else
+	} else {
 		PTRACE(5, "NB\tFailed to decode Cisco nonStandardInfo field");
-		
+	}
+
 	return true;
 }
 
@@ -982,6 +1028,7 @@ public:
 	bool IsTraversalClient() const { return m_h46018_client; }
 	bool IsTraversalServer() const { return m_h46018_server; }
 	bool IsTraversalZone() const { return m_h46018_client || m_h46018_server; }
+	bool IsH46024Supported() const;
 
 	// override from class RasRequester
 	virtual bool IsExpected(const RasMsg *) const;
@@ -1025,9 +1072,11 @@ bool LRQRequester::Send(NeighborList::List & neighbors, Neighbor *requester)
 	NeighborList::List::iterator iter = neighbors.begin();
 	while (iter != neighbors.end()) {
 		Neighbor *nb = *iter++;
-		if (nb != requester)
-			if (PrefixInfo info = m_sendto(nb, m_seqNum))
+		if (nb != requester) {
+			if (PrefixInfo info = m_sendto(nb, m_seqNum)) {
 				m_requests.insert(make_pair(info, nb));
+			}
+		}
 	}
 	if (m_requests.empty())
 		return false;
@@ -1045,7 +1094,8 @@ bool LRQRequester::Send(Neighbor * nb)
 		   m_requests.insert(make_pair(info, nb));
 
 	if (m_requests.empty()) {
-		PTRACE(2, "SRV\tError Sending LRQ to " << nb->GetIP());
+		PTRACE(2, "SRV\tError sending LRQ to " << nb->GetIP());
+		SNMP_TRAP(11, SNMPError, Network, "Error sending LRQ to " + AsString(nb->GetIP()));
 		return false;
 	}
 
@@ -1067,6 +1117,20 @@ H225_LocationConfirm * LRQRequester::WaitForDestination(int timeout)
 	return m_result ? &(H225_LocationConfirm &)(*m_result)->m_recvRAS : NULL;
 }
 
+bool LRQRequester::IsH46024Supported() const
+{
+	if (!m_result)
+		return false;
+
+	const H225_LocationConfirm *lcf = &(H225_LocationConfirm &)(*m_result)->m_recvRAS;
+	if (lcf && lcf->HasOptionalField(H225_LocationConfirm::e_genericData)) {
+		H460_FeatureSet fs = H460_FeatureSet(lcf->m_genericData);
+		if (fs.HasFeature(24))
+			return true;
+	}
+	return false;
+}
+
 bool LRQRequester::IsExpected(const RasMsg *ras) const
 {
 	return RasHandler::IsExpected(ras) && (ras->GetSeqNum() == m_seqNum);
@@ -1209,11 +1273,10 @@ void NeighborList::OnReload()
 		if (PCaselessString(type) == "GlonetGK")
 			type = "GlonetGK";
 		iter = find_if(m_neighbors.begin(), m_neighbors.end(),
-				compose1(bind2nd(equal_to<PString>(), nbid), mem_fun(&Neighbor::GetId))
-			      );
+				compose1(bind2nd(equal_to<PString>(), nbid), mem_fun(&Neighbor::GetId)));
 		bool newnb = (iter == m_neighbors.end());
 		Neighbor *nb = newnb ? Factory<Neighbor>::Create(type) : *iter;
-		if (nb->SetProfile(nbid, type)) {
+		if (nb && nb->SetProfile(nbid, type)) {
 			if (newnb)
 				m_neighbors.push_back(nb);
 		} else {
@@ -1242,11 +1305,6 @@ bool NeighborList::CheckIP(const PIPSocket::Address & addr) const
 	return find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsFrom), &addr)) != m_neighbors.end();
 }
 
-bool NeighborList::IsTraversalZone(const PIPSocket::Address & addr) const
-{
-	return find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsTraversalZone), &addr)) != m_neighbors.end();
-}
-
 bool NeighborList::IsTraversalClient(const PIPSocket::Address & addr) const
 {
 	return find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsTraversalClient), &addr)) != m_neighbors.end();
@@ -1280,7 +1338,7 @@ PString NeighborList::GetNeighborIdBySigAdr(const PIPSocket::Address & sigAd)
 	}
 	return (*findNeighbor)->GetId();
 }
- 
+
 PString NeighborList::GetNeighborGkIdBySigAdr(const PIPSocket::Address & sigAd)
 {
 	// Attempt to find the neigbor in the list
@@ -1426,7 +1484,14 @@ bool NeighborPolicy::OnRequest(AdmissionRequest & arq_obj)
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
 #ifdef HAS_H460
-			if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData) || request.IsTraversalZone()) {
+			if ((request.IsTraversalZone() || request.IsH46024Supported()) && RasServer::Instance()->IsGKRouted()) {
+				// overwrite callSignalAddress and replace with ours, we must proxy this call
+				endptr ep = RegistrationTable::Instance()->FindByEndpointId(arq_obj.GetRequest().m_endpointIdentifier); // should not be null
+				if (ep) {
+					 PIPSocket::Address epip;
+					if (GetIPFromTransportAddr(ep->GetCallSignalAddress(), epip))
+						route.m_destAddr = RasServer::Instance()->GetCallSignalAddress(epip);
+				}
 				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
 				H225_RasMessage ras;
 				ras.SetTag(H225_RasMessage::e_locationConfirm);
@@ -1445,11 +1510,22 @@ bool NeighborPolicy::OnRequest(AdmissionRequest & arq_obj)
 #endif
 			route.m_routeId = request.GetNeighborUsed();
 			route.m_flags |= Route::e_toNeighbor;
+			// check if we have to update the dialed alias (canMapAlias)
 			if ((lcf->HasOptionalField(H225_LocationConfirm::e_destinationInfo))
 				&& (lcf->m_destinationInfo.GetSize() > 0)) 
 			{
+				// new alias from neighbor
 				arq_obj.SetAliases(lcf->m_destinationInfo);
 				arq_obj.SetFlag(Routing::AdmissionRequest::e_aliasesChanged);
+			} else {
+				if (arq_obj.GetAliases() == NULL && arq_obj.GetDestIP() != NULL) {
+					// call dialed by IP
+					H225_ArrayOf_AliasAddress newAliases;
+					newAliases.SetSize(1);
+					H323SetAliasAddress(AsDotString(*arq_obj.GetDestIP()), newAliases[0], H225_AliasAddress::e_transportID);
+					arq_obj.SetAliases(newAliases);
+					arq_obj.SetFlag(Routing::AdmissionRequest::e_aliasesChanged);			
+				}
 			}
 			arq_obj.AddRoute(route);
 			RasMsg *ras = arq_obj.GetWrapper();
@@ -1485,7 +1561,6 @@ bool NeighborPolicy::OnRequest(LocationRequest & lrq_obj)
 	if (!hopCount)
 		return false;
 
-
 	if (requester && !requester->ForwardResponse()) {
 		LRQForwarder functor(lrq_obj);
 		LRQRequester request(functor);
@@ -1570,7 +1645,7 @@ bool NeighborPolicy::OnRequest(SetupRequest & setup_obj)
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
 #ifdef HAS_H460
-			if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData) || request.IsTraversalZone()) {
+			if (request.IsTraversalZone() || request.IsH46024Supported()) {
 				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
 				H225_RasMessage ras;
 				ras.SetTag(H225_RasMessage::e_locationConfirm);
@@ -1583,7 +1658,7 @@ bool NeighborPolicy::OnRequest(SetupRequest & setup_obj)
 					route.m_destEndpoint->SetTraversalRole(TraversalServer);
 				}
 				if (request.IsTraversalServer()) {
-                    route.m_destEndpoint->SetTraversalRole(TraversalClient);
+					route.m_destEndpoint->SetTraversalRole(TraversalClient);
 				}
 			}
 #endif
@@ -1612,7 +1687,7 @@ bool NeighborPolicy::OnRequest(FacilityRequest & facility_obj)
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
 #ifdef HAS_H460
-			if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData) || request.IsTraversalZone()) {
+			if (request.IsTraversalZone() || request.IsH46024Supported()) {
 				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
 				H225_RasMessage ras;
 				ras.SetTag(H225_RasMessage::e_locationConfirm);
@@ -1625,7 +1700,7 @@ bool NeighborPolicy::OnRequest(FacilityRequest & facility_obj)
 					route.m_destEndpoint->SetTraversalRole(TraversalServer);
 				}
 				if (request.IsTraversalServer()) {
-                    route.m_destEndpoint->SetTraversalRole(TraversalClient);
+					route.m_destEndpoint->SetTraversalRole(TraversalClient);
 				}
 			}
 #endif
@@ -1651,20 +1726,37 @@ protected:
 	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
 
-	virtual Route * CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly);
-	virtual Route * LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases);
-	virtual Route * LSLocalLookup(H225_ArrayOf_AliasAddress & aliases);
+	virtual Route * CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly, const PString & schema, WORD schemaPort, const PString & gateway, PBoolean & changed);
+	virtual Route * LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases, bool localonly, const PString & schema);
+
+
+	virtual void LoadConfig(const PString & instance);
 
 	bool m_resolveNonLocalLRQs;
+    PStringToString m_ls_schema;
+    PStringToString m_cs_schema;
 };
 
 SRVPolicy::SRVPolicy()
 {
 	m_name = "SRV";
-	m_resolveNonLocalLRQs = Toolkit::AsBool(GkConfig()->GetString("Routing::SRV", "ResolveNonLocalLRQ", "0"));
+	m_iniSection = "Routing::SRV";
+	m_resolveNonLocalLRQs = Toolkit::AsBool(GkConfig()->GetString(m_iniSection, "ResolveNonLocalLRQ", "0"));
+	m_ls_schema.SetAt("_h323ls._udp.","");
+	m_cs_schema.SetAt("_h323cs._tcp.","");
 }
 
-Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
+void SRVPolicy::LoadConfig(const PString & instance)
+{
+	if (instance.IsEmpty())
+		return;
+
+	m_ls_schema.SetSize(0);
+	m_cs_schema.SetSize(0);
+	m_cs_schema = GkConfig()->GetAllKeyValues(m_iniSection);
+}
+
+Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases, bool localonly, const PString & schema)
 {
 	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
 		// only apply to urlID and h323ID
@@ -1687,12 +1779,13 @@ Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress
 
 		// LS Record lookup
 		PStringList ls;
-		if (PDNS::LookupSRV(number, "_h323ls._udp.", ls)) {
+		if (PDNS::LookupSRV(number, schema, ls)) {
 			for (PINDEX i=0; i < ls.GetSize(); i++) {
 				PINDEX at = ls[i].Find('@');
 				PString ipaddr = ls[i].Mid(at + 1);
 				if (ipaddr.Left(7) == "0.0.0.0") {
 					PTRACE(1, "ROUTING\tERROR in LS SRV lookup (" << ls[i] << ")");
+					SNMP_TRAP(11, SNMPError, Network, "SRV LS lookup failed: " + ls[i]);
 					continue;
 				}
 				PTRACE(4, "ROUTING\tSRV LS located domain " << domain << " at " << ipaddr);
@@ -1702,6 +1795,7 @@ Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress
 				WORD port;
 				if (!GetTransportAddress(ipaddr, GK_DEF_UNICAST_RAS_PORT, socketip, port) && socketip.IsValid()) {
 					PTRACE(1, "ROUTING\tERROR in SRV LS IP " << ipaddr);
+					SNMP_TRAP(11, SNMPError, Network, "SRV LS lookup failed: " + ipaddr);
 					continue;
 				}
 				if (Toolkit::Instance()->IsGKHome(socketip)) {
@@ -1719,11 +1813,12 @@ Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress
 					} else {
 						return NULL;
 					}
-				} else {
+				} else if (!localonly) {
 					// Create a SRV gatekeeper object
 					GnuGK * nb = new GnuGK();
-					if (!nb->SetProfile(domain,addr)) {
-						PTRACE(4, "ROUTING\tERROR setting SRV neighbor profile " << domain << " at " << addr);
+					if (!nb->SetProfile(domain, addr)) {
+						PTRACE(2, "ROUTING\tERROR setting SRV neighbor profile " << domain << " at " << addr);
+						SNMP_TRAP(11, SNMPError, Configuration, "Error setting SRV neighbor profile " + domain + " at " + addr);
 						delete nb;
 						return NULL;
 					}
@@ -1731,102 +1826,55 @@ Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress
 					int m_neighborTimeout = GkConfig()->GetInteger(LRQFeaturesSection, "NeighborTimeout", 5) * 100;
 
 					// Send LRQ to retreive callers signaling address
-					LRQSender<AdmissionRequest> functor((AdmissionRequest &)request);
-					LRQRequester Request(functor);
-					if (Request.Send(nb)) {
-						if (H225_LocationConfirm *lcf = Request.WaitForDestination(m_neighborTimeout)) {
+					// Caution: we may only use the functor object of the right type and never touch the others!
+					LRQSender<AdmissionRequest> ArqFunctor((AdmissionRequest &)request);
+					LRQSender<SetupRequest> SetupFunctor((SetupRequest &)request);
+					LRQSender<FacilityRequest> FacilityFunctor((FacilityRequest &)request);
+					LRQSender<LocationRequest> LrqFunctor((LocationRequest &)request);
+					LRQRequester * pRequest = NULL;
+					if (dynamic_cast<AdmissionRequest *>(&request)) {
+						pRequest = new LRQRequester(ArqFunctor);
+					} else if (dynamic_cast<SetupRequest *>(&request)) {
+						pRequest = new LRQRequester(SetupFunctor);
+					} else if (dynamic_cast<FacilityRequest *>(&request)) {
+						pRequest = new LRQRequester(FacilityFunctor);
+					} else if (dynamic_cast<LocationRequest *>(&request)) {
+						pRequest = new LRQRequester(LrqFunctor);
+					}
+					if (pRequest && pRequest->Send(nb)) {
+						if (H225_LocationConfirm * lcf = pRequest->WaitForDestination(m_neighborTimeout)) {
 							Route * route = new Route(m_name, lcf->m_callSignalAddress);
 #ifdef HAS_H460
-							if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData)) {
+							if (pRequest->IsH46024Supported()) {
 								H225_RasMessage ras;
 								ras.SetTag(H225_RasMessage::e_locationConfirm);
-								H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
-								con = *lcf;
+								H225_LocationConfirm & conf = (H225_LocationConfirm &)ras;
+								conf = *lcf;
 								route->m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
 							}
 #endif
+							delete pRequest;
 							delete nb;
 							return route;
 						}
 					}
+					delete pRequest;
 					delete nb;
 					PTRACE(4, "ROUTING\tDNS SRV LRQ Error for " << domain << " at " << ipaddr);
 					// we found the directory for this domain, but it didn't have a destination, so we fail the call
 					Route * route = new Route();
 					route->m_flags |= Route::e_Reject;
 					return route;
-				}
-			}
-		}
-	}
-	return NULL;
-}
- 
-Route * SRVPolicy::LSLocalLookup(H225_ArrayOf_AliasAddress & aliases)
-{
-	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
-		// only apply to urlID and h323ID
-		if ((aliases[i].GetTag() != H225_AliasAddress::e_url_ID)
-			&& (aliases[i].GetTag() != H225_AliasAddress::e_h323_ID))
-			continue;
-		PString alias(AsString(aliases[i], FALSE));
-		PINDEX at = alias.Find('@');
-		// skip empty aliases or those without at-sign
-	    if ((alias.GetLength() == 0) || (at == P_MAX_INDEX))
-			continue;
- 
-		// DNS SRV Record lookup
-		PString number = "h323:" + alias;
-		PString domain = alias.Mid(at+1);
-		if (IsIPAddress(domain)
-			|| (domain.FindRegEx(PRegularExpression(":[0-9]+$", PRegularExpression::Extended)) != P_MAX_INDEX))
-			continue;	// don't use SRV record if domain part is IP or has port (Annex O, O.9), let dns policy handle them
- 
-		// LS Record lookup
-		PStringList ls;
-		if (PDNS::LookupSRV(number, "_h323ls._udp.", ls)) {
-			for (PINDEX i=0; i < ls.GetSize(); i++) {
-				PINDEX at = ls[i].Find('@');
-				PString ipaddr = ls[i].Mid(at + 1);
-				if (ipaddr.Left(7) == "0.0.0.0") {
-					PTRACE(1, "ROUTING\tERROR in LS SRV lookup (" << ls[i] << ")");
-					continue;
-				}
-				PTRACE(4, "ROUTING\tSRV LS located domain " << domain << " at " << ipaddr);
-				H323TransportAddress addr = H323TransportAddress(ipaddr);
- 
-				PIPSocket::Address socketip;
-				WORD port;
-				if (!GetTransportAddress(ipaddr, GK_DEF_UNICAST_RAS_PORT, socketip, port) && socketip.IsValid()) {
-					PTRACE(1, "ROUTING\tERROR in SRV LS IP " << ipaddr);
-					continue;
-				}
-				if (Toolkit::Instance()->IsGKHome(socketip)) {
-					// this is my domain, no need to send LRQs, just look into the endpoint table
-					PINDEX numberat = number.Find('@');	// always has an @
-					H225_ArrayOf_AliasAddress find_aliases;
-					find_aliases.SetSize(1);
-					PString local_alias = number.Mid(5,numberat-5);
-					H323SetAliasAddress(local_alias, find_aliases[0]);
-					endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
-					if (ep) {
-						// endpoint found locally
-						Route * route = new Route("srv", ep);
-						return route;
-					}
-				} else if (m_resolveNonLocalLRQs) {
-					// just rewrite the destination based on the SRV record and let the following policies handle it
-					H323SetAliasAddress(number, aliases[0]);
 				} else {
-					PTRACE(3, "ROUTING\tSkipped, using only local LS for LRQs");
+					PTRACE(4, "ROUTING\tDNS SRV no applied to LRQ");
 				}
 			}
 		}
 	}
 	return NULL;
 }
- 
-Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly)
+
+Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly, const PString & schema, WORD schemaPort, const PString & gateway, PBoolean & changed)
 {
 	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
 		// only apply to urlID and h323ID
@@ -1838,25 +1886,31 @@ Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly)
 		// skip empty aliases or those without at-sign
 	    if ((alias.GetLength() == 0) || (at == P_MAX_INDEX))
 			continue;
- 
+
 		// DNS SRV Record lookup
 		PString number = "h323:" + alias;
 		PString domain = alias.Mid(at+1);
 		if (IsIPAddress(domain)
 			|| (domain.FindRegEx(PRegularExpression(":[0-9]+$", PRegularExpression::Extended)) != P_MAX_INDEX))
-			continue;	// don't use SRV record if domain part is IP or has port (Annex O, O.9), let dns policy handle them
+				continue;	// don't use SRV record if domain part is IP or has port (Annex O, O.9), let dns policy handle them
 
 		// CS SRV Lookup
 		PStringList cs;
-		if (PDNS::LookupSRV(number, "_h323cs._tcp.", cs)) {
+		if (PDNS::LookupSRV(number, schema, cs)) {
 			for (PINDEX j = 0; j < cs.GetSize(); j++) {
 				H225_TransportAddress dest;
 				PINDEX in = cs[j].Find('@');
 				PString dom = cs[j].Mid(in+1);
 				if (dom.Left(7) == "0.0.0.0") {
 					PTRACE(1, "ROUTING\tERROR in CS SRV lookup (" << cs[j] << ")");
+					SNMP_TRAP(11, SNMPError, Network, "SRV CS lookup failed: " + cs[i]);
 					continue;
 				}
+
+				// If we have a gateway destination replace destination with our destination.
+				if (!gateway) 
+					dom = gateway;
+
 				PStringArray parts = SplitIPAndPort(dom, GK_DEF_ENDPOINT_SIGNAL_PORT);
 				dom = parts[0];
 				WORD port = (WORD)parts[1].AsUnsigned();
@@ -1865,6 +1919,15 @@ Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly)
 					PIPSocket::Address addr;
 					if (!(GetIPFromTransportAddr(dest, addr) && addr.IsValid()))
 						continue;
+
+					if (!gateway) {  // If we have a gateway destination send full URI
+						PStringArray parts = SplitIPAndPort(cs[j].Mid(in+1), schemaPort);
+						PString finalDest =  parts[0] + ":" + parts[1];
+						if (in > 0) finalDest = cs[j].Left(in) + "@" + finalDest;
+						H323SetAliasAddress(finalDest, aliases[i]);
+						changed = true;
+					} else
+						H323SetAliasAddress(cs[j].Left(in), aliases[i]);
 					Route * route = NULL;
 					if (Toolkit::Instance()->IsGKHome(addr)) {
 						H225_ArrayOf_AliasAddress find_aliases;
@@ -1891,21 +1954,41 @@ Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly)
 	}
 	return NULL;
 }
- 
+
 // used for ARQs and Setups
 bool SRVPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
-	Route * route = CSLookup(aliases, false);
-	if (route) {
-		if (route->m_flags & Route::e_Reject) {
-			request.SetFlag(RoutingRequest::e_Reject);
-		} else {
-			request.AddRoute(*route);
+	bool localonly = dynamic_cast<LocationRequest *>(&request) && !m_resolveNonLocalLRQs;
+
+	Route * route = NULL;
+	for (PINDEX i = 0; i < m_cs_schema.GetSize(); ++i) {
+		PString cs_schema = m_cs_schema.GetKeyAt(i);
+		PStringArray dest = m_cs_schema.GetDataAt(i).Tokenise(",;");
+		PString gwDestination = dest[0];
+		WORD defSchemaPort = GK_DEF_ENDPOINT_SIGNAL_PORT;
+		if (dest.GetSize() > 1)
+			defSchemaPort = (WORD)dest[1].AsInteger();
+
+		PBoolean changed = false;
+		route = CSLookup(aliases, localonly, cs_schema, defSchemaPort, gwDestination, changed);
+		if (route) {
+			if (route->m_flags & Route::e_Reject) {
+				request.SetFlag(RoutingRequest::e_Reject);
+			} else {
+				if (changed)
+					request.SetFlag(RoutingRequest::e_aliasesChanged);
+				request.AddRoute(*route);
+			}
+			delete route;
+			return true;
 		}
-		delete route;
-		return true;
 	}
-	route = LSLookup(request, aliases);
+
+	if (m_ls_schema.GetSize() == 0)
+		return false;
+
+	PString ls_schema = m_cs_schema.GetKeyAt(0);
+	route = LSLookup(request, aliases, localonly, ls_schema);
 	if (route) {
 		if (route->m_flags & Route::e_Reject) {
 			request.SetFlag(RoutingRequest::e_Reject);
@@ -1920,28 +2003,8 @@ bool SRVPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 
 // used for LRQs
 bool SRVPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddress & aliases)
-{ 
-	Route * route = CSLookup(aliases, !m_resolveNonLocalLRQs);
-	if (route) {
-		if (route->m_flags & Route::e_Reject) {
-			request.SetFlag(RoutingRequest::e_Reject);
-		} else {
-			request.AddRoute(*route);
-		}
-		delete route;
-		return true;
-	}
-	route = LSLocalLookup(aliases);
-	if (route) {
-		if (route->m_flags & Route::e_Reject) {
-			request.SetFlag(RoutingRequest::e_Reject);
-		} else {
-			request.AddRoute(*route);
-		}
-		delete route;
-		return true;
-	}
-	return false;
+{
+	return SRVPolicy::FindByAliases((RoutingRequest&)request, aliases);
 }
 #endif
 
@@ -1958,15 +2021,23 @@ protected:
 	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
 
+	virtual void LoadConfig(const PString & instance);
+
 	bool m_resolveLRQs;
 };
 
 RDSPolicy::RDSPolicy()
 {
 	m_name = "RDS";
+	m_iniSection = "Routing::RDS";
 	m_resolveLRQs = Toolkit::AsBool(GkConfig()->GetString("Routing::RDS", "ResolveLRQ", "0"));
 }
 
+void RDSPolicy::LoadConfig(const PString & instance) 
+{
+	m_resolveLRQs = Toolkit::AsBool(GkConfig()->GetString(m_iniSection, "ResolveLRQ", m_resolveLRQs));
+}
+
 bool RDSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
 	for (PINDEX a = 0; a < aliases.GetSize(); ++a) {
@@ -1998,20 +2069,32 @@ bool RDSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 				// Create a RDS gatekeeper object
 				GnuGK * nb = new GnuGK();
 				if (!nb->SetProfile(domain,addr)) {
-					PTRACE(4, "ROUTING\tERROR setting RDS neighbor profile " << domain << " at " << addr);
+					PTRACE(2, "ROUTING\tERROR setting RDS neighbor profile " << domain << " at " << addr);
+					SNMP_TRAP(11, SNMPError, Configuration, "Error setting RDS neighbor profile " + domain + " at " + addr);
+					delete nb;
 					return false;
 				}
 
 				int m_neighborTimeout = GkConfig()->GetInteger(LRQFeaturesSection, "NeighborTimeout", 5) * 100;
 
-				// Send LRQ to retreive callers signaling address 
-				LRQSender<AdmissionRequest> functor((AdmissionRequest &)request);
-				LRQRequester Request(functor);
-				if (Request.Send(nb)) {
-					if (H225_LocationConfirm *lcf = Request.WaitForDestination(m_neighborTimeout)) {
+				// Send LRQ to retreive callers signaling address
+				// Caution: we may only use the functor object of the right type and never touch the others!
+				LRQSender<AdmissionRequest> ArqFunctor((AdmissionRequest &)request);
+				LRQSender<SetupRequest> SetupFunctor((SetupRequest &)request);
+				LRQSender<FacilityRequest> FacilityFunctor((FacilityRequest &)request);
+				LRQRequester * pRequest = NULL;
+				if (dynamic_cast<AdmissionRequest *>(&request)) {
+					pRequest = new LRQRequester(ArqFunctor);
+				} else if (dynamic_cast<SetupRequest *>(&request)) {
+					pRequest = new LRQRequester(SetupFunctor);
+				} else if (dynamic_cast<FacilityRequest *>(&request)) {
+					pRequest = new LRQRequester(FacilityFunctor);
+				}
+				if (pRequest && pRequest->Send(nb)) {
+					if (H225_LocationConfirm *lcf = pRequest->WaitForDestination(m_neighborTimeout)) {
 						Route route(m_name, lcf->m_callSignalAddress);
 #ifdef HAS_H460
-						if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData)) {
+						if (pRequest->IsH46024Supported()) {
 							H225_RasMessage ras;
 							ras.SetTag(H225_RasMessage::e_locationConfirm);
 							H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
@@ -2021,10 +2104,14 @@ bool RDSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 #endif
 						request.AddRoute(route);
 						request.SetFlag(RoutingRequest::e_aliasesChanged);
+						delete pRequest;
+						delete nb;
 						return true;
 					}
 				}
 				PTRACE(4, "ROUTING\tDNS RDS LRQ Error for " << domain << " at " << ipaddr);
+				delete pRequest;
+				delete nb;
 			}
 		}
 	}
