@@ -3,7 +3,7 @@
 // ProxyChannel.h
 //
 // Copyright (c) Citron Network Inc. 2001-2003
-// Copyright (c) 2002-2011, Jan Willamowius
+// Copyright (c) 2002-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +13,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef PROXYCHANNEL_H
-#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.93 2011/11/02 16:06:50 willamowius Exp $"
+#define PROXYCHANNEL_H "@(#) $Id: ProxyChannel.h,v 1.128.2.2 2013/01/16 11:05:50 willamowius Exp $"
 
 #include <vector>
 #include <list>
@@ -56,6 +56,8 @@ struct SetupAuthData;
 extern const char *RoutedSec;
 extern const char *ProxySection;
 
+const WORD DEFAULT_PACKET_BUFFER_SIZE = 2048;
+
 void PrintQ931(int, const char *, const char *, const Q931 *, const H225_H323_UserInformation *);
 
 bool GetUUIE(const Q931 & q931, H225_H323_UserInformation & uuie);
@@ -76,7 +78,7 @@ public:
 	ProxySocket(
 		IPSocket *self,
 		const char *type,
-		WORD buffSize = 1536
+		WORD buffSize = DEFAULT_PACKET_BUFFER_SIZE
 	);
 	virtual ~ProxySocket() = 0; // abstract class
 
@@ -90,8 +92,8 @@ public:
 	void SetConnected(bool c) { connected = c; }
 	bool IsDeletable() const { return deletable; }
 	void SetDeletable() { deletable = true; }
-	ProxyHandler *GetHandler() const { return handler; }
-	void SetHandler(ProxyHandler *h) { handler = h; }
+	ProxyHandler * GetHandler() const { return handler; }
+	void SetHandler(ProxyHandler * h) { handler = h; }
 
 private:
 	ProxySocket();
@@ -124,9 +126,11 @@ public:
 	virtual bool ForwardData();
 	virtual bool TransmitData(const PBYTEArray &);
 
-	void RemoveRemoteSocket();
+	void DetachRemote();
+  	void RemoveRemoteSocket();
 	void SetRemoteSocket(TCPProxySocket * ret) { remote=ret; }
-	//TCPProxySocket * GetRemoteSocket() const { return remote; }
+	void LockRemote() { m_remoteLock.Wait(); }
+	void UnlockRemote() { m_remoteLock.Signal(); }
 
 private:
 	TCPProxySocket();
@@ -135,7 +139,6 @@ private:
 
 protected:
 	struct TPKTV3 {
-		TPKTV3() {}
 		TPKTV3(WORD);
 
 		BYTE header, padding;
@@ -144,6 +147,7 @@ protected:
 
 	bool ReadTPKT();
 
+	PMutex m_remoteLock;	// protect the remote member
 	TCPProxySocket *remote;
 	PBYTEArray buffer;
 
@@ -156,6 +160,99 @@ private:
 	unsigned tpktlen;
 };
 
+class RTPLogicalChannel;
+class UDPProxySocket : public UDPSocket, public ProxySocket {
+public:
+#ifndef LARGE_FDSET
+	PCLASSINFO( UDPProxySocket, UDPSocket )
+#endif
+
+	UDPProxySocket(const char * t, const H225_CallIdentifier & id);
+	~UDPProxySocket();
+
+	void UpdateSocketName();
+	void RemoveCallPtr() { m_call = NULL; }
+	void SetDestination(H245_UnicastAddress &, callptr &);
+	void SetForwardDestination(const Address &, WORD, H245_UnicastAddress *, callptr &);
+	void SetReverseDestination(const Address &, WORD, H245_UnicastAddress *, callptr &);
+	typedef void (UDPProxySocket::*pMem)(const Address &, WORD, H245_UnicastAddress *, callptr &);
+
+	bool Bind(const Address &localAddr, WORD pt);
+	int GetOSSocket() const { return os_handle; }
+	void SetNAT(bool);
+	bool isMute() { return mute; }
+	void SetMute(bool toMute) { mute = toMute; }
+	void OnHandlerSwapped() { std::swap(fnat, rnat); }
+	void SetRTPSessionID(WORD id) { m_sessionID = id; }
+#ifdef HAS_H235_MEDIA
+	void SetEncryptingRTPChannel(RTPLogicalChannel * lc) { m_encryptingLC = lc; }
+	void RemoveEncryptingRTPChannel(RTPLogicalChannel * lc) { if (m_encryptingLC == lc) m_encryptingLC = NULL; }
+	void SetDecryptingRTPChannel(RTPLogicalChannel * lc) { m_decryptingLC = lc; }
+	void RemoveDecryptingRTPChannel(RTPLogicalChannel * lc) { if (m_decryptingLC == lc) m_decryptingLC = NULL; }
+#endif
+#ifdef HAS_H46018
+	void SetUsesH46019fc(bool fc) { m_h46019fc = fc; }
+	// same socket is used for all directions; set if at least one side uses H.460.19
+	void SetUsesH46019() { m_useH46019 = true; }
+	bool UsesH46019() const { return m_useH46019; }
+	void SetH46019UniDirectional(bool val) { m_h46019uni = val; }
+	void SetMultiplexDestination(const H323TransportAddress & toAddress, H46019Side side);
+	void SetMultiplexID(PUInt32b multiplexID, H46019Side side);
+	void SetMultiplexSocket(int multiplexSocket, H46019Side side);
+#endif
+
+	// override from class ProxySocket
+	virtual Result ReceiveData();
+	virtual bool OnReceiveData(void *, PINDEX, Address &, WORD &) { return true; }
+
+protected:
+	virtual bool WriteData(const BYTE *, int);
+	virtual bool Flush();
+	virtual bool ErrorHandler(PSocket::ErrorGroup);
+
+	void SetMediaIP(const PString & direction, const Address & ip);
+
+	// RTCP handler
+	void BuildReceiverReport(const RTP_ControlFrame & frame, PINDEX offset, bool dst);
+
+	H225_CallIdentifier m_callID;
+	callptr * m_call;
+
+private:
+	UDPProxySocket();
+	UDPProxySocket(const UDPProxySocket&);
+	UDPProxySocket& operator=(const UDPProxySocket&);
+
+protected:
+	Address fSrcIP, fDestIP, rSrcIP, rDestIP;
+	WORD fSrcPort, fDestPort, rSrcPort, rDestPort;
+	bool fnat, rnat;
+	bool mute;
+	bool m_isRTPType;
+	bool m_isRTCPType;
+	bool m_dontQueueRTP;
+	bool m_EnableRTCPStats;
+	WORD m_sessionID;
+	RTPLogicalChannel * m_encryptingLC;
+	RTPLogicalChannel * m_decryptingLC;
+#ifdef HAS_H46018
+	bool m_h46019fc;
+	bool m_useH46019;
+	bool m_h46019uni;
+	bool m_h46019DetectionDone;
+	PMutex m_h46019DetectionLock;
+	// two (!), one or zero parties in a call through a UDPProxySocket may by multiplexed
+	// UDPProxySocket always receives regular RTP, but may send out multiplexed
+	H323TransportAddress m_multiplexDestination_A;	// OLC side of first logical channel in this session
+	PUInt32b m_multiplexID_A;	// ID _to_ A side (only valid if m_multiplexDestination_A is set)
+	int m_multiplexSocket_A;	// only valid if m_multiplexDestination_A is set
+	H323TransportAddress m_multiplexDestination_B;	// OLCAck side of first logical channel in this session
+	PUInt32b m_multiplexID_B;	// ID _to_ B side (only valid if m_multiplexDestination_B is set)
+	int m_multiplexSocket_B;	// only valid if m_multiplexDestination_ is set)
+	PMutex m_multiplexMutex;	// protect multiplex IDs, addresses and sockets against access from concurrent threads
+#endif
+};
+
 #if H323_H450
 class X880_Invoke;
 class H4501_InterpretationApdu;
@@ -181,9 +278,12 @@ public:
 	virtual bool EndSession();
 	virtual void OnError();
 
-	void SendReleaseComplete(const H225_CallTerminationCause * = 0);
+	void SendReleaseComplete(const H225_CallTerminationCause * = NULL);
 	void SendReleaseComplete(H225_ReleaseCompleteReason::Choices);
 
+	bool IsH245Tunneling() const { return m_h245Tunneling; }
+	bool IsH245TunnelingTranslation() const { return m_h245TunnelingTranslation; }
+	PASN_OctetString * GetNextQueuedH245Message();
 	bool HandleH245Mesg(PPER_Stream &, bool & suppress, H245Socket * h245sock = NULL);
 	void OnH245ChannelClosed() { m_h245socket = NULL; }
 	Address GetLocalAddr() { return localAddr; }
@@ -209,33 +309,52 @@ public:
 	void RemoveH245Handler();
 	void SaveTCS(const H245_TerminalCapabilitySet & tcs) { m_savedTCS = tcs; }
 	H245_TerminalCapabilitySet GetSavedTCS() const { return m_savedTCS; }
+	bool SendTunneledH245(const H245_MultimediaSystemControlMessage & h245msg);
+	bool SendTunneledH245(const PPER_Stream & strm);
 #ifdef HAS_H235_MEDIA
     bool HandleH235TCS(H245_TerminalCapabilitySet & tcs);
     bool HandleH235OLC(H245_OpenLogicalChannel & olc);
 #endif
-	bool CompareH245Socket(H245Socket * sock) const { return sock == m_h245socket; }	// compare pointers !
+	H245Socket * GetH245Socket() const { return m_h245socket; }
+	void SetH245Socket(H245Socket * sock) { m_h245socket = sock; }
+	bool CompareH245Socket(H245Socket * sock) const { return sock == m_h245socket; }	// intentionally comparing pointers
 	
 protected:
-	CallSignalSocket(CallSignalSocket *);
-	
 	void SetRemote(CallSignalSocket *);
 	bool CreateRemote(H225_Setup_UUIE &setupBody);
+
 public:
+#ifdef HAS_H46017
+	bool SendH46017Message(const H225_RasMessage & ras);
+	void CleanupCall();
+#endif
+	bool MaintainConnection() const { return m_maintainConnection; }
+
 #ifdef HAS_H46018
+	bool IsCaller() const { return m_callerSocket; }
+	bool IsTraversalClient() const;
+	bool IsTraversalServer() const;
 	bool CreateRemote(const H225_TransportAddress & addr);
 	bool OnSCICall(H225_CallIdentifier callID, H225_TransportAddress sigAdr);
 	bool IsCallFromTraversalServer() const { return m_callFromTraversalServer; }
 	bool IsCallToTraversalServer() const { return m_callToTraversalServer; }
 	void SetSessionMultiplexDestination(WORD session, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
-	void SetLCMultiplexDestination(unsigned lc, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
-	void SetLCMultiplexID(unsigned lc, bool isRTCP, PUInt32b multiplexID, H46019Side side);
-	void SetLCMultiplexSocket(unsigned lc, bool isRTCP, int multiplexSocket, H46019Side side);
 	const H245Handler * GetH245Handler() const { return m_h245handler; }
 #endif
+	void LockH245Handler() { m_h245handlerLock.Wait(); }
+	void UnlockH245Handler() { m_h245handlerLock.Signal(); }
+
+#ifdef HAS_H46023
+	bool IsH46024Call(const H225_Setup_UUIE & setupBody);
+#endif
+
+//#ifdef HAS_H46026
+//	bool IsH46026Call(const H225_Setup_UUIE & setupBody);
+//#endif
+
+	CallSignalSocket * GetRemote() const { return dynamic_cast<CallSignalSocket *>(remote); }
 
 protected:
-	CallSignalSocket * GetRemote() const { return (CallSignalSocket *)remote; }
-
 	void ForwardCall(FacilityMsg *msg);
 
 	/// signaling message handlers
@@ -263,7 +382,7 @@ protected:
 			if (m_call)
 				m_call->SetH245ResponseReceived();
 			if (SetH245Address(uu.m_h245Address))
-				return (m_h245handler != 0);
+				return (m_h245handler != NULL);
 			uu.RemoveOptionalField(UUIE::e_h245Address);
 			return true;
 		}
@@ -284,7 +403,7 @@ protected:
 private:
 	CallSignalSocket(const CallSignalSocket&);
 	CallSignalSocket& operator=(const CallSignalSocket&);
-	
+
 	void InternalInit();
 	void BuildReleasePDU(Q931 &, const H225_CallTerminationCause *) const;
 	// if return false, the h245Address field will be removed
@@ -330,13 +449,20 @@ protected:
 
 private:
 	WORD m_crv;
-	H245Handler *m_h245handler;
-	H245Socket *m_h245socket;
+	PMutex m_h245handlerLock;
+	H245Handler * m_h245handler;
+	H245Socket * m_h245socket;
 	bool m_h245Tunneling;
+	bool m_h245TunnelingTranslation;
+	std::queue<PASN_OctetString> m_h245Queue;
 	bool m_isnatsocket;
+	bool m_maintainConnection;	// eg. for H.460.17
 	Result m_result;
 	/// stored for use by ForwardCall, NULL if ForwardOnFacility is disabled
-	Q931 *m_setupPdu;
+	Q931 * m_setupPdu;
+#ifdef HAS_H235_MEDIA
+	H225_ArrayOf_ClearToken * m_setupClearTokens;
+#endif
 	/// true if the socket is connected to the caller, false if to the callee
 	bool m_callerSocket;
 	/// H.225.0 protocol version in use by the remote party
@@ -345,11 +471,17 @@ private:
 	PBYTEArray m_rawSetup;
 	PMutex infomutex;    // Information PDU processing Mutex
 	H245_TerminalCapabilitySet m_savedTCS;	// saved tcs to re-send
+#ifdef HAS_H46017
+	bool m_h46017Enabled;
+#endif
 #ifdef HAS_H46018
 	bool m_callFromTraversalServer; // is this call from a traversal server ?
 	bool m_callToTraversalServer;
 	bool m_senderSupportsH46019Multiplexing;
 #endif
+#ifdef HAS_H235_MEDIA
+	bool m_isH245Master;
+#endif
 };
 
 class CallSignalListener : public TCPListenSocket {
@@ -374,7 +506,7 @@ class MultiplexRTPListener : public UDPSocket {
 	PCLASSINFO ( MultiplexRTPListener, UDPSocket )
 #endif
 public:
-	MultiplexRTPListener(WORD pt, WORD buffSize = 1536);
+	MultiplexRTPListener(WORD pt, WORD buffSize = DEFAULT_PACKET_BUFFER_SIZE);
 	virtual ~MultiplexRTPListener();
 
 	virtual int GetOSSocket() const { return os_handle; }
@@ -382,30 +514,37 @@ public:
 	virtual void ReceiveData();
 protected:
 	BYTE * wbuffer;
-	WORD wbufsize, buflen;
+	WORD wbufsize;
 };
 
-class H46019Channel
+class RTPLogicalChannel;
+
+// class for a H.460.19 session: it includes both directions (the full RTP session)
+// when stored in the channel list in MultiplexedRTPHandler,
+// side A is the OLC side, side B is the OLCAck side of the first channel in the session
+// when the channel is handed out via GetChannelSwapped(), side A is the OLC side of _that_ channel
+class H46019Session
 {
 public:
-	H46019Channel(const H225_CallIdentifier & callid, WORD session, void * openedBy);
+	H46019Session(const H225_CallIdentifier & callid, WORD session, void * openedBy);
 
 	void Dump() const;
 
+	bool IsValid() const { return m_session != INVALID_RTP_SESSION; }
 	bool sideAReady(bool isRTCP) const { return isRTCP ? IsSet(m_addrA_RTCP) : IsSet(m_addrA); }
 	bool sideBReady(bool isRTCP) const { return isRTCP ? IsSet(m_addrB_RTCP) : IsSet(m_addrB); }
-	H46019Channel SwapSides() const; // return a copy with side A and B swapped
+	H46019Session SwapSides() const; // return a copy with side A and B swapped
 
 	bool IsKeepAlive(void * data, unsigned len, bool isRTCP) { return isRTCP ? true : (len == 12); };
 
 	void HandlePacket(PUInt32b receivedMultiplexID, const H323TransportAddress & fromAddress, void * data, unsigned len, bool isRTCP);
-	static void Send(PUInt32b sendMultiplexID, const H323TransportAddress & toAddress, int ossocket, void * data, unsigned len);
+	static void Send(PUInt32b sendMultiplexID, const H323TransportAddress & toAddress, int ossocket, void * data, unsigned len, bool bufferHasRoomForID=false);
 
 //protected:
 	H225_CallIdentifier m_callid;
 	WORD m_session;
-	void * m_openedBy;	// pointer to H245ProxyHandler used as an ID
-	void * m_otherSide;	// pointer to H245ProxyHandler used as an ID
+	void * m_openedBy;	// side A (pointer to H245ProxyHandler used as an ID)
+	void * m_otherSide;	// side B (pointer to H245ProxyHandler used as an ID)
 	H323TransportAddress m_addrA;
 	H323TransportAddress m_addrA_RTCP;
 	H323TransportAddress m_addrB;
@@ -419,6 +558,12 @@ public:
 	int m_osSocketToB;
 	int m_osSocketToB_RTCP;
 	bool m_EnableRTCPStats;
+#ifdef HAS_H235_MEDIA
+	RTPLogicalChannel * m_encryptingLC;
+	RTPLogicalChannel * m_decryptingLC;
+	PUInt32b m_encryptMultiplexID;
+	PUInt32b m_decryptMultiplexID;
+#endif
 };
 
 class MultiplexedRTPReader : public SocketsReader {
@@ -445,10 +590,14 @@ public:
 
 	virtual void OnReload() { /* currently not runtime changable */ }
 
-	virtual void AddChannel(const H46019Channel & cha);
-	virtual void UpdateChannel(const H46019Channel & cha);
-	virtual H46019Channel GetChannel(const H225_CallIdentifier & callid, WORD session, void * openedBy) const;
-	virtual void RemoveChannels(const H225_CallIdentifier & callid);
+	virtual void AddChannel(const H46019Session & cha);
+	virtual void UpdateChannel(const H46019Session & cha);
+	virtual H46019Session GetChannel(const H225_CallIdentifier & callid, WORD session) const;
+	virtual H46019Session GetChannelSwapped(const H225_CallIdentifier & callid, WORD session, void * openedBy) const;
+	virtual void RemoveChannels(H225_CallIdentifier callid);	// pass by value in case call gets removed
+#ifdef HAS_H235_MEDIA
+	virtual void RemoveChannel(H225_CallIdentifier callid, RTPLogicalChannel * rtplc);
+#endif
 	virtual void DumpChannels(const PString & msg = "") const;
 
 	virtual void HandlePacket(PUInt32b receivedMultiplexID, const H323TransportAddress & fromAddress, void * data, unsigned len, bool isRTCP);
@@ -462,7 +611,7 @@ public:
 protected:
 	MultiplexedRTPReader * m_reader;
 	mutable PReadWriteMutex listLock;
-	list<H46019Channel> m_h46019channels;
+	list<H46019Session> m_h46019channels;
 	PUInt32b idCounter; // we should make sure this counter is _not_ reset on reload
 };
 #endif
@@ -480,7 +629,7 @@ public:
 	void LoadConfig();
 	bool Detach(TCPProxySocket *);
 	void Remove(TCPProxySocket *);
-	
+
 private:
 	// override from class RegularJob
 	virtual void OnStart();
@@ -503,6 +652,9 @@ private:
 	std::list<PTime *> m_removedTime;
 	/// time to wait before deleting a closed socket
 	PTimeInterval m_socketCleanupTimeout;
+#ifdef HAS_H46017
+	bool m_h46017Enabled;
+#endif
 };
 
 class HandlerList {
