@@ -16,7 +16,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASPDU_H
-#define RASPDU_H "@(#) $Id: RasPDU.h,v 1.20 2011/09/16 18:46:47 willamowius Exp $"
+#define RASPDU_H "@(#) $Id: RasPDU.h,v 1.29 2012/05/16 16:00:36 willamowius Exp $"
 
 #include <list>
 #include "yasocket.h"
@@ -34,22 +34,35 @@ class CallSignalListener;
 class StatusListener;
 class MultiplexRTPListener;
 class RasServer;
+class CallSignalSocket;
 
 const unsigned MaxRasTag = H225_RasMessage::e_serviceControlResponse;
 
-struct GatekeeperMessage {
+class GatekeeperMessage {
+public:
+	GatekeeperMessage() : m_peerPort(0), m_socket(NULL)
+#ifdef HAS_H46017
+		, m_h46017Socket(NULL)
+#endif
+		{ }
+	unsigned GetTag() const { return m_recvRAS.GetTag(); }
+	const char *GetTagName() const;
+	bool Read(RasListener *);
+#ifdef HAS_H46017
+	bool Read(const PBYTEArray & buffer);
+#endif
+	bool Reply() const;
+
 	PPER_Stream m_rasPDU;
 	H225_RasMessage m_recvRAS;
 	H225_RasMessage m_replyRAS;
 	PIPSocket::Address m_peerAddr;
 	WORD m_peerPort;
 	PIPSocket::Address m_localAddr;
-	RasListener *m_socket;
-
-	unsigned GetTag() const { return m_recvRAS.GetTag(); }
-	const char *GetTagName() const;
-	bool Read(RasListener *);
-	bool Reply() const;
+	RasListener * m_socket;
+#ifdef HAS_H46017
+	CallSignalSocket * m_h46017Socket;
+#endif
 };
 
 class RasListener : public UDPSocket {
@@ -81,11 +94,11 @@ protected:
 
 class RasMsg : public Task {
 public:
-	virtual ~RasMsg() { delete m_msg; } //PTRACE(1, "Delete " << m_msg->GetTagName()); }
+	virtual ~RasMsg() { delete m_msg; }
 
 	// new virtual function
 	virtual bool Process() = 0;
-
+   
 	virtual int GetSeqNum() const = 0;
 	virtual H225_NonStandardParameter *GetNonStandardParam() = 0;
 
@@ -106,7 +119,6 @@ public:
 
 	GatekeeperMessage *operator->() { return m_msg; }
 	const GatekeeperMessage *operator->() const { return m_msg; }
-	void Release();
 
 	static void Initialize();
 protected:
@@ -120,7 +132,6 @@ protected:
 	// just pointers to global singleton objects
 	// cache for faster access
 	static Toolkit *Kit;
-	static GkStatus *StatusPort;
 	static RegistrationTable *EndpointTbl;
 	static CallTable *CallTbl;
 	static RasServer *RasSrv;
@@ -189,7 +200,7 @@ protected:
 			if (ValidateSocket(listener, oport))
 				return true;
 			else
-				listener = 0;
+				listener = NULL;
 		}
 		return false;
 	}
@@ -228,7 +239,6 @@ public:
 
 	// give the derived class an opportunity to create customized PDU
 	// default behavior: return the original one
-	// Note: call RasMsg::Release() if new one is created
 	virtual RasMsg *CreatePDU(RasMsg *ras) { return ras; }
 
 	// stop the handler
@@ -240,9 +250,6 @@ protected:
 	RasServer *m_rasSrv;
 
 private:
-	// delete the object after running RasMsg::Exec()
-	static void ProcessRAS(RasMsg *);
-
 	bool m_tagArray[MaxRasTag + 1];
 };
 
@@ -309,46 +316,4 @@ Requester<RAS>::Requester(H225_RasMessage & obj_ras, const Address & ip) : RasRe
 	this->m_rasSrv->RegisterHandler(this); // fix for GCC 3.4.2
 }
 
-/*****************************************************************
-
-The template class let you to modify the default handler of a
-given RAS message. Just explicitly specialize the Process method.
-For example,
-
-template<> bool HookedPDU<H225_RegistrationRequest>::Process()
-{
-	do_something_before_process();
-	// call the default handler
-	bool result = m_opdu->Process();
-	do_something_after_process();
-	return result;
-}
-
-Then add a creator to hook the interested messages
-
-	HookedPDU<H225_RegistrationRequest>::Creator HookedRRQ;
-
-Note the creator must be executed after RasServer::Run().
-
-*****************************************************************/
-
-template<class RAS>
-class HookedPDU : public RasPDU<RAS> {
-public:
-	HookedPDU(GatekeeperMessage *m, RasMsg *p) : RasPDU<RAS>(m), m_opdu(p) {}
-	~HookedPDU() { m_opdu->Release(); }
-
-	virtual bool Process() { return m_opdu->Process(); }
-
-	typedef typename RasPDU<RAS>::RasCreator RasCreator;
-	struct Creator : public RasPDU<RAS>::Creator {
-		Creator() { PAssert(this->m_old, "Error: Hook failed"); } // fix for GCC 3.4.2
-		virtual RasMsg *operator()(GatekeeperMessage *m) const
-		{ return new HookedPDU<RAS>(m, dynamic_cast<RasCreator &>(*(this->m_old))(m)); } // fix for GCC 3.4.2
-	};
-
-private:
-	RasMsg *m_opdu;
-};
-
 #endif // RASPDU_H
