@@ -15,7 +15,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASSRV_H
-#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.68 2011/10/12 09:38:47 willamowius Exp $"
+#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.80 2012/11/26 14:47:12 willamowius Exp $"
 
 #include <vector>
 #include <list>
@@ -91,12 +91,10 @@ public:
 	bool RegisterHandler(RasHandler *);
 	bool UnregisterHandler(RasHandler *);
 
-	void Check();
 	void LoadConfig();
 	void AddListener(RasListener *);
 	void AddListener(TCPListenSocket *);
 	bool CloseListener(TCPListenSocket *);
-	void AddListener(UDPSocket *);
 
 	WORD GetRequestSeqNum();
 
@@ -109,8 +107,8 @@ public:
 	H225_TransportAddress GetRasAddress(const Address &) const;
 	H225_TransportAddress GetCallSignalAddress(const Address &) const;
 
-	bool SendRas(const H225_RasMessage &, const Address &, WORD, RasListener * = 0);
-	bool SendRas(const H225_RasMessage &, const H225_TransportAddress &, RasListener * = 0);
+	bool SendRas(const H225_RasMessage &, const Address &, WORD, RasListener * = NULL);
+	bool SendRas(const H225_RasMessage &, const H225_TransportAddress &, RasListener * = NULL);
 	bool SendRas(const H225_RasMessage &, const Address &, WORD, const Address &);
 	bool SendRIP(H225_RequestSeqNum seqNum, unsigned ripDelay, const Address & addr, WORD port);
 
@@ -122,6 +120,9 @@ public:
 	bool RemoveCallOnDRQ() const { return bRemoveCallOnDRQ; }
 
 	PString GetParent() const;
+	bool IsPassThroughRegistrant();
+	bool RemoveAdditiveRegistration(const H225_ArrayOf_AliasAddress &);
+
 	GkClient *GetGkClient() { return gkClient; }
 	NeighborList *GetNeighbors() { return neighbors; }
 	VirtualQueue *GetVirtualQueue() { return vqueue; }
@@ -140,6 +141,11 @@ public:
 		return authList->Validate(ras, reason);
 	}
 
+	bool ValidateAdditivePDU(
+		RasPDU<H225_RegistrationRequest>& ras, 
+		RRQAuthData& authData
+		);
+
 	bool ValidatePDU(
 		RasPDU<H225_RegistrationRequest>& ras, 
 		RRQAuthData& authData
@@ -194,6 +200,9 @@ public:
 	template<class RAS> void SetAlternateGK(RAS & ras, const NetworkAddress & ip)
 	{
 		H225_ArrayOf_AlternateGK alternates = GetAltGKForIP(ip);
+#if HAS_DATABASE
+		Toolkit::Instance()->AlternateGKs().GetAlternateGK(ip.m_address, alternates);
+#endif
 		if (alternates.GetSize() > 0) {
 			// add alternates by IP
 			ras.IncludeOptionalField(RAS::e_alternateGatekeeper);
@@ -255,6 +264,10 @@ public:
 	// override from class RegularJob
 	virtual void Run();
 
+#ifdef HAS_H46017
+	virtual void ReadH46017Message(PBYTEArray ras, const PIPSocket::Address & fromIP, WORD fromPort, const PIPSocket::Address & localAddr, CallSignalSocket * s);
+#endif
+
 private:
 	// override from class RegularJob
 	virtual void OnStop();
@@ -270,6 +283,7 @@ private:
 	virtual void CleanUp();
 
 	// new virtual function
+	virtual void CreateRasJob(GatekeeperMessage * msg, bool syncronous = false);
 	virtual GkInterface *CreateInterface(const Address &);
 
 	typedef std::list<GkInterface *>::iterator ifiterator;
@@ -280,8 +294,9 @@ private:
 
 	std::list<GkInterface *> interfaces;
 
+	PMutex requests_mutex;
 	std::list<RasMsg *> requests;
-	PMutex hmutex;
+	PMutex handlers_mutex;
 	std::list<RasHandler *> handlers;
 
 	bool GKRoutedSignaling, GKRoutedH245;
