@@ -2,7 +2,7 @@
 //
 // bookkeeping for RAS-Server in H.323 gatekeeper
 //
-// Copyright (c) 2000-2011, Jan Willamowius
+// Copyright (c) 2000-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASTBL_H
-#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.223 2011/12/19 01:14:02 shorne Exp $"
+#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.249 2013/01/04 04:59:59 shorne Exp $"
 
 #include <list>
 #include <map>
@@ -46,7 +46,9 @@ enum H46019Side { SideA, SideB };
 const int INVALID_OSSOCKET = -1;
 const WORD INVALID_RTP_SESSION = 0;
 const PUInt32b INVALID_MULTIPLEX_ID = 0;
-const unsigned GNUGK_KEEPALIVE_RTP_PAYLOADTYPE = 127;	// GnuGk always sends this fixed payload type
+const BYTE GNUGK_KEEPALIVE_RTP_PAYLOADTYPE = 127;	// GnuGk always sends this fixed payload type
+const BYTE MIN_DYNAMIC_PAYLOAD_TYPE = 96;
+const BYTE MAX_DYNAMIC_PAYLOAD_TYPE = 127;
 
 enum PortType { RASPort=1, Q931Port=2, H245Port=3, RTPPort=4, T120Port=5, RadiusPort=6, StatusPort=7 };
 enum PortAction { PortOpen=1, PortClose=2 };
@@ -68,10 +70,10 @@ public:
 // The class T must have Lock() & Unlock() methods
 template<class T> class SmartPtr {
 public:
-	explicit SmartPtr(T *t = 0) : pt(t) { Inc(); }
+	explicit SmartPtr(T *t = NULL) : pt(t) { Inc(); }
 	SmartPtr(const SmartPtr<T> & p) : pt(p.pt) { Inc(); }
 	~SmartPtr() { Dec(); }
-	operator bool() const { return pt != 0; }
+	operator bool() const { return pt != NULL; }
 	T *operator->() const { return pt; }
 
 	bool operator==(const SmartPtr<T> & p) const { return pt == p.pt; }
@@ -119,6 +121,7 @@ public:
 	int GetTimeToLive() const;
 	PIPSocket::Address GetNATIP() const;
 	CallSignalSocket *GetSocket();
+	CallSignalSocket *GetAndRemoveSocket();
 
 	int GetCallTypeOfNumber(bool called = true) const { return called ? m_calledTypeOfNumber : m_callingTypeOfNumber; }
 	int GetCallPlanOfNumber(bool called = true) const { return called ? m_calledPlanOfNumber : m_callingPlanOfNumber; }
@@ -128,7 +131,8 @@ public:
 	virtual void SetCallSignalAddress(const H225_TransportAddress &);
 	virtual void SetEndpointIdentifier(const H225_EndpointIdentifier &);
 	virtual void SetTimeToLive(int);
-	virtual void SetAliases(const H225_ArrayOf_AliasAddress &);
+	virtual void SetAliases(const H225_ArrayOf_AliasAddress &, PBoolean = false);
+	virtual bool RemoveAliases(const H225_ArrayOf_AliasAddress &);
 	virtual void AddNumbers(const PString & numbers);
 	virtual void SetEndpointType(const H225_EndpointType &);
     virtual void SetEndpointInfo(const PString & vendor, const PString & version);
@@ -139,6 +143,9 @@ public:
 	virtual void Update(const H225_RasMessage & lightweightRRQ);
 	virtual bool IsGateway() const { return false; }
 
+	virtual void SetAdditiveRegistrant() {};
+	virtual bool IsAdditiveRegistrant() const { return false; }
+
 	/** Find if one of the given aliases matches any alias for this endpoint.
 
 		@return
@@ -176,7 +183,8 @@ public:
 
 	void SetNAT(bool nat);
 	void SetNATAddress(const PIPSocket::Address &, WORD port = 0);
-	void SetSocket(CallSignalSocket *);
+	void SetNATSocket(CallSignalSocket * socket);
+	void RemoveNATSocket();
 	void SetH46024(bool support);
 	void SetH46024A(bool support);
 	void SetH46024B(bool support);
@@ -210,6 +218,7 @@ public:
 	bool IsPermanent() const;
 	bool IsUsed() const;
 	bool IsUpdated(const PTime *) const;
+	void DeferTTL();
 	bool IsNATed() const;
 	bool SupportH46024() const;
 	bool SupportH46024A() const;
@@ -282,6 +291,7 @@ public:
 	    True if the endpoint can handle at least one more concurrent call.
 	*/
 	bool HasAvailableCapacity(const H225_ArrayOf_AliasAddress & aliases) const;
+	unsigned GetActiveCalls() const { return m_activeCall; }
 	// void DumpPrefixCapacity() const;
 	string LongestPrefixMatch(const PString & alias, int & capacity) const;
 	void UpdatePrefixStats(const PString & dest, int update);
@@ -292,7 +302,12 @@ public:
 	// adding a fixed alias to all calls _to_ this endpoint
 	PString GetAdditionalDestinationAlias() const { return m_additionalDestAlias; }
 
+	bool IsH46017Disabled() const { return m_h46017disabled; }
+	void SetUsesH46017(bool val) {  m_usesH46017 = val; }
+	bool UsesH46017() const { return m_usesH46017; }
 	bool IsH46018Disabled() const { return m_h46018disabled; }
+	void SetUsesH46026(bool val) {  m_usesH46026 = val; }
+	bool UsesH46026() const { return m_usesH46026; }
 	void SetTraversalRole(H46019TraversalType val) { m_traversalType = val; m_nat = (val == TraversalClient); }
 	H46019TraversalType GetTraversalRole() { return m_traversalType; }
 	bool IsTraversalServer() const { return m_traversalType == TraversalServer; }
@@ -374,8 +389,11 @@ protected:
 
 	EPNatTypes m_epnattype;
 	bool m_usesH46023, m_H46024, m_H46024a, m_H46024b, m_natproxy, m_internal, m_remote;
+	bool m_h46017disabled;
 	bool m_h46018disabled;
 	bool m_usesH460P;
+	bool m_usesH46017;
+    bool m_usesH46026;
 	H46019TraversalType m_traversalType;	// this is not what GnuGk acts like, but what this EPRec is a proxy for
 	
 	long m_bandwidth;	// bandwidth currently occupied by this endpoint
@@ -397,6 +415,9 @@ public:
 	virtual void Update(const H225_RasMessage & lightweightRRQ);
 	virtual bool IsGateway() const { return true; }
 
+	virtual void SetAdditiveRegistrant();
+	virtual bool IsAdditiveRegistrant() const;
+
 	/// Overiden from EndpointRec
 	virtual bool LoadConfig();
 
@@ -443,11 +464,6 @@ public:
 	    a dialed number.
 	*/
 	int GetPriority() const { return priority; }
-	
-	/// Set the priority for this gateway.
-	void SetPriority(
-		int newPriority
-		);
 
 	//void DumpPriorities() const;
 
@@ -465,6 +481,8 @@ protected:
 	bool defaultGW;
 	/// priority for this gateway (when more than one gw matches a dialed number)
 	int priority;
+
+	bool additiveRegistrant;
 };
 
 
@@ -537,6 +555,9 @@ public:
 	void ClearTable();
 	void CheckEndpoints();
 
+	// handle remote closing of a NAT socket
+	void OnNATSocketClosed(CallSignalSocket * s);
+
 	void PrintAllRegistrations(USocket *client, bool verbose=FALSE);
 	void PrintAllCached(USocket *client, bool verbose=FALSE);
 	void PrintRemoved(USocket *client, bool verbose=FALSE);
@@ -644,6 +665,7 @@ class H323TransportAddress;
 #define H46019_BOTH		(H46019_CALLER + H46019_CALLED)	// m_h46019dir = 3 ' Both need it
 #endif
 
+
 class CallRec {
 public:
 	/// flag to overwrite proxy settings for the call
@@ -757,8 +779,12 @@ public:
 	  */
 	bool NATOffLoad(bool iscalled, NatStrategy & natinst);
 
+	/** Return whether an unknown incoming call can be offloaded from having to proxy
+	  */
+	bool NATAssistCallerUnknown(NatStrategy & natinst);
+
 	/** Get String representation of the NATStrategy */
-	PString GetNATOffloadString(NatStrategy nat);
+	PString GetNATOffloadString(NatStrategy nat) const;
 
 
 	/** Get the NATStrategy Default is Unknown */
@@ -776,7 +802,7 @@ public:
     CallSignalSocket * H46024BSignalSocket(bool response);
  
 	/** Initiate Probe */
-	void H46024BInitiate(WORD sessionID, const H323TransportAddress & fwd, const H323TransportAddress & rev, unsigned muxID=0);
+	void H46024BInitiate(WORD sessionID, const H323TransportAddress & fwd, const H323TransportAddress & rev, unsigned muxID_fwd=0, unsigned muxID_rev=0);
  
 	/** Response Probe */
 	void H46024BRespond();
@@ -790,7 +816,7 @@ public:
 	/** Return whether the endpoints are registered at the same gatekeeper so
 	    only 1 gatekeeper is involved in the call
 	  */
-	bool SingleGatekeeper();
+	bool SingleGatekeeper() const;
 
 	/** Return remote party device information
 	  */
@@ -818,6 +844,7 @@ public:
 	void SetForward(CallSignalSocket *, const H225_TransportAddress &, const endptr &, const PString &, const PString &);
 	void RerouteDropCalling();
 	void RerouteDropCalled();
+	bool DropCalledAndTryNextRoute();
 	void SetBandwidth(long bandwidth) { m_bandwidth = bandwidth; if (m_bandwidth < 0) m_bandwidth = 0; }
 	void SetSocket(CallSignalSocket *, CallSignalSocket *);
 	void SetCallSignalSocketCalling(CallSignalSocket* socket);
@@ -832,7 +859,7 @@ public:
 	void Disconnect(bool = false); // send ReleaseComplete ?
 	void RemoveAll();
 	void RemoveSocket();
-	void SendReleaseComplete(const H225_CallTerminationCause * = 0);
+	void SendReleaseComplete(const H225_CallTerminationCause * = NULL);
 	void BuildDRQ(H225_DisengageRequest &, unsigned reason) const;
 
 	bool CompareCallId(const H225_CallIdentifier *CallId) const;
@@ -856,13 +883,14 @@ public:
 	bool IsH245Routed() const { return m_h245Routed; }
 	bool IsToParent() const { return m_toParent; }
 	bool IsForwarded() const { return m_forwarded; }
-	bool IsSocketAttached() const { return (m_callingSocket != 0); }
+	bool IsSocketAttached() const { return (m_callingSocket != NULL); }
 
 	PString GenerateCDR(
 		/// timestamp formatting string (empty for a default RFC822 format)
 		const PString & timestampFormat = PString::Empty()
 		) const;
 	PString PrintOn(bool verbose) const;
+	PString GetMediaRouting() const;
 	PString PrintPorts() const;
 
 	void Lock();
@@ -1252,8 +1280,6 @@ public:
 	void SetMediaOriginatingIp(const PIPSocket::Address &addr);
 	bool GetMediaOriginatingIp(PIPSocket::Address &addr) const;
 
-	void SetRADIUSClass(const PBYTEArray &bytes);
-	void SetRADIUSClass(void * bytes, PINDEX len);
 	PBYTEArray GetRADIUSClass() const;
 	
 	bool IsProceedingSent() const { return m_proceedingSent; }
@@ -1289,17 +1315,15 @@ public:
 	void RemoveKeepAllAlives();
 	
 	void SetSessionMultiplexDestination(WORD session, void * openedBy, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
-	void SetLCMultiplexDestination(unsigned lc, void * openedBy, bool isRTCP, const H323TransportAddress & toAddress, H46019Side side);
-	void SetLCMultiplexID(unsigned lc, void * openedBy, bool isRTCP, PUInt32b multiplexID, H46019Side side);
-	void SetLCMultiplexSocket(unsigned lc, void * openedBy, bool isRTCP, int multiplexSocket, H46019Side side);
 #endif
 
 #ifdef HAS_H235_MEDIA
     typedef NATType EncDir;
     H235Authenticators & GetAuthenticators() { return m_authenticators; }
     void SetMediaEncryption(EncDir dir);
-    bool IsMediaEncryption()                 { return m_encyptDir != none;  }
-    EncDir GetEncryptDirection()             { return m_encyptDir;      }
+    bool IsMediaEncryption() { return m_encyptDir != none; }
+    EncDir GetEncryptDirection() const { return m_encyptDir; }
+	BYTE GetNewDynamicPayloadType();
 #endif
 
 private:
@@ -1450,13 +1474,14 @@ private:
 	struct H46024Balternate {
 		 H245_TransportAddress forward;
 		 H245_TransportAddress reverse;
-		 unsigned multiplexID;
+		 unsigned multiplexID_fwd;
+		 unsigned multiplexID_rev;
+		 int sent;
 	};
  
 	std::map<WORD,H46024Balternate> m_H46024Balternate;
 	void BuildH46024AnnexBMessage(bool initiate,H245_MultimediaSystemControlMessage & h245msg, const std::map<WORD,H46024Balternate> & alt);
 	list<int> m_h46024Bflag;
-	PMutex m_H46024Bmutex;
 #endif
 	/// unregistered caller NAT'd
 	bool m_unregNAT;
@@ -1511,7 +1536,9 @@ private:
 
 #ifdef HAS_H235_MEDIA
     H235Authenticators m_authenticators;
-    EncDir m_encyptDir;
+    EncDir m_encyptDir;	// party for which we simulate encryption
+	PMutex m_PTMutex;
+    BYTE m_dynamicPayloadTypeCounter;
 #endif
 };
 
@@ -1574,7 +1601,9 @@ public:
 	void LoadConfig();
 	void UpdatePrefixCapacityCounters();    // after Reload
 
-	PINDEX Size() const { return m_activeCall; }
+	PINDEX Size() const { return m_activeCall; }	// number of currently active calls
+	unsigned TotalCallCount() const { return m_CallCount; }	// number of calls since startup
+	unsigned SuccessfulCallCount() const { return m_successCall; }	// number of succesfull calls since startup
 
 	/** @return
 	    Timeout value for a signaling channel to be opened after ACF
@@ -1606,7 +1635,6 @@ private:
 	}
 
 	bool InternalRemovePtr(CallRec *call);
-	void InternalRemove(const H225_CallIdentifier & CallId);
 	void InternalRemove(iterator);
 	void InternalRemoveFailedLeg(iterator);
 
@@ -1685,15 +1713,6 @@ inline H225_EndpointIdentifier EndpointRec::GetEndpointIdentifier() const
 	PWaitAndSignal lock(m_usedLock);
 	return m_endpointIdentifier;
 }
-  
-inline int EndpointRec::GetTimeToLive() const
-{
-	if (m_nat || IsTraversalClient()) {
-		// force timeToLive to 5 - 30 sec, 19 sec if not set
-		return m_timeToLive == 0 ? 19 : max(5, min(30, m_timeToLive));
-	}
-	return m_timeToLive;
-}
 
 inline H225_ArrayOf_AliasAddress EndpointRec::GetAliases() const
 {
@@ -1825,10 +1844,16 @@ inline PIPSocket::Address EndpointRec::GetNATIP() const
 }
 
 inline CallSignalSocket *EndpointRec::GetSocket() 
+{
+	PWaitAndSignal lock(m_usedLock);
+	return m_natsocket;
+}
+
+inline CallSignalSocket *EndpointRec::GetAndRemoveSocket() 
 {
 	PWaitAndSignal lock(m_usedLock);
 	CallSignalSocket *socket = m_natsocket;
-	m_natsocket = 0;
+	m_natsocket = NULL;
 	return socket;
 }
 
@@ -1850,6 +1875,12 @@ inline bool EndpointRec::IsUpdated(const PTime *now) const
 	return (!ttl || (*now - m_updatedTime).GetSeconds() < ttl);
 }
 
+inline void EndpointRec::DeferTTL()
+{
+	PWaitAndSignal lock(m_usedLock);
+	m_updatedTime = PTime();
+}
+
 inline PTime EndpointRec::GetUpdatedTime() const
 { 
 	PWaitAndSignal lock(m_usedLock);
