@@ -5,7 +5,7 @@
 // Routing Mechanism for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2003
-// Copyright (c) 2004-2011, Jan Willamowius
+// Copyright (c) 2004-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -15,16 +15,15 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef ROUTING_H
-#define ROUTING_H "@(#) $Id: Routing.h,v 1.58 2011/11/18 14:40:02 willamowius Exp $"
+#define ROUTING_H "@(#) $Id: Routing.h,v 1.78 2012/12/28 20:35:42 willamowius Exp $"
 
 #include <map>
 #include <list>
-#include "slist.h"
 #include "singleton.h"
+#include "factory.h"
 #include "RasTbl.h"
 #include "stl_supp.h"
 
-
 // forward references to avoid includes
 class H225_AdmissionRequest;
 class H225_LocationRequest;
@@ -55,7 +54,7 @@ public:
 		e_toNeighbor = 2,
 		e_Reject = 4
 	};
-	
+
 	Route();
 	Route(
 		const PString & policyName,
@@ -76,11 +75,11 @@ public:
 
 	bool operator< (const Route & rhs) { return m_priority < rhs.m_priority; }
 
+	unsigned GetPriority() const { return m_priority; }
+	void SetPriority(unsigned p) { m_priority = p; }
 	PString AsString() const;
-
 	bool IsFailoverActive(unsigned cause) const;
 
-
 	H225_TransportAddress m_destAddr; /// destination address for signaling
 	endptr m_destEndpoint; /// destination endpoint record (if available)
 	PString m_policy; /// name of the policy that found the route
@@ -106,14 +105,15 @@ public:
 	};
 	
 	RoutingRequest();
-	RoutingRequest(const std::list<Route> &failedRoutes);
+	RoutingRequest(const std::list<Route> & failedRoutes);
 	virtual ~RoutingRequest();
 
-	bool AddRoute(const Route &route);
-	bool GetFirstRoute(Route &route);
+	bool AddRoute(const Route & route);
+	bool GetFirstRoute(Route & route);
 	void RemoveAllRoutes();
+	bool HasRoutes() const { return !m_routes.empty(); }
 	std::list<Route> & GetRoutes() { return m_routes; }
-	
+
 	void SetRejectReason(unsigned reason) { m_reason = reason; }
 	void SetFlag(unsigned f) { m_flags |= f; }
 	unsigned GetRejectReason() const { return m_reason; }
@@ -154,12 +154,13 @@ public:
 	Wrapper *GetWrapper() { return m_wrapper; }
 	H225_ArrayOf_AliasAddress *GetAliases();
 	void SetAliases(H225_ArrayOf_AliasAddress & aliases);
+	const H225_TransportAddress * GetDestIP() const;
 	const ReqObj & GetRequest() const { return m_request; }
 	const Wrapper *GetWrapper() const { return m_wrapper; }
 	PString GetCallingStationId() const { return m_callingStationId; }
 	PUInt64 GetClientAuthId() const { return m_clientAuthId; }
 	const H225_ArrayOf_AliasAddress *GetAliases() const
-	{ return const_cast<Request<R, W> *>(this)->GetAliases(); }
+		{ return const_cast<Request<R, W> *>(this)->GetAliases(); }
 
 private:
 	ReqObj & m_request;
@@ -175,36 +176,72 @@ typedef Request<H225_Setup_UUIE, SetupMsg> SetupRequest;
 typedef Request<H225_Facility_UUIE, FacilityMsg> FacilityRequest;
 
 
-class Policy : public SList<Policy> {
+template<class T>
+class PolicyList {
 public:
-	Policy() : m_name("Undefined") {}
-	virtual ~Policy() {}
+	typedef T Base;	// for SimpleCreator template 
 
-	template <class R> bool HandleRas(Request<R,RasMsg> & request)
+	PolicyList() : m_next(NULL) { }
+	virtual ~PolicyList() { delete m_next; }  // delete whole list recursively
+
+	static T * Create(const PStringArray & rules);
+
+protected:
+	T * m_next;
+};
+
+
+// ignore overflow warning when comparing size
+#if (!_WIN32) && (GCC_VERSION >= 40400)
+#pragma GCC diagnostic ignored "-Wstrict-overflow"
+#endif
+
+template<class T>
+T *PolicyList<T>::Create(const PStringArray & rules)
+{
+	T * next = NULL;
+	for (int i = rules.GetSize(); --i >= 0; ) {
+		PStringArray id = rules[i].Tokenise("::", false);	// check for <policy>::<ID>
+		if (T * current = Factory<T>::Create(id[0])) {
+			if (id.GetSize() > 1) 
+				current->SetInstance(id[1]);
+			else
+				current->SetInstance("");
+
+			current->m_next = next;
+			next = current;
+		}
+	}
+	return next;
+}
+
+
+class Policy : public PolicyList<Policy> {
+public:
+	Policy() : m_name("Undefined"), m_iniSection("Routing::Undefined") { }
+	virtual ~Policy() { }
+
+	template <class R> bool HandleRas(Request<R, RasMsg> & request)
 	{
 		if( IsActive() ) {
-#if PTRACING
-			const char* tagname = request.GetWrapper()
+			const char * tagname = request.GetWrapper()
 				? request.GetWrapper()->GetTagName() : "unknown";
 			const unsigned seqnum = request.GetRequest().m_requestSeqNum.GetValue();
-			PTRACE(5,"ROUTING\tChecking policy "<<m_name
-				<<" for the request "<<tagname<<' '<<seqnum
-				);
-#endif
+			PTRACE(5, "ROUTING\tChecking policy " << m_name
+				<< " for the request " << tagname << ' ' << seqnum);
 			if( OnRequest(request) ) {
-#if PTRACING
-				PTRACE(5,"ROUTING\tPolicy "<<m_name
-					<<" applied to the request "<<tagname<<' '<<seqnum
-					);
-#endif
+				PTRACE(5, "ROUTING\tPolicy " << m_name
+					<< " applied to the request " << tagname << ' ' << seqnum);
 				return true;
 			}
 		}
 		return m_next && m_next->HandleRas(request);
 	}
 
-	bool Handle(SetupRequest &request);
-	bool Handle(FacilityRequest &request);
+	bool Handle(SetupRequest & request);
+	bool Handle(FacilityRequest & request);
+
+	void SetInstance(const PString & instance);
 
 protected:
 	// new virtual function
@@ -219,10 +256,13 @@ protected:
 	virtual bool OnRequest(SetupRequest &)	   { return false; }
 	virtual bool OnRequest(FacilityRequest &)  { return false; }
 
+	virtual void LoadConfig(const PString & instance) { }	// should be used to load config, always called after the policy object is created
+
 protected:
 	/// human readable name for the policy - it should be set inside constructors
 	/// of derived policies, default value is "undefined"
 	const char* m_name;
+	PString m_iniSection;
 };
 
 
@@ -231,7 +271,9 @@ class ExplicitPolicy : public Policy {
 public:
 	ExplicitPolicy();
 
-	static void MapDestination(H225_TransportAddress & addr);
+	static void MapDestination(H225_AdmissionRequest & arq);
+	static void MapDestination(H225_Setup_UUIE & setupBody);
+	static void MapDestination(H225_Facility_UUIE & facility);
 	static void OnReload();
 
 protected:
@@ -240,7 +282,7 @@ protected:
 	virtual bool OnRequest(SetupRequest &);
 	virtual bool OnRequest(FacilityRequest &);
 
-	static std::map<PString, H225_TransportAddress> m_destMap;
+	static std::map<PString, PString> m_destMap;
 };
 
 
@@ -260,6 +302,7 @@ protected:
 	virtual bool FindByAliases(LocationRequest&, H225_ArrayOf_AliasAddress&) = 0;
 };
 
+// this class passes incoming requests through the chain of routing policies
 class Analyzer : public Singleton<Analyzer> {
 public:
 	Analyzer();
@@ -331,7 +374,9 @@ protected:
 
 	virtual bool DNSLookup(const PString & hostname, PIPSocket::Address & addr) const;
 
-	static bool m_resolveNonLocalLRQs;
+	virtual void LoadConfig(const PString & instance);
+
+	bool m_resolveNonLocalLRQs;
 };
 
 // a policy to route call via external program
@@ -366,6 +411,8 @@ protected:
 	virtual bool OnRequest(AdmissionRequest &);
 	virtual bool OnRequest(SetupRequest &);
 
+	virtual void LoadConfig(const PString & instance);
+
 private:
 	NumberAnalysisPolicy(const NumberAnalysisPolicy &);
 	NumberAnalysisPolicy& operator=(const NumberAnalysisPolicy &);
@@ -386,12 +433,17 @@ protected:
 	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
 
+	virtual bool FindByAliasesInternal(const PString & schema, RoutingRequest &, H225_ArrayOf_AliasAddress &, PBoolean &);
+
+	virtual void LoadConfig(const PString & instance);
+
 	bool m_resolveLRQs;
+	PStringToString m_enum_schema;
 };
 
 class DestinationRoutes {
 public:
-	DestinationRoutes() { m_endChain = false; m_reject = false; m_rejectReason = 0; m_aliasesChanged = false; }
+	DestinationRoutes();
 	~DestinationRoutes() { }
 	
 	bool EndPolicyChain() const { return m_endChain; }
@@ -402,8 +454,8 @@ public:
 	bool ChangeAliases() const { return m_aliasesChanged; }
 	H225_ArrayOf_AliasAddress GetNewAliases() const { return m_newAliases; }
 	void SetNewAliases(const H225_ArrayOf_AliasAddress & aliases) { m_newAliases = aliases; m_aliasesChanged = true; }
-	
-	void AddRoute(const Route & route, bool endChain = true) { m_routes.push_back(route); m_endChain = endChain; }
+
+	void AddRoute(const Route & route, bool endChain = true);
 
 	std::list<Route> m_routes;
 
@@ -415,11 +467,11 @@ protected:
 	H225_ArrayOf_AliasAddress m_newAliases;
 };
 
-// a policy to route calls via an SQL database
-class SqlPolicy : public Policy {
+// superclass for dynamic policies like sql and lua scripring
+class DynamicPolicy : public Policy {
 public:
-	SqlPolicy();
-	virtual ~SqlPolicy();
+	DynamicPolicy();
+	virtual ~DynamicPolicy() { }
 
 protected:
 	virtual bool IsActive() const { return m_active; }
@@ -428,7 +480,34 @@ protected:
 	virtual bool OnRequest(LocationRequest &);
 	virtual bool OnRequest(SetupRequest &);
 
-	virtual void DatabaseLookup(
+	virtual void RunPolicy(
+		/*in */
+		const PString & source,
+		const PString & calledAlias,
+		const PString & calledIP,
+		const PString & caller,
+		const PString & callingStationId,
+		const PString & callid,
+		const PString & messageType,
+		const PString & clientauthid,
+		/* out: */
+		DestinationRoutes & destination) = 0;
+
+protected:
+	// active ?
+	bool m_active;
+};
+
+// a policy to route calls via an SQL database
+class SqlPolicy : public DynamicPolicy {
+public:
+	SqlPolicy();
+	virtual ~SqlPolicy();
+
+protected:
+	virtual void LoadConfig(const PString & instance);
+
+	virtual void RunPolicy(
 		/*in */
 		const PString & source,
 		const PString & calledAlias,
@@ -442,8 +521,6 @@ protected:
 		DestinationRoutes & destination);
 
 protected:
-	// active ?
-	bool m_active;
 	// connection to the SQL database
 	GkSQLConnection* m_sqlConn;
 	// parametrized query string for the routing query
@@ -452,6 +529,7 @@ protected:
 	long m_timeout;
 };
 
+
 // a policy to route all calls to one default endpoint
 class CatchAllPolicy : public Policy {
 public:
@@ -464,6 +542,8 @@ protected:
 	virtual bool OnRequest(SetupRequest & request) { return CatchAllRoute(request); }
 
 	bool CatchAllRoute(RoutingRequest & request) const;
+
+	virtual void LoadConfig(const PString & instance);
 	
 	PString m_catchAllAlias;
 	PString m_catchAllIP;
