@@ -2,7 +2,7 @@
 //
 // SoftPBX.cxx
 //
-// Copyright (c) 2000-2011, Jan Willamowius
+// Copyright (c) 2000-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -226,7 +226,7 @@ void SoftPBX::DisconnectIp(const PString & Ip)
 		|| (IsIPv6Address(Ip) && Ip.Find("]:") == P_MAX_INDEX)
 		) {
 		// no port specified
-		while (Call = CallTable::Instance()->FindBySignalAdrIgnorePort(callSignalAddress)) {
+		while ((Call = CallTable::Instance()->FindBySignalAdrIgnorePort(callSignalAddress))) {
 			unsigned CallNumber = Call->GetCallNumber();
 			Call->Disconnect(true);
 			CallTable::Instance()->RemoveCall(Call);
@@ -236,7 +236,7 @@ void SoftPBX::DisconnectIp(const PString & Ip)
 		}
 	} else {
 		// port specify, search for it
-		while (Call = CallTable::Instance()->FindBySignalAdr(callSignalAddress)) {
+		while ((Call = CallTable::Instance()->FindBySignalAdr(callSignalAddress))) {
 			unsigned CallNumber = Call->GetCallNumber();
 			Call->Disconnect(true);
 			CallTable::Instance()->RemoveCall(Call);
@@ -278,7 +278,7 @@ void SoftPBX::DisconnectEndpoint(const endptr &ep)
 	}
 	callptr Call;
 	// remove all calls of ep
-	while (Call = CallTable::Instance()->FindCallRec(ep)) {
+	while ((Call = CallTable::Instance()->FindCallRec(ep))) {
 		Call->Disconnect();
 		CallTable::Instance()->RemoveCall(Call);
 	}
@@ -327,7 +327,7 @@ void SoftPBX::TransferCall(const PString & SourceAlias, const PString & Destinat
 	endptr lDestForward;
 	endptr lSrcForward;
 	callptr lCall;
-	CallSignalSocket *lForwardedSocket = 0;
+	CallSignalSocket *lForwardedSocket = NULL;
 
 	PStringList lBufferAliasArrayString;
 	H225_ArrayOf_AliasAddress lBufferAliasArray;
@@ -492,6 +492,7 @@ void SoftPBX::MakeCall(const PString & SourceAlias, const PString & DestinationA
 		MakeCallEndPoint::Instance()->ThirdPartyMakeCall(SourceAlias, DestinationAlias);
 	} else {
 		PTRACE(1, "GK\tSoftPBX: MakeCall registration of pseudo-endpoint failed");
+		SNMP_TRAP(7, SNMPError, General, "MakeCall endpoint registration failed");
 		delete MakeCallEndPoint::Instance();	// delete this failed instance, it will never work
 	}
 }
