@@ -2,7 +2,7 @@
 //
 // Toolkit class for the GnuGk
 //
-// Copyright (c) 2000-2011, Jan Willamowius
+// Copyright (c) 2000-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef TOOLKIT_H
-#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.88 2011/11/10 11:03:46 willamowius Exp $"
+#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.102 2012/11/29 14:29:16 willamowius Exp $"
 
 #include <vector>
 #include <map>
@@ -123,7 +123,7 @@ class Toolkit : public Singleton<Toolkit>
 		typedef PIPSocket::InterfaceTable InterfaceTable;
 
 	public:
-		RouteTable() : rtable_begin(0), rtable_end(0), DynExtIP(false), ExtIP("") { /* initialize later */ }
+		RouteTable() : rtable_begin(NULL), rtable_end(NULL), DynExtIP(false), ExtIP("") { /* initialize later */ }
 		virtual ~RouteTable() { ClearTable(); }
 		Address GetLocalAddress(unsigned version) const;
 		Address GetLocalAddress(const Address & dest) const;
@@ -131,12 +131,12 @@ class Toolkit : public Singleton<Toolkit>
 
 		void InitTable();
 		void ClearTable();
-		bool IsEmpty() const { return rtable_begin == 0; }
+		bool IsEmpty() const { return rtable_begin == NULL; }
 
 		std::vector<NetworkAddress> GetInternalNetworks() const { return m_internalnetworks; }
 		void AddInternalNetwork(const NetworkAddress & network);
 
-	    virtual bool IsMasquerade(const PIPSocket::Address &) const { return false; }
+	    virtual bool IsMasquerade(PIPSocket::Address &) const { return false; }
 	protected:
 		class RouteEntry : public PIPSocket::RouteEntry {
 		public:
@@ -298,6 +298,28 @@ class Toolkit : public Singleton<Toolkit>
 	AssignedGatekeepers AssignedGKs() { return m_AssignedGKs; }
 #endif
 
+#if HAS_DATABASE
+	class AlternateGatekeepers {
+	 public:
+	    AlternateGatekeepers();
+		~AlternateGatekeepers();
+
+	    void LoadConfig(PConfig *);
+        bool GetAlternateGK(const PIPSocket::Address & ip, H225_ArrayOf_AlternateGK & gklist);
+		bool QueryAlternateGK(const PIPSocket::Address & ip, PStringArray & addresses);
+
+     private:
+	   bool m_sqlactive;
+	   // connection to the SQL database
+	   GkSQLConnection* m_sqlConn;
+	   // parametrized query string for the auth condition string retrieval
+	   PString m_query;
+	   // query timeout
+	   long m_timeout;
+	};
+	AlternateGatekeepers AlternateGKs() { return m_AlternateGKs; }
+#endif
+
 #if HAS_DATABASE
 	class QoSMonitor {
 	  public:
@@ -331,7 +353,6 @@ class Toolkit : public Singleton<Toolkit>
 	bool RewriteE164(H225_ArrayOf_AliasAddress & aliases);
 
 	bool RewritePString(PString & s) { return m_Rewrite.RewritePString(s); }
-	bool IsNumeric(const PString & s);
 
 	// Class to allow correct use of STL inside PDictionary type
 	class GWRewriteEntry : public PObject {
@@ -367,8 +388,10 @@ class Toolkit : public Singleton<Toolkit>
 	void SetGKHome(const PStringArray &);
 	bool IsGKHome(const PIPSocket::Address & addr) const;
 
+	bool RemoveAllH235Tokens() const { return m_alwaysRemoveH235Tokens; }
+	bool RemoveH235TokensFrom(const PIPSocket::Address & addr) const;
 #ifdef HAS_H235_MEDIA
-	bool IsH235MediaEnabled() const { return m_H235MediaEnabled; }
+	bool IsH235HalfCallMediaEnabled() const { return m_H235HalfCallMediaEnabled; }
 #endif
 
 	bool isBehindNAT(PIPSocket::Address &) const;
@@ -382,11 +405,16 @@ class Toolkit : public Singleton<Toolkit>
 	bool GetH46023STUN(const PIPSocket::Address & addr,  H323TransportAddress & stun);
 	bool H46023SameNetwork(const PIPSocket::Address & addr1, const PIPSocket::Address & addr2);
 #endif
+#ifdef HAS_H46026
+	bool IsH46026Enabled() const;
+#endif
 #ifdef HAS_H460P
 	bool IsH460PEnabled() const;
 	GkPresence & GetPresenceHandler();
 #endif
+	bool IsSNMPEnabled() const;
 	bool IsIPv6Enabled() const;
+	bool IsPortNotificationActive();
 	void PortNotification(PortType type, PortAction action, const PString & protocol,
 							const PIPSocket::Address & addr, WORD port,
 							const H225_CallIdentifier & callID = 0);
@@ -550,16 +578,18 @@ class Toolkit : public Singleton<Toolkit>
 
 	/// Inbound rewrite for ANI/CLI
 	void RewriteCLI(
-		SetupMsg &msg /// Q.931 Setup message to be rewritten
+		SetupMsg & msg /// Q.931 Setup message to be rewritten
 		) const;
 		
 	/// Outbound rewrite for ANI/CLI
 	void RewriteCLI(
 		SetupMsg &msg, /// Q.931 Setup message to be rewritten
-		SetupAuthData &authData, /// additional data
+		SetupAuthData & authData, /// additional data
 		const PIPSocket::Address &destAddr /// callee's IP
 		) const;
 
+	void RewriteSourceAddress(SetupMsg & msg) const;
+
 	void SetRerouteCauses(unsigned char *causeMap);
 
 	/** Map H225_ReleaseCompleteReason code to Q.931 cause value.
@@ -599,6 +629,7 @@ protected:
 	AssignedGatekeepers m_AssignedGKs;
 #endif
 #if HAS_DATABASE
+    AlternateGatekeepers m_AlternateGKs;
 	QoSMonitor m_qosMonitor;
 #endif
 	RouteTable m_RouteTable;
@@ -633,8 +664,10 @@ private:
 	/// global cause code translation
 	std::map<unsigned, unsigned> m_receivedCauseMap;
 	std::map<unsigned, unsigned> m_sentCauseMap;
+	bool m_alwaysRemoveH235Tokens;
+	std::vector<NetworkAddress> m_removeH235TokensfromNetwork;
 #ifdef HAS_H235_MEDIA
-	bool m_H235MediaEnabled;
+	bool m_H235HalfCallMediaEnabled;
 #endif
 	// is H460.18 enabled ?
 #ifdef HAS_H46018
