@@ -2,7 +2,7 @@
 //
 // gk.cxx for GNU Gatekeeper
 //
-// Copyright (c) 2000-2011, Jan Willamowius
+// Copyright (c) 2000-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -33,6 +33,7 @@
 #include "gk.h"
 #include "config.h"
 #include "capctrl.h"
+#include "snmp.h"
 
 using std::vector;
 
@@ -42,7 +43,6 @@ PIPSocket::Address GNUGK_INADDR_ANY(INADDR_ANY);
 
 PReadWriteMutex ConfigReloadMutex;
 
-#if PTRACING
 PTextFile* Gatekeeper::m_logFile = NULL;
 PFilePath Gatekeeper::m_logFilename;
 PMutex Gatekeeper::m_logFileMutex;
@@ -51,48 +51,20 @@ int Gatekeeper::m_rotateMinute = 0;
 int Gatekeeper::m_rotateHour = 0;
 int Gatekeeper::m_rotateDay = 0;
 GkTimer* Gatekeeper::m_rotateTimer = GkTimerManager::INVALID_HANDLE;
-#endif // PTRACING
 
 PSemaphore ShutdownMutex(1,1);
 
-namespace { // keep the global objects private
-
-PTimedMutex ReloadMutex;
-
-#ifndef _WIN32
-PString pidfile("/var/run/gnugk.pid");
-#endif
-
-void ShutdownHandler()
-{
-#if PTRACING
-	Gatekeeper::EnableLogFileRotation(false);
-#endif
-	// delete singleton objects
-	PTRACE(3, "GK\tDeleting global reference tables");
-
-	Job::StopAll();
-	if (CapacityControl::InstanceExists())
-		delete CapacityControl::Instance();
-	if (CallTable::InstanceExists())
-		delete CallTable::Instance();
-	if (RegistrationTable::InstanceExists())
-		delete RegistrationTable::Instance();
-	if (RasServer::InstanceExists())
-		delete RasServer::Instance();
-	if (MakeCallEndPoint::InstanceExists())
-		delete MakeCallEndPoint::Instance();
-	if (Toolkit::InstanceExists())
-		delete Toolkit::Instance();
-	PTRACE(3, "GK\tdelete ok");
-
-#if PTRACING
-	Gatekeeper::CloseLogFile();
-#endif
-}
-
-static const char * KnowConfigEntries[][2] = {
+const char * KnownConfigEntries[][2] = {
 	// valid config entries
+	{ "AlternateGatekeepers::SQL", "CacheTimeout" },
+	{ "AlternateGatekeepers::SQL", "Database" },
+	{ "AlternateGatekeepers::SQL", "Driver" },
+	{ "AlternateGatekeepers::SQL", "Host" },
+	{ "AlternateGatekeepers::SQL", "Library" },
+	{ "AlternateGatekeepers::SQL", "MinPoolSize" },
+	{ "AlternateGatekeepers::SQL", "Password" },
+	{ "AlternateGatekeepers::SQL", "Query" },
+	{ "AlternateGatekeepers::SQL", "Username" },
 	{ "AssignedAliases::SQL", "CacheTimeout" },
 	{ "AssignedAliases::SQL", "Database" },
 	{ "AssignedAliases::SQL", "Driver" },
@@ -108,7 +80,9 @@ static const char * KnowConfigEntries[][2] = {
 	{ "AssignedGatekeepers::SQL", "Host" },
 	{ "AssignedGatekeepers::SQL", "Library" },
 	{ "AssignedGatekeepers::SQL", "MinPoolSize" },
+	{ "AssignedGatekeepers::SQL", "Password" },
 	{ "AssignedGatekeepers::SQL", "Query" },
+	{ "AssignedGatekeepers::SQL", "Username" },
 	{ "CTI::Agents", "CTI_Timeout" },	// obsolete, but still accepted
 	{ "CTI::Agents", "RequestTimeout" },
 	{ "CTI::Agents", "VirtualQueue" },	// obsolete, but still accepted
@@ -131,19 +105,28 @@ static const char * KnowConfigEntries[][2] = {
 	{ "CallTable", "IRRFrequency" },
 	{ "CallTable", "SingleFailoverCDR" },
 	{ "CallTable", "TimestampFormat" },
-	{ "Endpoint", "DisableH46018" },
 	{ "Endpoint", "Discovery" },
 	{ "Endpoint", "E164" },
+#ifdef HAS_H46018
+	{ "Endpoint", "EnableH46018" },
+#endif
+#ifdef HAS_H46023
+	{ "Endpoint", "EnableH46023" },
+#endif
 	{ "Endpoint", "EndpointIdentifier" },
 	{ "Endpoint", "ForwardDestIp" },
 	{ "Endpoint", "Gatekeeper" },
 	{ "Endpoint", "GatekeeperIdentifier" },
 	{ "Endpoint", "H323ID" },
+	{ "Endpoint", "HideGk" },
 	{ "Endpoint", "NATKeepaliveInterval" },
 	{ "Endpoint", "NATRetryInterval" },
 	{ "Endpoint", "Password" },
 	{ "Endpoint", "Prefix" },
+	{ "Endpoint", "ProductId" },
+	{ "Endpoint", "ProductVersion" },
 	{ "Endpoint", "RRQRetryInterval" },
+	{ "Endpoint", "EnableAdditiveRegistration" },
 	{ "Endpoint", "TimeToLive" },
 	{ "Endpoint", "Type" },
 	{ "Endpoint", "UnregisterOnReload" },
@@ -176,6 +159,7 @@ static const char * KnowConfigEntries[][2] = {
 	{ "Gatekeeper::Auth", "SimplePasswordAuth" },
 	{ "Gatekeeper::Auth", "default" },
 	{ "Gatekeeper::Main", "AlternateGKs" },
+	{ "Gatekeeper::Main", "Authenticators" },
 	{ "Gatekeeper::Main", "Bind" },
 	{ "Gatekeeper::Main", "CompareAliasCase" },
 	{ "Gatekeeper::Main", "CompareAliasType" },
@@ -184,6 +168,7 @@ static const char * KnowConfigEntries[][2] = {
 #ifdef hasIPV6
 	{ "Gatekeeper::Main", "EnableIPv6" },
 #endif
+	{ "Gatekeeper::Main", "EnableTTLRestrictions" },
 	{ "Gatekeeper::Main", "EncryptAllPasswords" },
 	{ "Gatekeeper::Main", "EndpointIDSuffix" },
 	{ "Gatekeeper::Main", "EndpointSignalPort" },
@@ -195,6 +180,7 @@ static const char * KnowConfigEntries[][2] = {
 	{ "Gatekeeper::Main", "Home" },
 	{ "Gatekeeper::Main", "ListenQueueLength" },
 	{ "Gatekeeper::Main", "MaxASNArraySize" },
+	{ "Gatekeeper::Main", "MaximumBandwidthPerCall" },
 	{ "Gatekeeper::Main", "MinimumBandwidthPerCall" },
 	{ "Gatekeeper::Main", "MulticastGroup" },
 	{ "Gatekeeper::Main", "MulticastPort" },
@@ -211,6 +197,7 @@ static const char * KnowConfigEntries[][2] = {
 	{ "Gatekeeper::Main", "TimestampFormat" },
 	{ "Gatekeeper::Main", "TotalBandwidth" },
 	{ "Gatekeeper::Main", "TraceLevel" },
+	{ "Gatekeeper::Main", "TTLExpireDropCall" },
 	{ "Gatekeeper::Main", "UnicastRasPort" },
 	{ "Gatekeeper::Main", "UseBroadcastListener" },
 	{ "Gatekeeper::Main", "UseMulticastListener" },
@@ -223,17 +210,23 @@ static const char * KnowConfigEntries[][2] = {
 	{ "GkH350::Settings", "ServerName" },
 	{ "GkH350::Settings", "ServerPort" },
 	{ "GkH350::Settings", "ServiceControl" },
+#ifdef hasLDAPStartTLS
 	{ "GkH350::Settings", "StartTLS" },
+#endif
 	{ "GkPresence::SQL", "CacheTimeout" },
 	{ "GkPresence::SQL", "Database" },
 	{ "GkPresence::SQL", "Driver" },
 	{ "GkPresence::SQL", "Host" },
+	{ "GkPresence::SQL", "IncrementalUpdate" },
 	{ "GkPresence::SQL", "Library" },
+	{ "GkPresence::SQL", "Password" },
 	{ "GkPresence::SQL", "MinPoolSize" },
 	{ "GkPresence::SQL", "QueryAdd" },
 	{ "GkPresence::SQL", "QueryDelete" },
 	{ "GkPresence::SQL", "QueryList" },
 	{ "GkPresence::SQL", "QueryUpdate" },
+	{ "GkPresence::SQL", "UpdateWorkerTimer" },
+	{ "GkPresence::SQL", "Username" },
 	{ "GkQoSMonitor", "CallEndOnly" },
 	{ "GkQoSMonitor", "DetailFile" },
 	{ "GkQoSMonitor", "Enable" },
@@ -244,9 +237,11 @@ static const char * KnowConfigEntries[][2] = {
 	{ "GkQoSMonitor::SQL", "Library" },
 	{ "GkQoSMonitor::SQL", "Password" },
 	{ "GkQoSMonitor::SQL", "Query" },
+	{ "GkQoSMonitor::SQL", "Username" },
 	{ "GkStatus::Filtering", "Enable" },
 	{ "GkStatus::Filtering", "ExcludeFilter" },
 	{ "GkStatus::Filtering", "IncludeFilter" },
+	{ "GkStatus::Filtering", "NewRCFOnly" },
 	{ "LogFile", "Filename" },
 	{ "LogFile", "Rotate" },
 	{ "LogFile", "RotateDay" },
@@ -269,6 +264,7 @@ static const char * KnowConfigEntries[][2] = {
 	{ "Proxy", "Enable" },
 	{ "Proxy", "EnableRTCPStats" },
 	{ "Proxy", "EnableRTPMute" },
+	{ "Proxy", "ExplicitRoutes" },
 	{ "Proxy", "InternalNetwork" },
 	{ "Proxy", "ProxyAlways" },
 	{ "Proxy", "ProxyForNAT" },
@@ -277,6 +273,8 @@ static const char * KnowConfigEntries[][2] = {
 	{ "Proxy", "RTPMultiplexPort" },
 	{ "Proxy", "RTCPMultiplexPort" },
 	{ "Proxy", "RTPPortRange" },
+	{ "Proxy", "RemoveMCInFastStartTransmitOffer" },
+	{ "Proxy", "SearchBothSidesOnCLC" },
 	{ "Proxy", "T120PortRange" },
 	{ "RadAcct", "AppendCiscoAttributes" },
 	{ "RadAcct", "DefaultAcctPort" },
@@ -307,6 +305,7 @@ static const char * KnowConfigEntries[][2] = {
 	{ "RadAliasAuth", "SharedSecret" },
 	{ "RadAliasAuth", "SocketDeleteTimeout" },
 	{ "RadAliasAuth", "UseDialedNumber" },
+	{ "RadAliasAuth", "EmptyUsername" },
 	{ "RadAuth", "AppendCiscoAttributes" },
 	{ "RadAuth", "DefaultAuthPort" },
 	{ "RadAuth", "IdCacheTimeout" },
@@ -338,46 +337,82 @@ static const char * KnowConfigEntries[][2] = {
 	{ "RasSrv::RRQFeatures", "AcceptEndpointIdentifier" },
 	{ "RasSrv::RRQFeatures", "AcceptGatewayPrefixes" },
 	{ "RasSrv::RRQFeatures", "AcceptMCUPrefixes" },
+	{ "RasSrv::RRQFeatures", "AccHTTPLink" },
 	{ "RasSrv::RRQFeatures", "AliasTypeFilter" },
 	{ "RasSrv::RRQFeatures", "IRQPollCount" },
 	{ "RasSrv::RRQFeatures", "OverwriteEPOnSameAddress" },
+	{ "RasSrv::RRQFeatures", "EnableAdditiveRegistration" },
 	{ "RasSrv::RRQFeatures", "SupportDynamicIP" },
 	{ "RoutedMode", "AcceptNeighborCalls" },
 	{ "RoutedMode", "AcceptNeighborsCalls" },
 	{ "RoutedMode", "AcceptUnregisteredCalls" },
 	{ "RoutedMode", "ActivateFailover" },
 	{ "RoutedMode", "AlertingTimeout" },
+	{ "RoutedMode", "AlwaysRewriteSourceCallSignalAddress" },
 	{ "RoutedMode", "AutoProxyIPv4ToIPv6Calls" },
 	{ "RoutedMode", "CallSignalHandlerNumber" },
 	{ "RoutedMode", "CallSignalPort" },
+	{ "RoutedMode", "CalledTypeOfNumber" },
+	{ "RoutedMode", "CallingTypeOfNumber" },
 	{ "RoutedMode", "CpsCheckInterval" },
 	{ "RoutedMode", "CpsLimit" },
+	{ "RoutedMode", "DisableH245Tunneling" },
 	{ "RoutedMode", "DisableRetryChecks" },
 	{ "RoutedMode", "DropCallsByReleaseComplete" },
 	{ "RoutedMode", "ENUMservers" },
-    { "RoutedMode", "EnableH235Media" },
+#ifdef HAS_H235_MEDIA
+	{ "RoutedMode", "EnableH235HalfCallMedia" },
+#endif
 	{ "RoutedMode", "EnableH450.2" },
+#ifdef HAS_H46017
+	{ "RoutedMode", "EnableH46017" },
+#endif
+#ifdef HAS_H46018
 	{ "RoutedMode", "EnableH46018" },
+#endif
+#ifdef HAS_H46023
 	{ "RoutedMode", "EnableH46023" },
+#endif
+#ifdef HAS_H46026
+	{ "RoutedMode", "EnableH46026" },
+#endif
+#ifdef HAS_H460P
+	{ "RoutedMode", "EnableH460P" },
+#endif
 	{ "RoutedMode", "FailoverCauses" },
 	{ "RoutedMode", "ForceNATKeepAlive" },
 	{ "RoutedMode", "ForwardOnFacility" },
 	{ "RoutedMode", "GKRouted" },
 	{ "RoutedMode", "GenerateCallProceeding" },
 	{ "RoutedMode", "GkRouted" },
+#ifdef HAS_H235_MEDIA
+	{ "RoutedMode", "H235HalfCallMediaStrength" },
+#endif
 	{ "RoutedMode", "H245PortRange" },
 	{ "RoutedMode", "H245Routed" },
+	{ "RoutedMode", "H245TunnelingTranslation" },
 	{ "RoutedMode", "H4502EmulatorTransferMethod" },
+#ifdef HAS_H46018
 	{ "RoutedMode", "H46018NoNat" },
+#endif
+#ifdef HAS_H46023
 	{ "RoutedMode", "H46023PublicIP" },
 	{ "RoutedMode", "H46023SignalGKRouted" },
 	{ "RoutedMode", "H46023STUN" },
+	{ "RoutedMode", "H46024ForceDirect" },
+#endif
 	{ "RoutedMode", "NATStdMin" },
 	{ "RoutedMode", "Q931PortRange" },
+	{ "RoutedMode", "RDSservers" },
 	{ "RoutedMode", "RemoveCallOnDRQ" },
+	{ "RoutedMode", "RemoveFaxUDPOptionsFromRM" },
 	{ "RoutedMode", "RemoveH235Call" },
 	{ "RoutedMode", "RemoveH245AddressOnTunneling" },
 	{ "RoutedMode", "RemoveH460Call" },
+	{ "RoutedMode", "RemoveSorensonSourceInfo" },
+#ifdef HAS_H235_MEDIA
+	{ "RoutedMode", "RequireH235HalfCallMedia" },
+#endif
 	{ "RoutedMode", "RtpHandlerNumber" },
 	{ "RoutedMode", "ScreenCallingPartyNumberIE" },
 	{ "RoutedMode", "ScreenDisplayIE" },
@@ -394,27 +429,56 @@ static const char * KnowConfigEntries[][2] = {
 	{ "RoutedMode", "TranslateReceivedQ931Cause" },
 	{ "RoutedMode", "TreatUnregisteredNAT" },
 	{ "RoutedMode", "TranslateSentQ931Cause" },
+	{ "RoutedMode", "TranslateSorensonSourceInfo" },
 	{ "RoutedMode", "UseProvisionalRespToH245Tunneling" },
 	{ "Routing::CatchAll", "CatchAllAlias" },
 	{ "Routing::CatchAll", "CatchAllIP" },
 	{ "Routing::DNS", "ResolveNonLocalLRQ" },
 	{ "Routing::ENUM", "ResolveLRQ" },
+	{ "Routing::Forwarding", "CacheTimeout" },
+	{ "Routing::Forwarding", "Database" },
+	{ "Routing::Forwarding", "Driver" },
+	{ "Routing::Forwarding", "Host" },
+	{ "Routing::Forwarding", "Library" },
+	{ "Routing::Forwarding", "MinPoolSize" },
+	{ "Routing::Forwarding", "Password" },
+	{ "Routing::Forwarding", "Query" },
+	{ "Routing::Forwarding", "Username" },
+#ifdef HAS_LUA
+	{ "Routing::Lua", "Script" },
+	{ "Routing::Lua", "ScriptFile" },
+#endif
 	{ "Routing::RDS", "ResolveLRQ" },
 	{ "Routing::SRV", "ResolveNonLocalLRQ" },
+	{ "Routing::Sql", "CacheTimeout" },
 	{ "Routing::Sql", "Database" },
 	{ "Routing::Sql", "Driver" },
+	{ "Routing::Sql", "EnableRegexRewrite" },
 	{ "Routing::Sql", "Host" },
 	{ "Routing::Sql", "Library" },
 	{ "Routing::Sql", "MinPoolSize" },
 	{ "Routing::Sql", "Password" },
 	{ "Routing::Sql", "Query" },
 	{ "Routing::Sql", "Username" },
+#ifdef HAS_SNMP
+	{ "SNMP", "AllowRequestsFrom" },
+	{ "SNMP", "AgentListenIP" },
+	{ "SNMP", "AgentListenPort" },
+	{ "SNMP", "EnableSNMP" },
+	{ "SNMP", "Implementation" },
+	{ "SNMP", "Standalone" },
+	{ "SNMP", "TrapCommunity" },
+	{ "SNMP", "TrapHost" },
+#endif
+	{ "SQLAcct", "CacheTimeout" },
 	{ "SQLAcct", "AlertQuery" },
 	{ "SQLAcct", "Database" },
 	{ "SQLAcct", "Driver" },
 	{ "SQLAcct", "Host" },
 	{ "SQLAcct", "Library" },
 	{ "SQLAcct", "MinPoolSize" },
+	{ "SQLAcct", "OffQuery" },
+	{ "SQLAcct", "OnQuery" },
 	{ "SQLAcct", "Password" },
 	{ "SQLAcct", "RegisterQuery" },
 	{ "SQLAcct", "StartQuery" },
@@ -446,12 +510,14 @@ static const char * KnowConfigEntries[][2] = {
 	{ "SQLAuth", "Password" },
 	{ "SQLAuth", "RegQuery" },
 	{ "SQLAuth", "Username" },
+	{ "SQLConfig", "CacheTimeout" },
 	{ "SQLConfig", "ConfigQuery" },
 	{ "SQLConfig", "Database" },
 	{ "SQLConfig", "Driver" },
 	{ "SQLConfig", "GWPrefixesQuery" },
 	{ "SQLConfig", "Host" },
 	{ "SQLConfig", "Library" },
+	{ "SQLConfig", "MinPoolSize" },
 	{ "SQLConfig", "NeighborsQuery" },
 	{ "SQLConfig", "Password" },
 	{ "SQLConfig", "PermanentEndpointsQuery" },
@@ -477,6 +543,8 @@ static const char * KnowConfigEntries[][2] = {
 	{ "SyslogAcct", "ConnectEvent" },
 	{ "SyslogAcct", "StartEvent" },
 	{ "SyslogAcct", "StopEvent" },
+	{ "SyslogAcct", "SyslogFacility" },
+	{ "SyslogAcct", "SyslogLevel" },
 	{ "SyslogAcct", "TimestampFormat" },
 	{ "SyslogAcct", "UpdateEvent" },
 
@@ -484,7 +552,12 @@ static const char * KnowConfigEntries[][2] = {
 	{ "EP::", "CalledTypeOfNumber" },
 	{ "EP::", "CallingTypeOfNumber" },
 	{ "EP::", "Capacity" },
+#ifdef HAS_H46017
+	{ "EP::", "DisableH46017" },
+#endif
+#ifdef HAS_H46018
 	{ "EP::", "DisableH46018" },
+#endif
 	{ "EP::", "GatewayPrefixes" },
 	{ "EP::", "GatewayPriority" },
 	{ "EP::", "MaxBandwidth" },
@@ -500,7 +573,9 @@ static const char * KnowConfigEntries[][2] = {
 	{ "Neighbor::", "ForwardLRQ" },
 	{ "Neighbor::", "ForwardResponse" },
 	{ "Neighbor::", "GatekeeperIdentifier" },
+#ifdef HAS_H46018
 	{ "Neighbor::", "H46018Client" },
+#endif
 	{ "Neighbor::", "Host" },
 	{ "Neighbor::", "Password" },
 	{ "Neighbor::", "SendAuthUser" },
@@ -513,6 +588,7 @@ static const char * KnowConfigEntries[][2] = {
 	{ "FileIPAuth", "*" },
 	{ "GkStatus::Auth", "*" },
 	{ "H225toQ931", "*" },
+	{ "H350PasswordAuth", "*" },
 	{ "ModeSelection", "*" },
 	{ "NATedEndpoints", "*" },
 	{ "PrefixAuth", "*" },
@@ -536,10 +612,44 @@ static const char * KnowConfigEntries[][2] = {
 	{ "RoutingPolicy::OnLRQ", "*" },
 	{ "RoutingPolicy::OnSetup", "*" },
 	{ "SimplePasswordAuth", "*" },
-
 	{ NULL }	// the end
 };
 
+namespace { // keep the global objects private
+
+PTimedMutex ReloadMutex;
+
+#ifndef _WIN32
+PString pidfile("/var/run/gnugk.pid");
+#endif
+
+void ShutdownHandler()
+{
+	Gatekeeper::EnableLogFileRotation(false);
+	// delete singleton objects
+	PTRACE(3, "GK\tDeleting global reference tables");
+
+	Job::StopAll();
+	if (CapacityControl::InstanceExists())
+		delete CapacityControl::Instance();
+	if (CallTable::InstanceExists())
+		delete CallTable::Instance();
+	if (RegistrationTable::InstanceExists())
+		delete RegistrationTable::Instance();
+	if (RasServer::InstanceExists())
+		delete RasServer::Instance();
+	if (MakeCallEndPoint::InstanceExists())
+		delete MakeCallEndPoint::Instance();
+	if (Toolkit::InstanceExists())
+		delete Toolkit::Instance();
+#if defined(HAS_SNMP)
+	DeleteSNMPAgent();
+#endif
+	PTRACE(3, "GK\tdelete ok");
+
+	Gatekeeper::CloseLogFile();
+}
+
 bool CheckConfig(PConfig * cfg, const PString & mainsection)
 {
 	unsigned warnings = 0;
@@ -557,6 +667,10 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 		if (sect.Left(10) == "Neighbor::") {
 			sect = "Neighbor::";
 		}
+		// strip off instance ID for routing sections
+		if (sect.Left(9) == "Routing::" && sect.Find("::", 9) != P_MAX_INDEX) {
+			sect = sect.Left(sect.Find("::", 9));
+		}
 		if (sect == mainsection) {
 			mainsectionfound = true;
 		}
@@ -564,16 +678,18 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 		unsigned j = 0;
 		bool found = false;
 		bool section_checkable = true;
-		while ((ks = KnowConfigEntries[j][0])) {
+		while ((ks = KnownConfigEntries[j][0])) {
 			if (sect == ks) {
 				found = true;
-				section_checkable = (PString(KnowConfigEntries[j][1]) != "*");
+				section_checkable = (PString(KnownConfigEntries[j][1]) != "*");
 				break;
 			}
 			j++;
 		}
 		if (!found) {
+			cerr << "WARNING: Config section [" << sect << "] unknown" << endl;
 			PTRACE(0, "WARNING: Config section [" << sect << "] unknown");
+			SNMP_TRAP(7, SNMPError, Configuration, "Config section [" + sect + "] unknown");
 			warnings++;
 		} else if (!section_checkable) {
 			//PTRACE(0, "Section " << sect << " can't be checked in detail");
@@ -584,12 +700,12 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 				PCaselessString key = entries.GetKeyAt(j);
 				PString value = entries.GetDataAt(j);
 				if (value.IsEmpty()) {
-					PTRACE(0, "WARNING: Empty switch: [" << sect << "] " << key << "=");
+					PTRACE(2, "WARNING: Empty entry: [" << sect << "] " << key << "=");
 				}
 				unsigned k = 0;
 				bool entry_found = false;
-				while ((ks = KnowConfigEntries[k][0])) {
-					const char * ke = KnowConfigEntries[k][1];
+				while ((ks = KnownConfigEntries[k][0])) {
+					const char * ke = KnownConfigEntries[k][1];
 					k++;
 					if ((sect == ks) && (key == ke)) {
 						entry_found = true;
@@ -597,7 +713,9 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 					}
 				}
 				if (!entry_found) {
+					cerr << "WARNING: Config entry [" << sect << "] " << key << "=" << value << " unknown" << endl;
 					PTRACE(0, "WARNING: Config entry [" << sect << "] " << key << "=" << value << " unknown");
+					SNMP_TRAP(7, SNMPError, Configuration, "Config entry [" + sect + "] " + key + " unknown");
 					warnings++;
 				}
 			}
@@ -605,6 +723,7 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 	}
 	if (!mainsectionfound) {
 		PTRACE(0, "WARNING: This doesn't look like a GNU Gatekeeper configuration file!");
+		SNMP_TRAP(7, SNMPError, Configuration, "No config file");
 	}
 
 	return (warnings == 0);
@@ -615,15 +734,11 @@ bool CheckConfig(PConfig * cfg, const PString & mainsection)
 // or we get core dump
 void ExitGK()
 {
-#if PTRACING
 	Gatekeeper::EnableLogFileRotation(false);
-#endif
 
 	delete Toolkit::Instance();
 
-#if PTRACING
 	Gatekeeper::CloseLogFile();
-#endif
 	exit(0);
 }
 
@@ -634,7 +749,11 @@ void ReloadHandler()
 	Gatekeeper::ReopenLogFile();
 
 	// only one thread must do this
+#if PTLIB_VER < 2120
 	if (ReloadMutex.WillBlock())
+#else
+	if (!ReloadMutex.Try())
+#endif
 		return;
 
 	/*
@@ -679,6 +798,8 @@ void ReloadHandler()
 	*/
 	PTRACE(3, "GK\tCarry on current calls.");
 
+	SNMP_TRAP(3, SNMPInfo, General, "Full config reloaded");
+
 	/*
 	** Leave critical Section
 	*/
@@ -694,9 +815,7 @@ BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 
 	if( dwCtrlType == CTRL_LOGOFF_EVENT ) {
 		eventName = "CTRL_LOGOFF_EVENT";
-#if PTRACING
-		PTRACE(2,"GK\tGatekeeper received " <<eventName);
-#endif
+		PTRACE(2, "GK\tGatekeeper received " << eventName);
 		// prevent shut down
 		return FALSE;
 	}
@@ -710,8 +829,8 @@ BOOL WINAPI WinCtrlHandlerProc(DWORD dwCtrlType)
 	else if( dwCtrlType == CTRL_SHUTDOWN_EVENT )
 		eventName = "CTRL_SHUTDOWN_EVENT";
 
-#if PTRACING
-	PTRACE(1,"GK\tGatekeeper shutdown due to "<<eventName);
+#ifndef hasPTLibTraceOnShutdownBug
+	PTRACE(1, "GK\tGatekeeper shutdown due to "<<eventName);
 #endif
 
 	PWaitAndSignal shutdown(ShutdownMutex);
@@ -783,10 +902,6 @@ bool Gatekeeper::SetUserAndGroup(const PString & username)
 
 void UnixShutdownHandler(int sig)
 {
-//#ifdef P_MACOSX
-//	RasServer::Instance()->Stop();
-//	_exit(0);
-//#endif
 	if (ShutdownMutex.WillBlock() || !RasServer::Instance()->IsRunning())
 		return;
 	PWaitAndSignal shutdown(ShutdownMutex);
@@ -835,6 +950,10 @@ Gatekeeper::Gatekeeper(const char * _manuf,
 	GetArguments().SetArgs("-d -p " + pidfile);
 #endif
 #endif
+#if (PTLIB_VER >= 2100) && (PTLIB_VER < 2120)
+	// work around a bug in PTLib 2.10.x that doesn't start the housekeeping thread to delete auto-delete threads
+	SignalTimerChange();
+#endif
 }
 
 #ifdef COMPILE_AS_SERVICE
@@ -890,16 +1009,13 @@ const PString Gatekeeper::GetArgumentsParseString() const
 		("r-routed."
 		 "-h245routed."
 		 "d-direct."
-		 "i-interface:"
 		 "l-timetolive:"
 		 "b-bandwidth:"
 #ifdef HAS_SETUSERNAME
 		 "u-user:"
 #endif
-#if PTRACING
 		 "t-trace."
 		 "o-output:"
-#endif
 		 "c-config:"
 		 "s-section:"
 		 "-pid:"
@@ -959,7 +1075,6 @@ bool Gatekeeper::InitHandlers(const PArgList & args)
 
 bool Gatekeeper::InitLogging(const PArgList & args)
 {
-#if PTRACING
 	// Syslog is the default when compiled as service, but we don't want that
 	PTrace::ClearOptions(PTrace::SystemLogStream);
 	PTrace::SetOptions(PTrace::DateAndTime | PTrace::TraceLevel | PTrace::FileAndLine);
@@ -971,7 +1086,6 @@ bool Gatekeeper::InitLogging(const PArgList & args)
 			return FALSE;
 		}
 	}
-#endif
 
 	return TRUE;
 }
@@ -1010,16 +1124,13 @@ void Gatekeeper::PrintOpts()
 		"  -r  --routed       : Use gatekeeper routed call signaling\n"
 		"  -rr --h245routed   : Use H.245 control channel routed\n"
 		"  -d  --direct       : Use direct endpoint call signaling\n"
-		"  -i  --interface IP : The IP that the gatekeeper listen to\n"
 		"  -l  --timetolive n : Time to live for client registration\n"
 		"  -b  --bandwidth n  : Specify the total bandwidth\n"
 #ifdef HAS_SETUSERNAME
 		"  -u  --user name    : Run as this user\n"
 #endif
-#if PTRACING
 		"  -t  --trace        : Set trace verbosity\n"
 		"  -o  --output file  : Write trace to this file\n"
-#endif
 		"  -c  --config file  : Specify which config file to use\n"
 		"  -s  --section sec  : Specify which main section to use in the config file\n"
 		"      --pid file     : Specify the pid file\n"
@@ -1086,13 +1197,9 @@ void Gatekeeper::Main()
 		ExitGK();
 	}
 
-	if (args.HasOption('i'))
-		Toolkit::Instance()->SetGKHome(args.GetOptionString('i').Lines());
-
 	if (!InitConfig(args) || !InitHandlers(args))
 		ExitGK();
 
-
 	// set trace level + output file from config , if not set on the command line (for service)
 	PString fake_cmdline;
 	if (args.GetOptionCount('t') == 0) {
@@ -1106,6 +1213,8 @@ void Gatekeeper::Main()
 			fake_cmdline += " -o " + log_trace_file;
 	}
 	if (!fake_cmdline.IsEmpty()) {
+		for (int t=0; t < args.GetOptionCount('t'); t++)
+			fake_cmdline += " -t";
 		PArgList fake_args(fake_cmdline);
 		fake_args.Parse(GetArgumentsParseString());
 		InitLogging(fake_args);
@@ -1141,6 +1250,7 @@ void Gatekeeper::Main()
 
 	// Copyright notice
 	cout <<
+		"For documentation and updates please visit http://www.gnugk.org/.\n\n"
 		"This program is free software; you can redistribute it and/or\n"
 		"modify it under the terms of the GNU General Public License version 2.\n"
 		"We also explicitly grant the right to link this code\n"
@@ -1188,7 +1298,7 @@ void Gatekeeper::Main()
 	else if (args.HasOption('d'))
 		RasSrv->SetRoutedMode(false, false);
 #ifdef HAS_H235_MEDIA
-    else if (Toolkit::Instance()->IsH235MediaEnabled()) 
+    else if (Toolkit::Instance()->IsH235HalfCallMediaEnabled()) 
         RasSrv->SetRoutedMode(true, true);
 #endif 
 	else 
@@ -1200,6 +1310,11 @@ void Gatekeeper::Main()
 	// Load RDS servers
 	RasSrv->SetRDSServers();
 
+#ifdef HAS_SNMP
+	if (SelectSNMPImplementation() == "PTLib")
+		SNMP_TRAP(1, SNMPInfo, General, "GnuGk started");	// when NOT registering as agent, send started trap here already
+#endif
+
 #if defined(_WIN32)
 	// 1) prevent CTRL_CLOSE_EVENT, CTRL_LOGOFF_EVENT and CTRL_SHUTDOWN_EVENT
 	//    dialog box from being displayed.
@@ -1217,9 +1332,7 @@ void Gatekeeper::Main()
 	// graceful shutdown
 	cerr << "\nShutting down gatekeeper . . . ";
 
-#if !defined(_WIN32) || PTLIB_VER < 2102
-	ShutdownHandler();    // <-- on Windows PTLIB 2.10.2 and 2.11.x shuting down worker threads segfaults
-#endif
+	ShutdownHandler();
 	cerr << "done\n";
 
 #ifdef _WIN32
@@ -1229,7 +1342,6 @@ void Gatekeeper::Main()
 #endif // _WIN32
 }
 
-#if PTRACING
 namespace {
 const char* const logConfigSectionName = "Logfile";
 }
@@ -1514,7 +1626,7 @@ bool Gatekeeper::RotateLogFile()
 
 	m_logFile->SetPosition(0, PFile::End);
 	PTrace::SetStream(m_logFile);
-	PTRACEX(1, "GK\tLogging restarted.");
+	PTRACEX(1, "GK\tLogging restarted\n" + Toolkit::GKVersion());
 	return true;
 }
 
@@ -1554,7 +1666,7 @@ bool Gatekeeper::ReopenLogFile()
 	}
 	m_logFile->SetPosition(0, PFile::End);
 	PTrace::SetStream(m_logFile);
-	PTRACEX(1, "GK\tLogging restarted");
+	PTRACEX(1, "GK\tLogging restarted\n" + Toolkit::GKVersion());
 	return true;
 }
 
@@ -1571,4 +1683,3 @@ void Gatekeeper::CloseLogFile()
 #endif
 	m_logFile = NULL;
 }
-#endif // PTRACING
