@@ -9,12 +9,12 @@
 // We also explicitly grant the right to link this code
 // with the OpenH323/H323Plus and OpenSSL library.
 //
+
 //////////////////////////////////////////////////////////////////
 
 
 #ifndef GK_H
-#define GK_H "@(#) $Id: gk.h,v 1.38 2011/09/18 23:08:53 willamowius Exp $"
-
+#define GK_H "@(#) $Id: gk.h,v 1.43 2012/05/23 08:57:32 shorne Exp $"
 #include "config.h"
 #include "version.h"
 #ifdef COMPILE_AS_SERVICE
@@ -30,6 +30,7 @@
 #endif
 
 extern PSemaphore ShutdownMutex;
+extern const char * KnownConfigEntries[][2];
 
 class GkTimer;
 
@@ -62,7 +63,6 @@ class Gatekeeper : public PProcess
 	virtual void OnControl();
 #endif
 
-#if PTRACING
 	enum RotationIntervals {
 		Hourly,
 		Daily,
@@ -74,6 +74,7 @@ class Gatekeeper : public PProcess
 	static bool SetLogFilename(
 		const PString& filename
 		);
+
 		
 	static bool RotateLogFile();
 	static bool ReopenLogFile();	
@@ -88,7 +89,6 @@ class Gatekeeper : public PProcess
 	static void RotateOnTimer(
 		GkTimer * timer /// timer object that triggered rotation
 		);
-#endif // PTRACING
 
  protected:
 	/** returns the template string for which the cmommand line is parsed */
@@ -99,13 +99,14 @@ class Gatekeeper : public PProcess
 	 * before the fun starts.
 	 * Every method may return #FALSE# to abort #Main# and end the program.
 	 */
+
 	//@{
 
 	/** installs the signal handlers; First called init method. */
 	virtual bool InitHandlers(const PArgList & args);
-
+	
 	virtual bool InitConfig(const PArgList & args);
-
+	
 	/** initiates logging and tracing; Called after #InitConfig# */
 	virtual bool InitLogging(const PArgList & args);
 
@@ -116,20 +117,16 @@ class Gatekeeper : public PProcess
 		The group that will be set is the user's default group.
 	*/
 	virtual bool SetUserAndGroup(const PString & username);
-
 	//@}
 
 private:
-#if PTRACING
 	/// parse rotation interval from the config
 	static void GetRotateInterval(
 		PConfig & cfg, /// the config
 		const PString & section /// name of the config section to check
 		);
-#endif
 
 private:
-#if PTRACING
 	/// rotate file after the specified period of time (if >= 0)
 	static int m_rotateInterval;
 	/// a minute when the interval based rotation should occur
@@ -148,12 +145,11 @@ private:
 	static PMutex m_logFileMutex;
 	/// human readable names for rotation intervals
 	static const char* const m_intervalNames[];
-#endif // PTRACING
 
 #ifdef COMPILE_AS_SERVICE
 	PString savedArguments;
 #endif
 
 };
-
 #endif // GK_H
+
