@@ -5,7 +5,7 @@
  * support for accounting to the gatekeeper.
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
- * Copyright (c) 2005-2010, Jan Willamowius
+ * Copyright (c) 2005-2012, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -21,6 +21,7 @@
 #include "stl_supp.h"
 #include "Toolkit.h"
 #include "gktimer.h"
+#include "snmp.h"
 #include "gkacct.h"
 
 using std::find;
@@ -72,7 +73,7 @@ GkAcctLogger::GkAcctLogger(
 
 GkAcctLogger::~GkAcctLogger()
 {
-	PTRACE(1,"GKACCT\tDestroyed module "<<GetName());
+	PTRACE(1, "GKACCT\tDestroyed module "<<GetName());
 }
 
 int GkAcctLogger::GetEvents(
@@ -191,17 +192,17 @@ void GkAcctLogger::SetupAcctParams(
 	params["Dialed-Number"] = GetDialedNumber(call);
 
 	endptr caller;
-	if (caller = call->GetCallingParty()) {
+	if ((caller = call->GetCallingParty())) {
 		params["caller-epid"] = caller->GetEndpointIdentifier().GetValue();
 	}
 	endptr callee;
-	if (callee = call->GetCalledParty()) {
+	if ((callee = call->GetCalledParty())) {
 		params["callee-epid"] = callee->GetEndpointIdentifier().GetValue();
 	}
 	params["call-attempts"] = PString(call->GetNoCallAttempts());
 	params["last-cdr"] = call->GetNoRemainingRoutes() > 0 ? "0" : "1";
 
-	if (call->GetMediaOriginatingIp(addr))
+	if ((call->GetMediaOriginatingIp(addr)))
 		params["media-oip"] = addr.AsString();
 	params["codec"] = call->GetCodec();
 	params["bandwidth"] = call->GetBandwidth();
@@ -223,10 +224,18 @@ void GkAcctLogger::SetupAcctEndpointParams(
 		params["endpoint-ip"] = addr.AsString();
 		params["endpoint-port"] = port;
 	}
-	
-	params["aliases"] = AsString(ep->GetAliases(), false);
-	params["aliases"].Replace(PString("="), PString(","), true);	// make list comma separated
+
+	PString aliasString = AsString(ep->GetAliases(), false);
+	aliasString.Replace(PString("="), PString(","), true);	// make list comma separated
+	params["aliases"] = aliasString;
+
+	// The username is always the last in the Alias List
+	PStringArray aliasList = aliasString.Tokenise(",");
+    if (aliasList.GetSize() > 0)
+	    params["u"] = aliasList[aliasList.GetSize()-1];
+
 	params["epid"] = ep->GetEndpointIdentifier().GetValue();
+	params["g"] = Toolkit::GKName();
 }
 
 // avoid warning in PTLib object.h
@@ -735,14 +744,13 @@ GkAcctLogger::Status FileAcct::Log(
 		} else
 			PTRACE(1, "GKACCT\t" << GetName() << " - write CDR text for event "
 				<< evt << ", call no. " << call->GetCallNumber()
-				<< " failed: " << m_cdrFile->GetErrorText()
-				);
+				<< " failed: " << m_cdrFile->GetErrorText());
 	} else
 		PTRACE(1, "GKACCT\t" << GetName() << " - write CDR text for event "
 			<< evt << ", for call no. " << call->GetCallNumber()
-			<< " failed: CDR file is closed"
-			);
-		
+			<< " failed: CDR file is closed");
+
+	SNMP_TRAP(6, SNMPError, Accounting, GetName() + " failed");
 	return Fail;
 }
 
@@ -816,9 +824,10 @@ void FileAcct::Rotate()
 		if (!PFile::Rename(fn, fn.GetFileName() + PTime().AsString(".yyyyMMdd-hhmmss"))) {
 			PTRACE(1, "GKACCT\t" << GetName() << " rotate failed - could not "
 				"rename the log file");
+			SNMP_TRAP(6, SNMPError, Accounting, GetName() + " failed");
 		}
 	}
-	
+
 	m_cdrFile = OpenCDRFile(fn);
 	m_cdrLines = 0;
 }
@@ -851,7 +860,7 @@ GkAcctLoggerList::GkAcctLoggerList()
 {
 	// should not be less than 10 seconds
 	if (m_acctUpdateInterval)
-		m_acctUpdateInterval = PMAX(10, m_acctUpdateInterval);
+		m_acctUpdateInterval = PMAX((long)10, m_acctUpdateInterval);
 }
 
 GkAcctLoggerList::~GkAcctLoggerList()
@@ -867,7 +876,7 @@ void GkAcctLoggerList::OnReload()
 		);
 	// should not be less than 10 seconds
 	if (m_acctUpdateInterval)
-		m_acctUpdateInterval = PMAX(10, m_acctUpdateInterval);
+		m_acctUpdateInterval = PMAX((long)10, m_acctUpdateInterval);
 
 	DeleteObjectsInContainer(m_loggers);
 	m_loggers.clear();
@@ -910,7 +919,6 @@ bool GkAcctLoggerList::LogAcctEvent(
 		switch (status)
 		{
 		case GkAcctLogger::Ok:
-#if PTRACING
 			if (PTrace::CanTrace(3)) {
 				ostream& strm = PTrace::Begin(3,__FILE__,__LINE__);
 				strm << "GKACCT\t" << logger->GetName() << " logged event " << evt;
@@ -918,20 +926,17 @@ bool GkAcctLoggerList::LogAcctEvent(
 					strm << " for call no. " << call->GetCallNumber();
 				PTrace::End(strm);
 			}
-#endif
 			break;
 			
 		default:
-#if PTRACING
 			if (PTrace::CanTrace(3)) {
 				ostream& strm = PTrace::Begin(3, __FILE__, __LINE__);
-				strm << "GKACCT\t" << logger->GetName() << " failed to log event "
-					<< evt;
+				strm << "GKACCT\t" << logger->GetName() << " failed to log event " << evt;
+				SNMP_TRAP(7, SNMPError, Accounting, logger->GetName() + " failed");
 				if (call)
 					strm << " for call no. " << call->GetCallNumber();
 				PTrace::End(strm);
 			}
-#endif
 			// required and sufficient rules always determine 
 			// status of the request
 			if (logger->GetControlFlag() == GkAcctLogger::Required
@@ -950,7 +955,6 @@ bool GkAcctLoggerList::LogAcctEvent(
 	if (finalResult && status != GkAcctLogger::Ok)
 		finalResult = false;
 		
-#if PTRACING
 	if (PTrace::CanTrace(2)) {
 		ostream& strm = PTrace::Begin(2, __FILE__, __LINE__);
 		strm << "GKACCT\t" << (finalResult ? "Successfully logged event " 
@@ -958,8 +962,9 @@ bool GkAcctLoggerList::LogAcctEvent(
 		if (call)
 			strm << " for call no. " << call->GetCallNumber();
 		PTrace::End(strm);
+		if (!finalResult)
+			SNMP_TRAP(7, SNMPError, Accounting, "Failed to log event " + evt);
 	}
-#endif
 	return finalResult;
 }
 
@@ -982,7 +987,6 @@ bool GkAcctLoggerList::LogAcctEvent(
 		switch (status)
 		{
 		case GkAcctLogger::Ok:
-#if PTRACING
 			if (PTrace::CanTrace(3)) {
 				ostream& strm = PTrace::Begin(3,__FILE__,__LINE__);
 				strm << "GKACCT\t" << logger->GetName() << " logged event " << evt;
@@ -990,20 +994,17 @@ bool GkAcctLoggerList::LogAcctEvent(
 					strm << " for endpoint " << ep->GetEndpointIdentifier().GetValue();
 				PTrace::End(strm);
 			}
-#endif
 			break;
 			
 		default:
-#if PTRACING
 			if (PTrace::CanTrace(3)) {
 				ostream& strm = PTrace::Begin(3, __FILE__, __LINE__);
-				strm << "GKACCT\t" << logger->GetName() << " failed to log event "
-					<< evt;
+				strm << "GKACCT\t" << logger->GetName() << " failed to log event " << evt;
 				if (ep)
 					strm << " for endpoint " << ep->GetEndpointIdentifier().GetValue();
 				PTrace::End(strm);
+				SNMP_TRAP(7, SNMPError, Accounting, logger->GetName() + " failed to log event " + PString(evt));
 			}
-#endif
 			// required and sufficient rules always determine 
 			// status of the request
 			if (logger->GetControlFlag() == GkAcctLogger::Required
@@ -1022,7 +1023,6 @@ bool GkAcctLoggerList::LogAcctEvent(
 	if (finalResult && status != GkAcctLogger::Ok)
 		finalResult = false;
 		
-#if PTRACING
 	if (PTrace::CanTrace(2)) {
 		ostream& strm = PTrace::Begin(2, __FILE__, __LINE__);
 		strm << "GKACCT\t" << (finalResult ? "Successfully logged event " 
@@ -1030,8 +1030,9 @@ bool GkAcctLoggerList::LogAcctEvent(
 		if (ep)
 			strm << " for endpoint " << ep->GetEndpointIdentifier().GetValue();
 		PTrace::End(strm);
+		if (!finalResult)
+			SNMP_TRAP(7, SNMPError, Accounting, "Failed to log event " + evt);
 	}
-#endif
 	return finalResult;
 }
 
