@@ -2,7 +2,7 @@
 //
 // gkauth.cxx
 //
-// Copyright (c) 2001-2011, Jan Willamowius
+// Copyright (c) 2001-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -45,9 +45,9 @@ using std::greater;
 
 ARQAuthData::ARQAuthData(
 	/// an endpoint requesting admission
-	const endptr& ep,
+	const endptr & ep,
 	/// call record matching this ARQ (if any)
-	const callptr& call
+	const callptr & call
 	) : m_rejectReason(-1), m_callDurationLimit(-1), 
 	m_requestingEP(ep),	m_call(call), m_billingMode(-1),
 	m_proxyMode(CallRec::ProxyDetect),
@@ -56,7 +56,7 @@ ARQAuthData::ARQAuthData(
 }
 
 ARQAuthData::ARQAuthData(
-	const ARQAuthData& obj
+	const ARQAuthData & obj
 	) : m_rejectReason(obj.m_rejectReason),
 	m_callDurationLimit(obj.m_callDurationLimit), 
 	m_requestingEP(obj.m_requestingEP), m_call(obj.m_call), 
@@ -231,7 +231,6 @@ GkAuthenticator::GkAuthenticator(
 		}
 	}
 
-#if PTRACING
 	// convert bit flags to human readable names
 	PString rasFlagsStr, miscFlagsStr;
 	
@@ -261,9 +260,7 @@ GkAuthenticator::GkAuthenticator(
 		miscFlagsStr = "NONE";
 	
 	PTRACE(1, "GKAUTH\t" << GetName() << " rule added to check RAS: "
-		<< rasFlagsStr << ", OTHER: " << miscFlagsStr
-		);
-#endif	
+		<< rasFlagsStr << ", OTHER: " << miscFlagsStr);
 
 	m_h235Authenticators = NULL;
 }
@@ -642,24 +639,28 @@ GkAuthenticatorList::GkAuthenticatorList()
 #endif
 {
 #ifdef OpenH323Factory
-	PStringList authlist = Toolkit::Instance()->GetAuthenticatorList();
 
 	PFactory<H235Authenticator>::KeyList_T keyList = PFactory<H235Authenticator>::GetKeyList();
 	PFactory<H235Authenticator>::KeyList_T::const_iterator r;
-	for (r = keyList.begin(); r != keyList.end(); ++r) {
-		H235Authenticator * Auth = PFactory<H235Authenticator>::CreateInstance(*r);
-		if (authlist.GetSize() > 0) {
-			for (PINDEX i=0; i< authlist.GetSize(); i++) {
-				if (PString(Auth->GetName()) == authlist[i]) {
-					PTRACE(4,"GKAUTH\tLoaded Authenticator " << Auth->GetName() << " from Policy");
-					m_h235authenticators.Append(Auth);
-				}
+
+	// if a global list of autenticators is configured, use it in the priority order supplied
+	PStringList authlist = Toolkit::Instance()->GetAuthenticatorList();
+	if (authlist.GetSize() > 0) {
+		for (PINDEX i = 0; i < authlist.GetSize(); ++i) {
+			for (r = keyList.begin(); r != keyList.end(); ++r) {
+			H235Authenticator * Auth = PFactory<H235Authenticator>::CreateInstance(*r);
+			if (PString(Auth->GetName()) == authlist[i])
+				m_h235authenticators.Append(Auth);
+			else delete Auth;
 			}
-		} else {
-	        m_h235authenticators.Append(Auth);
+		}
+	} else {
+		for (r = keyList.begin(); r != keyList.end(); ++r) {
+			H235Authenticator * Auth = PFactory<H235Authenticator>::CreateInstance(*r);
+			m_h235authenticators.Append(Auth);
 		}
 	}
-#endif
+#endif 
 }
 
 GkAuthenticatorList::~GkAuthenticatorList()
@@ -686,6 +687,7 @@ void GkAuthenticatorList::OnReload()
 	std::list<GkAuthenticator*> authenticators;	
 	GkAuthenticator *auth;
 	
+// TODO None of this works. Authenticators are never populated (is it needed?) - SH
 	const PStringArray authRules = GkConfig()->GetKeys(GkAuthSectionName);
 	for (PINDEX r = 0; r < authRules.GetSize(); r++) {
 		auth = Factory<GkAuthenticator>::Create(authRules[r]);
@@ -777,7 +779,8 @@ void GkAuthenticatorList::OnReload()
 			auth = *iter++;
 			if (auth->IsH235Capable() 
 					&& (auth->GetControlFlag() == GkAuthenticator::e_Optional
-						|| auth->GetControlFlag() == GkAuthenticator::e_Alternative)) {
+						|| auth->GetControlFlag() == GkAuthenticator::e_Alternative)
+						) {
 				if (mechanisms.GetSize() == 0) {
 					auth->GetH235Capability(mechanisms, algorithmOIDs);
 					if (algorithmOIDs.GetSize() == 0 )
@@ -832,7 +835,6 @@ void GkAuthenticatorList::OnReload()
 
 	if (mechanisms.GetSize() > 0 && algorithmOIDs.GetSize() > 0) {
 		if (PTrace::CanTrace(4)) {
-#if PTRACING
 			ostream& strm = PTrace::Begin(4,__FILE__,__LINE__);
 			strm <<"GkAuth\tH.235 capabilities selected for GCF:\n";
 			strm <<"\tAuthentication mechanisms: \n";
@@ -842,7 +844,6 @@ void GkAuthenticatorList::OnReload()
 			for (i = 0; i < algorithmOIDs.GetSize(); i++)
 				strm << "\t\t" << algorithmOIDs[i] << '\n';
 			PTrace::End(strm);
-#endif
 		}
 #ifdef OpenH323Factory
 	}
@@ -880,11 +881,11 @@ void GkAuthenticatorList::SelectH235Capability(
 		return;
 
 #ifdef OpenH323Factory
-    for (PINDEX auth = 0; auth < m_h235authenticators.GetSize(); auth++) {
-     for (PINDEX cap = 0; cap < grq.m_authenticationCapability.GetSize(); cap++) {
-      for (PINDEX alg = 0; alg < grq.m_algorithmOIDs.GetSize(); alg++) {
-        if (m_h235authenticators[auth].IsCapability(grq.m_authenticationCapability[cap],
-                                              grq.m_algorithmOIDs[alg])) {
+	for (PINDEX auth = 0; auth < m_h235authenticators.GetSize(); auth++) {
+	 for (PINDEX cap = 0; cap < grq.m_authenticationCapability.GetSize(); cap++) {
+	  for (PINDEX alg = 0; alg < grq.m_algorithmOIDs.GetSize(); alg++) {
+		if (m_h235authenticators[auth].IsCapability(grq.m_authenticationCapability[cap],
+						grq.m_algorithmOIDs[alg])) {
 			std::list<GkAuthenticator*>::const_iterator iter = m_authenticators.begin();
 			while (iter != m_authenticators.end()) {
 			 GkAuthenticator* gkauth = *iter++;
@@ -898,10 +899,10 @@ void GkAuthenticatorList::SelectH235Capability(
 				  return;
 			  }
 		   }
-        }
-      }
-    }
-   }
+		}
+	  }
+	 }
+	}
 
 #else
 	H225_ArrayOf_AuthenticationMechanism & mechanisms = *m_mechanisms;
@@ -960,6 +961,7 @@ bool GkAuthenticatorList::Validate(
 					return true;
 			} else if (result == GkAuthenticator::e_fail) {
 				PTRACE(3, "GKAUTH\t" << auth->GetName() << " RRQ check failed");
+				SNMP_TRAP(8, SNMPError, Authentication, auth->GetName() + " RRQ check failed");
 				return false;
 			}
 		}
@@ -983,8 +985,8 @@ bool GkAuthenticatorList::Validate(
 			const int result = auth->Check(request, authData);
 			if (authData.m_callDurationLimit == 0) {
 				PTRACE(3, "GKAUTH\t" << auth->GetName() << " ARQ check failed: "
-					"call duration 0"
-					);
+					"call duration 0");
+				SNMP_TRAP(8, SNMPError, Authentication, auth->GetName() + " ARQ check failed");
 				return false;
 			}
 			if (authData.m_callDurationLimit >= 0 && oldDurationLimit >= 0)
@@ -1002,6 +1004,7 @@ bool GkAuthenticatorList::Validate(
 					return true;
 			} else if (result == GkAuthenticator::e_fail) {
 				PTRACE(3, "GKAUTH\t" << auth->GetName() << " ARQ check failed");
+				SNMP_TRAP(8, SNMPError, Authentication, auth->GetName() + " ARQ check failed");
 				return false;
 			}
 		}
@@ -1026,8 +1029,8 @@ bool GkAuthenticatorList::Validate(
 			const int result = auth->Check(setup, authData);
 			if (authData.m_callDurationLimit == 0) {
 				PTRACE(3, "GKAUTH\t" << auth->GetName() << " Setup check failed: "
-					"call duration limit 0"
-					);
+					"call duration limit 0");
+				SNMP_TRAP(8, SNMPError, Authentication, auth->GetName() + " Setup check failed");
 				return false;
 			}
 			if (authData.m_callDurationLimit >= 0 && oldDurationLimit >= 0)
@@ -1045,6 +1048,7 @@ bool GkAuthenticatorList::Validate(
 					return true;
 			} else if (result == GkAuthenticator::e_fail) {
 				PTRACE(3, "GKAUTH\t" << auth->GetName() << " Setup check failed");
+				SNMP_TRAP(8, SNMPError, Authentication, auth->GetName() + " Setup check failed");
 				return false;
 			}
 		}
@@ -1108,38 +1112,41 @@ SimplePasswordAuth::SimplePasswordAuth(
 	m_disabledAlgorithms = GetConfig()->GetString(name, "DisableAlgorithm", "").Tokenise(",;", FALSE);
 
 #ifdef OpenH323Factory
-	PStringList authlist = Toolkit::Instance()->GetAuthenticatorList();
-
     PFactory<H235Authenticator>::KeyList_T keyList = PFactory<H235Authenticator>::GetKeyList();
     PFactory<H235Authenticator>::KeyList_T::const_iterator r;
-    for (r = keyList.begin(); r != keyList.end(); ++r) {
-		H235Authenticator * Auth = PFactory<H235Authenticator>::CreateInstance(*r);
-		// only use, if it's not disabled for this GnuGk authentication method
-		if (m_disabledAlgorithms.GetStringsIndex(Auth->GetName()) == P_MAX_INDEX) {
-			if (authlist.GetSize() > 0) {
-				// if a global list of autenticators is configures, use it only if its on the list
-				for (PINDEX i=0; i< authlist.GetSize(); i++) {
-					if (PString(Auth->GetName()) == authlist[i])
-						AppendH235Authenticator(Auth);
-				}
-			} else {
-				// without global authenticator list: use all
+
+	PStringList authlist = Toolkit::Instance()->GetAuthenticatorList();
+	// if a global list of autenticators is configured, use it in the priority order supplied
+	if (authlist.GetSize() > 0) {
+		for (PINDEX i = 0; i < authlist.GetSize(); ++i) {
+		  for (r = keyList.begin(); r != keyList.end(); ++r) {
+			H235Authenticator * Auth = PFactory<H235Authenticator>::CreateInstance(*r);
+			if (PString(Auth->GetName()) == authlist[i])
 				AppendH235Authenticator(Auth);
-			}
+			else delete Auth;
+		  }
+		}
+	} else {
+		for (r = keyList.begin(); r != keyList.end(); ++r) {
+			H235Authenticator * Auth = PFactory<H235Authenticator>::CreateInstance(*r);
+			// only use, if it's not disabled for this GnuGk authentication method
+			if (m_disabledAlgorithms.GetStringsIndex(Auth->GetName()) == P_MAX_INDEX)
+				AppendH235Authenticator(Auth);
+			else delete Auth;
 		}
 	}
 #else
 	H235Authenticator* authenticator;
 
 	if (m_disabledAlgorithms.GetStringsIndex("MD5") == P_MAX_INDEX) {
-		authenticator = new H235AuthSimpleMD5;
+		authenticator = new H235AuthSimpleMD5();
 		authenticator->SetLocalId("dummy");
 		authenticator->SetRemoteId("dummy");
 		authenticator->SetPassword("dummy");
 		AppendH235Authenticator(authenticator);
 	}
 	if (m_disabledAlgorithms.GetStringsIndex("CAT") == P_MAX_INDEX) {
-		authenticator = new H235AuthCAT;
+		authenticator = new H235AuthCAT();
 		authenticator->SetLocalId("dummy");
 		authenticator->SetRemoteId("dummy");
 		authenticator->SetPassword("dummy");
@@ -1148,7 +1155,7 @@ SimplePasswordAuth::SimplePasswordAuth(
 
 #if P_SSL
 	if (m_disabledAlgorithms.GetStringsIndex("H.235.1") == P_MAX_INDEX) {
-		authenticator = new H235AuthProcedure1;
+		authenticator = new H235AuthProcedure1();
 		authenticator->SetLocalId("dummy");
 		authenticator->SetRemoteId("dummy");
 		authenticator->SetPassword("dummy");
@@ -1165,11 +1172,6 @@ SimplePasswordAuth::~SimplePasswordAuth()
 	delete m_cache;
 }
 
-int SimplePasswordAuth::Check(RasPDU<H225_GatekeeperRequest> & request, unsigned &)
-{
-	return doCheck(request);
-}
-
 int SimplePasswordAuth::Check(
 	RasPDU<H225_RegistrationRequest> & request, 
 	RRQAuthData& /*authData*/
@@ -1456,52 +1458,47 @@ int SimplePasswordAuth::CheckCryptoTokens(
 	return e_next;
 }
 
-bool SimplePasswordAuth::ResolveUserName(
-		const H225_ArrayOf_ClearToken& tokens,
-	    const H225_ArrayOf_CryptoH323Token& cryptotokens,
-		PString & username
-		)
+bool SimplePasswordAuth::ResolveUserName(const H235_ClearToken & token, PString & username)
 {
-	for (PINDEX i = 0; i < cryptotokens.GetSize(); i++) {
-		// MD5
-		if (cryptotokens[i].GetTag() == H225_CryptoH323Token::e_cryptoEPPwdHash) {
-			H225_CryptoH323Token_cryptoEPPwdHash& pwdhash = cryptotokens[i];
-			username = AsString(pwdhash.m_alias, false);
-			return true;
-		} else if (cryptotokens[i].GetTag() == H225_CryptoH323Token::e_nestedcryptoToken) {
-			H235_ClearToken clearToken;
-			bool found = false;
-			const H235_CryptoToken& nestedCryptoToken = cryptotokens[i];
-
-		    // H235.1
-			if (nestedCryptoToken.GetTag() == H235_CryptoToken::e_cryptoHashedToken) {
-			  const H235_CryptoToken_cryptoHashedToken& cryptoHashedToken = nestedCryptoToken;
-			  clearToken = cryptoHashedToken.m_hashedVals;
-			  found = true;
-			}
-			// H235.2
-			if (nestedCryptoToken.GetTag() == H235_CryptoToken::e_cryptoSignedToken) {			
-	          const H235_CryptoToken_cryptoSignedToken & cryptoSignedToken = nestedCryptoToken;
-              H235_SIGNED<H235_EncodedGeneralToken> m_Signed = cryptoSignedToken.m_token;
-			  if (m_Signed.m_toBeSigned.DecodeSubType(clearToken))
-				  found = true;
-			}
-		
-			if (found && (clearToken.HasOptionalField(H235_ClearToken::e_sendersID))) {
-               username = clearToken.m_sendersID; 
-			   return true;
-			}
-		} 
+	// CAT
+	if (token.HasOptionalField(H235_ClearToken::e_generalID)) {
+		username = token.m_generalID;
+		return true;
 	}
+	return false;
+}
 
-	for (PINDEX j = 0; j < tokens.GetSize(); j++) {
-		H235_ClearToken& token = tokens[j];
-		// CAT
-		if (token.HasOptionalField(H235_ClearToken::e_generalID)) {
-			username = token.m_generalID;
-			return true;
+bool SimplePasswordAuth::ResolveUserName(const H225_CryptoH323Token & cryptotoken, PString & username)
+{
+	// MD5
+	if (cryptotoken.GetTag() == H225_CryptoH323Token::e_cryptoEPPwdHash) {
+		const H225_CryptoH323Token_cryptoEPPwdHash & pwdhash = cryptotoken;
+		username = AsString(pwdhash.m_alias, false);
+		return true;
+	} else if (cryptotoken.GetTag() == H225_CryptoH323Token::e_nestedcryptoToken) {
+		H235_ClearToken clearToken;
+		bool found = false;
+		const H235_CryptoToken & nestedCryptoToken = cryptotoken;
+
+		// H235.1
+		if (nestedCryptoToken.GetTag() == H235_CryptoToken::e_cryptoHashedToken) {
+			const H235_CryptoToken_cryptoHashedToken& cryptoHashedToken = nestedCryptoToken;
+			clearToken = cryptoHashedToken.m_hashedVals;
+			found = true;
 		}
-	}
+		// H235.2
+		if (nestedCryptoToken.GetTag() == H235_CryptoToken::e_cryptoSignedToken) {			
+			const H235_CryptoToken_cryptoSignedToken & cryptoSignedToken = nestedCryptoToken;
+			H235_SIGNED<H235_EncodedGeneralToken> m_Signed = cryptoSignedToken.m_token;
+			if (m_Signed.m_toBeSigned.DecodeSubType(clearToken))
+				found = true;
+		}
+
+		if (found && (clearToken.HasOptionalField(H235_ClearToken::e_sendersID))) {
+		   username = clearToken.m_sendersID; 
+		   return true;
+		}
+	} 
 
 	return false;
 }
@@ -1663,12 +1660,10 @@ bool AliasAuth::doCheck(
 	)
 {
 	const PStringArray authrules(condition.Tokenise("&|", FALSE));
-#if PTRACING
 	if (authrules.GetSize() < 1) {
 		PTRACE(2, "GKAUTH\t" << GetName() << " contains an empty auth condition");
 		return false;
 	}
-#endif
 	for (PINDEX i = 0; i < authrules.GetSize(); ++i)
 		for (PINDEX j = 0; j < sigaddr.GetSize(); ++j)
 			if (CheckAuthRule(sigaddr[j], authrules[i])) {
@@ -1916,11 +1911,9 @@ public:
 		) : AuthRule(fate, inverted), m_pattern(aliasStr) 
 	{ 
 		m_priority = -1;
-#if PTRACING
 		SetName(PString((fate == e_allow) ? "allow alias" : "deny alias")
 			+ (inverted ? ":!" : ":") + aliasStr
 			);
-#endif
 	}
 
 	virtual bool Match(
@@ -2030,10 +2023,8 @@ IPAuthRule::IPAuthRule(Result fate, const PString & ipStr, bool inverted)
 	DWORD n = ~PIPSocket::Net2Host(DWORD(m_netmask));
 	for (m_priority = 0; n; n >>= 1)
 		++m_priority;
-#if PTRACING
 	SetName(PString((fate == e_allow) ? "allow ip(" : "deny ip(")
 		+ PString(m_priority) + (inverted ? "):!" : "):") + ipStr);
-#endif
 }
 
 bool IPAuthRule::Match(const AuthObj & aobj)
@@ -2129,7 +2120,7 @@ PrefixAuth::PrefixAuth(
 					is_inverted(rules[j], pp)
 					);
 			else {
-				rls[j] = new NullRule;
+				rls[j] = new NullRule();
 			}
 		}
 
@@ -2236,7 +2227,6 @@ int PrefixAuth::doCheck(
 			}
 		}
 	}
-#if PTRACING
 	if (m_defaultRule == e_ok)
 		PTRACE(4, "GKAUTH\t" << GetName() << " default rule accepted "
 			"the request"
@@ -2249,7 +2239,6 @@ int PrefixAuth::doCheck(
 		PTRACE(4, "GKAUTH\t" << GetName() << " could not reject or "
 			"accept the request"
 			);
-#endif
 	return m_defaultRule;
 }
 
