@@ -14,7 +14,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKAUTH_H
-#define GKAUTH_H "@(#) $Id: gkauth.h,v 1.54 2011/03/30 11:27:15 willamowius Exp $"
+#define GKAUTH_H "@(#) $Id: gkauth.h,v 1.61 2012/12/11 19:16:56 willamowius Exp $"
 
 #include <map>
 #include <list>
@@ -22,6 +22,7 @@
 #include "rwlock.h"
 #include <h235auth.h>
 #include "Toolkit.h"
+#include "snmp.h"
 #include "h323util.h"
 
 class H225_GatekeeperRequest;
@@ -134,9 +135,7 @@ private:
 /// by all authenticators
 struct SetupAuthData
 {
-	SetupAuthData(
-		const SetupAuthData& obj
-		);
+	SetupAuthData(const SetupAuthData & obj);
 	SetupAuthData(
 		/// call associated with the message (if any)
 		const callptr& call,
@@ -527,8 +526,7 @@ class SimplePasswordAuth : public GkAuthenticator
 public:
 	enum SupportedRasChecks {
 		/// bitmask of RAS checks implemented by this module
-		SimplePasswordAuthRasChecks = RasInfo<H225_GatekeeperRequest>::flag
-			| RasInfo<H225_RegistrationRequest>::flag
+		SimplePasswordAuthRasChecks = RasInfo<H225_RegistrationRequest>::flag
 			| RasInfo<H225_UnregistrationRequest>::flag
 			| RasInfo<H225_BandwidthRequest>::flag
 			| RasInfo<H225_DisengageRequest>::flag
@@ -546,7 +544,6 @@ public:
 	virtual ~SimplePasswordAuth();
 
 	// overriden from class GkAuthenticator
-	virtual int Check(RasPDU<H225_GatekeeperRequest>& req, unsigned& rejectReason);
 	virtual int Check(RasPDU<H225_UnregistrationRequest>& req, unsigned& rejectReason);
 	virtual int Check(RasPDU<H225_BandwidthRequest>& req, unsigned& rejectReason);
 	virtual int Check(RasPDU<H225_DisengageRequest>& req, unsigned& rejectReason);
@@ -614,21 +611,30 @@ protected:
 		/// an array of cryptoTokens to be checked
 		const H225_ArrayOf_CryptoH323Token& cryptoTokens, 
 		/// aliases for the endpoint that generated the tokens
-		const H225_ArrayOf_AliasAddress* aliases,
+		const H225_ArrayOf_AliasAddress * aliases,
 		/// raw data for RAS PDU - required to validate some tokens
 		/// like H.235 Auth Procedure I
-		const PBYTEArray& rawPDU
+		const PBYTEArray & rawPDU
 		);
 
-	/** Retrieve username carried inside the tokens. 
+	/** Retrieve username carried inside the token. 
 	    @return
 	    username carried inside the token
 	*/
 	bool ResolveUserName(
 		/// an array of tokens to be checked
-		const H225_ArrayOf_ClearToken& tokens,
-		/// aliases for the endpoint that generated the tokens
-	    const H225_ArrayOf_CryptoH323Token& crytotokens,
+		const H235_ClearToken & token,
+		/// UserName detected.
+		PString & username
+		);
+
+	/** Retrieve username carried inside the token. 
+	    @return
+	    username carried inside the token
+	*/
+	bool ResolveUserName(
+		/// an array of tokens to be checked
+		const H225_CryptoH323Token & crytotoken,
 		/// UserName detected.
 		PString & username
 		);
@@ -643,71 +649,75 @@ protected:
 	*/
 	template<class RAS> int doCheck(
 		/// RAS request to be authenticated
-		const RasPDU<RAS>& request,
+		const RasPDU<RAS> & request,
 		/// list of aliases for the endpoint sending the request
-		const H225_ArrayOf_AliasAddress* aliases = NULL
-		)
+		const H225_ArrayOf_AliasAddress* aliases = NULL)
 	{
-		const RAS& req = request;
+		const RAS & req = request;
 		bool finalResult = false;
 		
 #ifdef OpenH323Factory
-
 		if (m_h235Authenticators == NULL) {
-			PTRACE(4, "GKAUTH\tSuccess: No Loaded Authenticators");
-			return e_ok;
-		}
-
-		PString username = PString::Empty();
-		PString password = PString::Empty();
-		if (!ResolveUserName(req.m_tokens, req.m_cryptoTokens,username)) {
-            PTRACE(4, "GKAUTH\t" << GetName() << " No username resolved from tokens.");
-			return e_fail;
-		}
-
-		if ((aliases == NULL) || (FindAlias(*aliases, username) == P_MAX_INDEX)) {
-            PTRACE(4, "GKAUTH\t" << GetName() << " Token username " << username << " does not match aliases for Endpoint");
-			return e_fail;
-		}
-
-		if (!InternalGetPassword(username, password)) {
-				PTRACE(4, "GKAUTH\t" << GetName() << " password not found for " << username );
-			// do not return false let the authenticator decide whether it requires a password or not.
+			PTRACE(4, "GKAUTH\tNo Loaded Authenticators");
+			return GetDefaultStatus();
 		}
 
         for (PINDEX i = 0; i < m_h235Authenticators->GetSize();  i++) {
-          H235Authenticator * authenticator = (H235Authenticator *)(*m_h235Authenticators)[i].Clone();
+			H235Authenticator * authenticator = (H235Authenticator *)(*m_h235Authenticators)[i].Clone();
+			authenticator->SetLocalId(Toolkit::GKName());
 
-		  authenticator->SetLocalId(Toolkit::GKName());
-		  authenticator->SetRemoteId(username);
-		  authenticator->SetPassword(password);
-
-          H235Authenticator::ValidationResult result = authenticator->ValidateTokens(req.m_tokens, 
-			                                                        req.m_cryptoTokens, request->m_rasPDU);
-          switch (result) {
-             case H235Authenticator::e_OK :
-               PTRACE(4, "GKAUTH\tAuthenticator " << authenticator->GetName() << " succeeded");
-               return e_ok;
-
-             case H235Authenticator::e_Absent :
-               PTRACE(6, "GKAUTH\tAuthenticator " << authenticator->GetName() << " absent from PDU");
-               break;
-
-             case H235Authenticator::e_Disabled :
-               PTRACE(6, "GKAUTH\tAuthenticator " << authenticator->GetName() << " disabled");
-               break;
-
-             default : // Various other failure modes
-               PTRACE(6, "GKAUTH\tAuthenticator " << authenticator->GetName() << " failed: " << (int)result);
-               return e_fail;
-           }
-
-         }
+			H235Authenticator::ValidationResult result;
+			for (PINDEX t = 0; t < req.m_tokens.GetSize(); t++) {
+				PString username = PString::Empty();
+				PString password = PString::Empty();
+				if (!ResolveUserName(req.m_tokens[t], username)) {
+		            PTRACE(4, "GKAUTH\t" << GetName() << " No username resolved from tokens.");
+					continue;	// skip to next token
+				}
+				if ((aliases == NULL) || (FindAlias(*aliases, username) == P_MAX_INDEX)) {
+					PTRACE(4, "GKAUTH\t" << GetName() << " Token username " << username << " does not match aliases for Endpoint");
+					continue;	// skip to next token
+				}
+				if (!InternalGetPassword(username, password)) {
+					PTRACE(4, "GKAUTH\t" << GetName() << " password not found for " << username );
+					// do not return false let the authenticator decide whether it requires a password or not.
+				}
+				authenticator->SetRemoteId(username);
+				authenticator->SetPassword(password);
+				result = authenticator->ValidateClearToken(req.m_tokens[t]);
+				if (result == H235Authenticator::e_OK) {
+					PTRACE(4, "GKAUTH\tAuthenticator " << authenticator->GetName() << " succeeded");
+					return e_ok;
+				}
+			}
+			for (PINDEX t = 0; t < req.m_cryptoTokens.GetSize(); t++) {
+				PString username = PString::Empty();
+				PString password = PString::Empty();
+				if (!ResolveUserName(req.m_cryptoTokens[t], username)) {
+		            PTRACE(4, "GKAUTH\t" << GetName() << " No username resolved from tokens.");
+					continue;	// skip to next token
+				}
+				if ((aliases == NULL) || (FindAlias(*aliases, username) == P_MAX_INDEX)) {
+					PTRACE(4, "GKAUTH\t" << GetName() << " Token username " << username << " does not match aliases for Endpoint");
+					continue;	// skip to next token
+				}
+				if (!InternalGetPassword(username, password)) {
+					PTRACE(4, "GKAUTH\t" << GetName() << " password not found for " << username );
+					// do not return false let the authenticator decide whether it requires a password or not.
+				}
+				authenticator->SetRemoteId(username);
+				authenticator->SetPassword(password);
+				result = authenticator->ValidateCryptoToken(req.m_cryptoTokens[t], request->m_rasPDU);
+				if (result == H235Authenticator::e_OK) {
+					PTRACE(4, "GKAUTH\tAuthenticator " << authenticator->GetName() << " succeeded");
+					return e_ok;
+				}
+			}
+		}
 #else
 		int result;
 		if (req.HasOptionalField(RAS::e_cryptoTokens)) {
-			if ((result = CheckCryptoTokens(req.m_cryptoTokens, aliases, 
-					request->m_rasPDU)) == e_fail)
+			if ((result = CheckCryptoTokens(req.m_cryptoTokens, aliases, request->m_rasPDU)) == e_fail)
 				return e_fail;
 			finalResult = (result == e_ok);
 		}
@@ -721,9 +731,7 @@ protected:
 	}
 
 	/// Set new timeout for username/password pairs cache
-	void SetCacheTimeout(
-		long newTimeout
-		) { m_cache->SetTimeout(newTimeout); }
+	void SetCacheTimeout(long newTimeout) { m_cache->SetTimeout(newTimeout); }
 
 	/** @return
 	    True if usernames should match one of endpoint aliases.
@@ -938,8 +946,8 @@ public:
 						return true;
 				} else if (result == GkAuthenticator::e_fail) {
 					PTRACE(3, "GKAUTH\t" << auth->GetName() << ' '
-						<< request.GetTagName() << " check failed"
-						);
+						<< request.GetTagName() << " check failed");
+					SNMP_TRAP(8, SNMPError, Authentication, auth->GetName() + " check failed");
 					return false;
 				}
 			}
