@@ -2,7 +2,7 @@
 //
 // H.323 utility functions
 //
-// Copyright (c) 2000-2011, Jan Willamowius
+// Copyright (c) 2000-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -140,6 +140,19 @@ PString AsDotString(const H225_TransportAddress & addr, bool showPort)
 	return result;
 }
 
+PString AsString(const H323TransportAddress & ta)
+{
+	if (!IsSet(ta))
+		return "";
+
+	PIPSocket::Address ip;
+	WORD port = 0;
+	if (ta.GetIpAndPort(ip, port))
+		return AsString(ip, port);
+	else
+		return "";
+}
+
 PString AsString(const H225_EndpointType & terminalType)
 {
 	PString terminalTypeString;
@@ -198,13 +211,13 @@ PString AsString(const H225_AliasAddress & terminalAlias, bool includeAliasType)
 }
 
 
-PString AsString(const H225_ArrayOf_AliasAddress & terminalAlias, bool includeAliasName)
+PString AsString(const H225_ArrayOf_AliasAddress & terminalAlias, bool includeAliasTypeName)
 {
 	PString aliasListString = PString::Empty();
 
 	for(PINDEX cnt = 0; cnt < terminalAlias.GetSize(); cnt++ )
 	{
-		aliasListString += AsString(terminalAlias[cnt], includeAliasName);
+		aliasListString += AsString(terminalAlias[cnt], includeAliasTypeName);
 
 		if (cnt < (terminalAlias.GetSize() - 1)) {
 			aliasListString += "=";
@@ -245,6 +258,32 @@ PString StripAliasType(const PString & alias)
 	}
 }
 
+// check if an alias is a valid US 10D or 11D number
+bool Is10Dor11Dnumber(const H225_AliasAddress & alias)
+{
+	PString num = AsString(alias, false);
+	if (num.GetLength() < 10 || num.GetLength() > 11)
+		return false;
+	// must start with 1 if its 11D
+	if (num.GetLength() == 11) {
+		if (num.Left(1) != "1")
+			return false;
+		else
+			num = num.Right(10);
+	}
+	// check parts
+	PString area = num.Left(3);
+	PString local = num.Right(7);
+	// area code must start with 2..9
+	if (area.Left(1) == "0" || area.Left(1) == "1")
+		return false;
+	if (local == "0000000")
+		return false;
+
+	// OK, all checks passed
+	return true;
+}
+
 H245_TransportAddress IPToH245TransportAddr(const PIPSocket::Address & ip, WORD Port)
 {
 	H245_TransportAddress Result;
@@ -299,7 +338,6 @@ H225_TransportAddress SocketToH225TransportAddr(const PIPSocket::Address & Addr,
 	} else {
 		Result.SetTag( H225_TransportAddress::e_ipAddress );
 		H225_TransportAddress_ipAddress & ResultIP = Result;
-
 		for (int i = 0; i < 4; ++i)
 			ResultIP.m_ip[i] = Addr[i];
 		ResultIP.m_port = Port;
@@ -347,7 +385,7 @@ bool GetIPAndPortFromTransportAddr(const H225_TransportAddress & addr, PIPSocket
 		port = (WORD)ipaddr.m_port;
 		return true;
 	}
-	 return false;
+	return false;
 }
 
 PStringArray SplitIPAndPort(const PString & str, WORD default_port)
@@ -548,6 +586,12 @@ bool IsSet(const H323TransportAddress & addr)
 	return (addr != H323TransportAddress());
 }
 
+bool IsValidE164(const PString & s) 
+{
+	return (!s.IsEmpty() && strspn(s, "1234567890*#+,") == strlen(s));
+}
+
+
 PString GetGUIDString(
 	const H225_GloballyUniqueID & id, /// 128-bit identifier to convert
 	bool fixedLength /// skip leading zeros (false) or not (true)
@@ -562,7 +606,7 @@ PString GetGUIDString(
 		const unsigned hex = ((unsigned)(id[i])<<24) | ((unsigned)(id[i+1])<<16) 
 			| ((unsigned)(id[i+2])<<8) | ((unsigned)(id[i+3]));
 		i += 4;
-		
+
 		idstr += fixedLength ? PString(PString::Printf, "%08x", hex)
 			: PString(PString::Unsigned, (long)hex, 16);
 		if (j < 3)
@@ -689,3 +733,49 @@ PString RewriteString(
 	
 	return result;
 }
+
+PString RewriteWildcard(
+	const PString & s,
+	const PString & expression 
+	)
+{
+	PString o = expression;
+
+	if (o.IsEmpty())
+		return s;
+	
+	int b1 = o.Find('{');
+	if (b1 == P_MAX_INDEX) return o; /// No rewrite
+
+	int b2 = o.Find('}');
+	if (b2 == P_MAX_INDEX || b1 > b2) return s;  /// Logic error ignore rewrite
+
+	PString exp = o.Mid(b1+1,b2-b1-1);
+
+	PString r;
+	if (exp *= "\\1")
+		r = s;
+	else {
+	   int start = -1;
+       if (exp.Find('^') != P_MAX_INDEX)  start = 0;
+	   else if (exp.Find('$') != P_MAX_INDEX) start = 1;
+	  
+	   if (start < 0) return s;   /// Logic error ignore rewrite
+
+	   int c1 = exp.Find('(');
+	   if (c1 == P_MAX_INDEX) return s;   /// Logic error ignore rewrite
+	   int c2 = exp.Find(')');
+	   if (c2 == P_MAX_INDEX || c1 > c2) return s;   /// Logic error ignore rewrite
+
+	   int i = exp.Mid(c1+1,c2-c1-1).AsInteger();
+	   if (i == 0) return s;   /// Logic error ignore rewrite
+
+       if (b1 > 0) r = s.Left(i);
+	   else r = s.Mid(s.GetLength()-i);
+	}
+
+    if (b1 > 0) 
+		return o.Left(b1-1) + r;
+	else
+		return r + o.Mid(b2+1);
+};
