@@ -2,7 +2,7 @@
 //
 // H.323 utility functions
 //
-// Copyright (c) 2000-2011, Jan Willamowius
+// Copyright (c) 2000-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +13,7 @@
 
 
 #ifndef H323UTIL_H
-#define H323UTIL_H "@(#) $Id: h323util.h,v 1.43 2011/11/11 12:07:59 willamowius Exp $"
+#define H323UTIL_H "@(#) $Id: h323util.h,v 1.49 2012/12/23 06:39:46 shorne Exp $"
 
 #include <ptlib.h>
 #include <ptlib/sockets.h>
@@ -48,18 +48,23 @@ PString AsString(const H225_TransportAddress & ta);
 
 PString AsDotString(const H225_TransportAddress & ip, bool showPort=true);
 
+PString AsString(const H323TransportAddress & ta);
+
 PString AsString(const H225_EndpointType & terminalType);
 
-PString AsString(const H225_AliasAddress & terminalAlias, bool includeAliasName = TRUE);
+PString AsString(const H225_AliasAddress & terminalAlias, bool includeAliasTypeName = true);
 
 PString AsString(const PBYTEArray & array);
 
-PString AsString(const H225_ArrayOf_AliasAddress & terminalAlias, bool includeAliasName = TRUE);
+PString AsString(const H225_ArrayOf_AliasAddress & terminalAlias, bool includeAliasTypeName = true);
 
 PString AsString(const PASN_OctetString & Octets);
 
 PString StripAliasType(const PString & alias);
 
+// check if an alias is a valid US 10 digit or 11 digit number (no formatting allowed)
+bool Is10Dor11Dnumber(const H225_AliasAddress & alias);
+
 H245_TransportAddress IPToH245TransportAddr(const PIPSocket::Address & ip, WORD Port);
 
 // convert a string (dot notation without port) into an H245 transport address
@@ -110,6 +115,9 @@ bool IsLoopback(const PIPSocket::Address &);
 
 bool IsSet(const H323TransportAddress & addr);
 
+// check if s only contains characters that are valid for E.164s (1234567890*#+,)
+bool IsValidE164(const PString & s);
+
 /** Find an alias which tag is of type specified by #primaryTags#.
     If no such aliases are found, #secondaryTags# are examined.
     If still no match is found and #exactMatch# is false, the first
@@ -213,5 +221,19 @@ PString RewriteString(
 	const char *value /// new string that replaces the prefix string
 	);
 	
+/** Rewrite Wildcard match 
+	Lightweight Regex match rewrite.
+	Only basic number matching supported at the moment
+	Example  
+		12345678  {\1}@mydomain.com  = 12345678@mydomain.com
+		12345678  {^\d(4)}@mydomain.com = 1234@mydomain.com
+		12345678  {\d(4)$}@mydomain.com = 5678@mydomain.com
+	
+	@return	Rewritten string.
+*/
+PString RewriteWildcard(
+	const PString & s,          /// original string to rewrite
+	const PString & expression /// prefix string that matched					
+	);
 	
 #endif // H323UTIL_H
