@@ -3,7 +3,7 @@
  *
  * unit tests for h323util.cxx
  *
- * Copyright (c) 2011, Jan Willamowius
+ * Copyright (c) 2011-2012, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -83,11 +83,11 @@ TEST_F(H323UtilTest, H225TransportAddressAsString) {
 	EXPECT_STREQ("2001:db8:85a3:8d3:1319:8a2e:370:7344",        AsDotString(h225transport_withipv6, false));
 }
 
-// fails but not currently used by GnuGk
-//TEST_F(H323UtilTest, H323TransportAddressAsString) {
-//	EXPECT_STREQ("6.7.8.9:4567", AsString(h323transport_withipv4));
-//	EXPECT_STREQ("[2001:0db8:85a3:08d3:1319:8a2e:0370:7344]:5678", AsString(h323transport_withipv6));
-//}
+TEST_F(H323UtilTest, H323TransportAddressAsString) {
+	EXPECT_STREQ("", AsString(H323TransportAddress()));
+	EXPECT_STREQ("6.7.8.9:4567", AsString(h323transport_withipv4));
+	EXPECT_STREQ("[2001:db8:85a3:8d3:1319:8a2e:370:7344]:5678", AsString(h323transport_withipv6));
+}
 
 TEST_F(H323UtilTest, H225TransportAddressGetVersion) {
 	EXPECT_EQ(4u, GetVersion(h225transport_withipv4));
@@ -155,6 +155,28 @@ TEST_F(H323UtilTest, ByteArrayAsString) {
 	EXPECT_STREQ("abc", AsString(bytes));
 }
 
+TEST_F(H323UtilTest, Is10Dor11Dnumber) {
+	H225_AliasAddress alias;
+	H323SetAliasAddress(PString("jan"), alias);
+	EXPECT_FALSE(Is10Dor11Dnumber(alias));
+	H323SetAliasAddress(PString("123456789"), alias);
+	EXPECT_FALSE(Is10Dor11Dnumber(alias));
+	H323SetAliasAddress(PString("123456789012"), alias);
+	EXPECT_FALSE(Is10Dor11Dnumber(alias));
+	H323SetAliasAddress(PString("10000000000"), alias);
+	EXPECT_FALSE(Is10Dor11Dnumber(alias));
+	H323SetAliasAddress(PString("0000000000"), alias);
+	EXPECT_FALSE(Is10Dor11Dnumber(alias));
+	H323SetAliasAddress(PString("1238888777"), alias);
+	EXPECT_FALSE(Is10Dor11Dnumber(alias));
+	H323SetAliasAddress(PString("2238888777"), alias);
+	EXPECT_TRUE(Is10Dor11Dnumber(alias));
+	H323SetAliasAddress(PString("2008888777"), alias);
+	EXPECT_TRUE(Is10Dor11Dnumber(alias));
+	H323SetAliasAddress(PString("2008000000"), alias);
+	EXPECT_TRUE(Is10Dor11Dnumber(alias));
+}
+
 TEST_F(H323UtilTest, SetSockaddr) {
 	// H323TransportAddress IPv4 version
 	struct sockaddr_in sin;
