@@ -2,8 +2,8 @@
 //
 // Presence in H.323 gatekeeper
 //
-// Copyright (c) 2009-2010, Simon Horne
-// Copyright (c) 2009-2010, Jan Willamowius
+// Copyright (c) 2009-2012, Simon Horne
+// Copyright (c) 2009-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License (GPL)
 // see file COPYING for details.
@@ -28,7 +28,8 @@
 #include <h323pdu.h>
 #include <ptclib/delaychan.h>
 
-#define DEFAULT_PRESWORKER_TIMER 2   // fire thread every 2 seconds;
+#define DEFAULT_PRESWORKER_TIMER 5   // fire thread every 5 seconds;
+
 
 ///////////////////////////////////////////////////////
 
@@ -37,7 +38,7 @@ class PresWorker : public PThread
 public:
 	PCLASSINFO(PresWorker, PThread)
 
-	/// create a new Worker thread and start it immediatelly
+	/// create a new Worker thread and start it immediately
 	PresWorker(
 		GkPresence * _handler,
 		int _waitTime
@@ -55,9 +56,10 @@ protected:
 	void ProcessMessages();
 
 private:
-	GkPresence * handler;
-	long waitTime;
-	bool shutDown;
+	GkPresence *	handler;
+	long			waitTime;
+	bool			shutDown;
+    PSyncPointAck	exitWorker;
 
 	PAdaptiveDelay m_delay;
 
@@ -66,7 +68,7 @@ private:
 PresWorker::PresWorker(GkPresence * _handler , int _waitTime)
 : PThread(5000, AutoDeleteThread), handler(_handler), waitTime(_waitTime), shutDown(false)
 {
-	PTRACE(4,"PRES\tPresence Thread instance fire every " << waitTime << " sec");
+	PTRACE(4, "PRES\tPresence Thread instance fire every " << waitTime << " sec");
 	Resume();
 }
 
@@ -77,21 +79,22 @@ PresWorker::~PresWorker()
 	
 void PresWorker::Main()
 {
-	while (!shutDown) {
-
-		// Go do the coding
-		ProcessMessages();
-
-		// Wait
-		m_delay.Delay(waitTime);
+	while (!exitWorker.Wait(0)) {
+		handler->DatabaseIncrementalUpdate();
+		if (shutDown) continue;
+			ProcessMessages();
+		if (shutDown) continue;
+			m_delay.Delay(waitTime);
 	}
+	exitWorker.Acknowledge();
 }
 
 void PresWorker::Close()
 {
 	if (!shutDown) {
-	    PTRACE(4,"PRES\tPresence Thread Shutdown");
+		PTRACE(4, "PRES\tPresence Thread Shutdown");
 		shutDown = true;
+		exitWorker.Signal();
 	}
 }
 
@@ -183,9 +186,10 @@ H460P_PresenceInstruction & BuildInstructionMsg(const H460P_PresenceInstruction
 	return msg;
 }
 
-H460P_PresenceInstruction & BuildInstructionMsg(unsigned type, const H225_AliasAddress & addr, H460P_PresencePDU & msg)
+H460P_PresenceInstruction & BuildInstructionMsg(unsigned type, const H225_AliasAddress & addr, H460P_PresencePDU & msg, 
+												const PString & display = PString(), const PString & avatar = PString())
 {
-	H323PresenceInstruction instruct((H323PresenceInstruction::Instruction)type, AsString(addr,0));
+	H323PresenceInstruction instruct((H323PresenceInstruction::Instruction)type, AsString(addr,0), display, avatar);
 	return BuildInstructionMsg(instruct,msg);
 }
 
@@ -199,14 +203,24 @@ H460P_PresenceSubscription & BuildSubscriptionMsg(const H460P_PresenceSubscripti
 	return msg;
 }
 
-H460P_PresenceSubscription & BuildSubscriptionMsg(const OpalGloballyUniqueID & id, const H225_AliasAddress remote,
-												const H225_AliasAddress local, H460P_PresencePDU & msg)
+#if H460P_VER < 2
+H460P_PresenceSubscription & BuildSubscriptionMsg(const OpalGloballyUniqueID & id, const H225_AliasAddress & remote,
+												const H225_AliasAddress & local, H460P_PresencePDU & msg)
 {
 	H323PresenceSubscription sub;
 	sub.SetSubscription(id);
 	sub.SetSubscriptionDetails(local,remote);
 	return BuildSubscriptionMsg(sub,msg);
 }
+#else
+H460P_PresenceSubscription & BuildSubscriptionMsg(const OpalGloballyUniqueID & id, const H323PresenceID & pid , H460P_PresencePDU & msg)
+{
+	H323PresenceSubscription sub;
+	sub.SetSubscription(id);
+	sub.SetSubscriptionDetails(pid.m_subscriber,pid.m_Alias,pid.m_Display,pid.m_Avatar,pid.m_Category);
+	return BuildSubscriptionMsg(sub,msg);
+}
+#endif
 
 H460P_PresenceNotification & BuildNotificationMsg(const H460P_PresenceNotification & notification, H460P_PresencePDU & msg)
 {
@@ -251,7 +265,13 @@ bool UpdateInstruction(const H460P_PresenceInstruction & addr, H323PresenceInstr
 {
 	bool found = false;
 	for (PINDEX i=0; i< inst.GetSize(); ++i) {
-		if (AsString(addr,0) == inst[i].GetAlias()) {
+#if H460P_VER < 2
+		const H225_AliasAddress & alias = addr;
+#else
+		const H460P_PresenceAlias & palias = addr;
+		const H225_AliasAddress & alias = palias.m_alias;
+#endif
+		if (AsString(alias,0) == inst[i].GetAlias()) {
 			found = true;
 			inst[i] = (const H323PresenceInstruction &)addr;
 		}
@@ -312,7 +332,7 @@ H460P_PresenceIdentifier & AsPresenceId(const PString & id)
 PString AsPresenceString(const H460P_PresenceIdentifier & pid)
 {
 	OpalGloballyUniqueID uid(pid.m_guid);
-	return uid.AsString();
+	return uid.AsString().ToUpper();
 }
 
 bool IsLocalPresence(const H225_AliasAddress & alias, const H323PresenceStore & store)
@@ -348,7 +368,8 @@ PString PresMsgType(unsigned tag)
 GkPresence::GkPresence()
  : m_enabled(false), m_sqlactive(false), m_worker(NULL)
 #if HAS_DATABASE
-	, m_sqlConn(NULL), m_timeout(-1)
+	, m_sqlConn(NULL), m_timeout(-1), m_lastTimeStamp(0)
+	, m_incrementalUpdate(false)
 #endif
 {
 }
@@ -437,56 +458,144 @@ void GkPresence::LoadConfig(PConfig * cfg)
 		return;
 	}
 
-	m_sqlactive = DatabaseLoad();
+	m_incrementalUpdate = cfg->GetBoolean(authName, "IncrementalUpdate", false);
+
+	m_sqlactive = DatabaseLoad(false);
+
+	m_worker = new PresWorker(this, cfg->GetInteger(authName, "UpdateWorkerTimer", DEFAULT_PRESWORKER_TIMER)*1000);
 #endif
 
 	if (!m_sqlactive) {
 		PTRACE(1, "H460P\tNo backend database support. Please recompile GnuGk with database support.");
 		return;
 	}
-
-	m_worker = new PresWorker(this, cfg->GetInteger("RoutedMode", "H460PActThread", DEFAULT_PRESWORKER_TIMER)*1000);
 }
 
-bool GkPresence::DatabaseLoad()
+#if PTRACING
+PString presenceFieldName(int type)
+{
+	static const char * const fields[] = {
+		"GUID",
+		"Subscriber",
+		"Alias",
+		"Local",
+		"Status",
+		"Active",
+		"Updated",
+		"Display",
+		"Avatar",
+		"Category",
+		"Unknown"
+	};
+
+	if (type < 10)
+		return fields[type];
+
+	return fields[10];
+}
+#endif
+
+void UpdateLocalPresence(H460P_PresenceNotification & local, const H460P_PresenceNotification & received)
+{
+	H460P_Presentity & l = local.m_presentity;
+	const H460P_Presentity & r = received.m_presentity;
+
+	l.m_state = r.m_state;
+	if (r.HasOptionalField(H460P_Presentity::e_supportedFeatures)) {
+		l.IncludeOptionalField(H460P_Presentity::e_supportedFeatures);
+		l.m_supportedFeatures = r.m_supportedFeatures;
+	}
+	if (r.HasOptionalField(H460P_Presentity::e_geolocation)) {
+		l.IncludeOptionalField(H460P_Presentity::e_geolocation);
+		l.m_geolocation = r.m_geolocation;
+	}
+	if (r.HasOptionalField(H460P_Presentity::e_display)) {
+		l.IncludeOptionalField(H460P_Presentity::e_display);
+		l.m_display = r.m_display;
+	}
+	if (r.HasOptionalField(H460P_Presentity::e_genericData)) {
+		l.IncludeOptionalField(H460P_Presentity::e_genericData);
+		bool found;
+		for (PINDEX i=0; i < r.m_genericData.GetSize(); ++i) {
+		   found = false;
+			for (PINDEX j=0; i < l.m_genericData.GetSize(); ++j) {
+				if (r.m_genericData[i].m_id == l.m_genericData[j].m_id) {
+					l.m_genericData[j] = r.m_genericData[i];
+					found = true;
+					break;
+				}
+			}
+			if (!found) {
+				int sz = l.m_genericData.GetSize();
+				l.m_genericData.SetSize(sz+1);
+				l.m_genericData[sz] = r.m_genericData[i];
+			}
+		}
+	}
+}
+
+bool GkPresence::DatabaseLoad(PBoolean incremental)
 {
 #ifdef HAS_DATABASE
 	GkSQLResult::ResultRow resultRow;
 	std::map<PString, PString> params;
+	params["t"] = m_lastTimeStamp;
 	GkSQLResult* result = m_sqlConn->ExecuteQuery(m_queryList,params, m_timeout);
 	if (result == NULL) {
 		PTRACE(2, "H460PSQL\tSubList failed - timeout or fatal error");
+		SNMP_TRAP(5, SNMPError, Database, "Presense SQL query failed");
 		return false;
 	}
-	if (result->GetNumRows() > 0 && result->GetNumFields() < 7) {
+
+	int fieldCount = 7;
+	if (result->GetNumRows() > 0 && result->GetNumFields() < fieldCount) {
 		PTRACE(2, "H460PSQL\tBad-formed query - "
-			"insufficient columns found in the result set expect 7"
-			);
+			"insufficient columns found in the result set expect " << fieldCount);
+		SNMP_TRAP(5, SNMPError, Database, "Presense SQL query failed");
 		return false;
 	}
 	if (result->GetNumRows() == 0) {
-		PTRACE(2, "H460PSQL\tDatabase returns 0 entries, assume table is empty.");
+		if (!incremental) {
+		  PTRACE(4, "H460PSQL\tDatabase returns 0 entries, assume table is empty.");
+		}
 		return true;
 	}
 
-	// Clear the cache.
-	localStore.clear();
-	remoteIds.clear();
-	remoteIdmap.clear();
+	PTRACE(4, "H460PSQL\tLoad fetched " << result->GetNumRows() << " entries. TimeStamp " << m_lastTimeStamp);
+	m_lastTimeStamp = PTime().GetTimeInSeconds();
 
+	// Clear the cache.
+	if (!incremental) {
+		localStore.clear();
+		remoteIds.clear();
+		remoteIdmap.clear();
+	}
+
+#if PTRACING
+	PINDEX j=0;
+#endif
 	PStringArray retval;
 	while (result->FetchRow(retval)) {
 		if (retval[0].IsEmpty()) {
 			PTRACE(1, "H460PSQL\tQuery Invalid value found.");
+			SNMP_TRAP(5, SNMPError, Database, "Presense SQL query failed");
 			continue;
 		}
-		PTRACE(6, "H460PSQL\tQuery result: " << retval[0] << " " << retval[1]
-								    << " " << retval[2] << " " << retval[3]
-									<< " " << retval[4] << " " << retval[5]
-									<< " " << retval[6] << " " << retval[7]
-									);
 
-									
+#if PTRACING
+		PStringStream results;
+		for (PINDEX i=0; i < retval.GetSize(); ++i) {
+			results << " " << presenceFieldName(i) << " "; 
+			if (i == 4) results << H323PresenceInstruction::GetInstructionString(retval[i].AsInteger());
+#if H460P_VER >= 2
+			else if (i == 9) results << H323PresenceInstruction::GetCategoryString(retval[i].AsInteger());
+#endif
+			else results << retval[i];
+		}
+		PTRACE(6, "H460PSQL\tR: " << j << " " << results);
+		j++;
+#endif
+
 		H460P_PresenceIdentifier & pid = AsPresenceId(retval[0]);
 
 		H323PresenceID id;
@@ -496,29 +605,51 @@ bool GkPresence::DatabaseLoad()
 			id.m_Status = (H323PresenceInstruction::Instruction)retval[4].AsInteger();
 			id.m_Active = Toolkit::AsBool(retval[5]);
 			id.m_Updated = PTime(retval[6]);
+#if H460P_VER >= 2
+			if (retval.GetSize() >= 8) id.m_Display = retval[7];
+								else id.m_Display = PString();
+			if (retval.GetSize() >= 9) id.m_Avatar = retval[8];
+								else id.m_Avatar = PString();
+			if (retval.GetSize() >= 10) id.m_Category = (H323PresenceInstruction::Category)retval[9].AsInteger();
+								else id.m_Category = H323PresenceInstruction::e_UnknownCategory;
+#endif
 		
+#if H460P_VER < 2
+			H323PresenceInstruction instruct(id.m_Status, retval[2]);
+#else
+			H323PresenceInstruction instruct(id.m_Status, retval[2], id.m_Display, id.m_Avatar, id.m_Category);
+#endif
 		// Load the local Store with active or subscriber pending subscriptions
 		if (id.m_Active || (id.m_isSubscriber && (id.m_Status == H323PresenceInstruction::e_pending))) {
 			H323PresenceStore::iterator itx = localStore.find(id.m_subscriber);
 			if (itx != localStore.end()) {
-				if (id.m_subscriber != id.m_Alias) {
-					H323PresenceInstruction instruct(id.m_Status, retval[2]);
+				if (id.m_subscriber != id.m_Alias) 
 					itx->second.m_Instruction.Add(instruct);
-				}
 			} else {
 				H323PresenceEndpoint store;
-				store.m_Notify.SetSize(1);
 				if (id.m_subscriber != id.m_Alias) {
-					H323PresenceInstruction instruct(id.m_Status, retval[2]);
-					store.m_Instruction.Add(instruct);
+						// Set Default Notification
+						H460P_PresenceNotification n;
+						n.m_presentity.m_state = H460P_PresenceState::e_offline;
+						store.m_Notify.SetSize(1);
+						UpdateLocalPresence(store.m_Notify[0],n);
+						// Load Instructions
+						store.m_Instruction.Add(instruct);
+					localStore.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(id.m_subscriber,store));
 				}
-				localStore.insert(pair<H225_AliasAddress,H323PresenceEndpoint>(id.m_subscriber,store));
 			}
 		}
 
 		if (id.m_subscriber == id.m_Alias)
 			continue;
 
+		if (incremental && IsLocalAvailable(id.m_subscriber,aliasList)) {
+			// Queue up any new messages
+			H460P_PresencePDU msg;
+			BuildInstructionMsg(instruct,msg);
+			EnQueuePresence(id.m_subscriber,msg);
+		}
+
 		// Load the identifier store
 		AddPresenceMap(pid,id,remoteIds,remoteIdmap);
 	}
@@ -530,6 +661,14 @@ bool GkPresence::DatabaseLoad()
 #endif
 }
 
+void GkPresence::DatabaseIncrementalUpdate()
+{
+#if HAS_DATABASE
+	if (m_incrementalUpdate)
+		DatabaseLoad(true);
+#endif
+}
+
 bool GkPresence::DatabaseAdd(const PString & identifier, const H323PresenceID & id)
 {
 #ifdef HAS_DATABASE
@@ -542,6 +681,11 @@ bool GkPresence::DatabaseAdd(const PString & identifier, const H323PresenceID &
 	params["u"] = AsString(id.m_subscriber,0);
 	params["a"] = AsString(id.m_Alias,0);
 	params["s"] = (id.m_isSubscriber ? "1" : "0");
+#if H460P_VER >= 2
+	params["d"] = id.m_Display;
+	params["v"] = id.m_Avatar;
+	params["c"] = id.m_Category;
+#endif
 
 	return (m_sqlConn->ExecuteQuery(m_queryAdd, params, m_timeout) != NULL);
 #else
@@ -592,7 +736,7 @@ bool GkPresence::HandleNewAlias(const H225_AliasAddress & addr)
 		id.m_Active = true;
 		id.m_isSubscriber = true;
 
-	DatabaseAdd(uid.AsString(),id);
+	DatabaseAdd(uid.AsString().ToUpper(),id);
     DatabaseUpdate(H323PresenceInstruction::e_subscribe,uid.AsString());
 
 	// load the cache with empty store
@@ -607,7 +751,11 @@ H460P_PresenceSubscription & GkPresence::HandleSubscription(bool isNew, const H4
 		HandleStatusUpdates(pid, id.m_subscriber, e_pending, id.m_Alias, &id);
 
 	H460P_PresencePDU msg;
-    H460P_PresenceSubscription & sub = BuildSubscriptionMsg(pid.m_guid,id.m_Alias,id.m_subscriber,msg);
+#if H460P_VER < 2
+	H460P_PresenceSubscription & sub = BuildSubscriptionMsg(pid.m_guid,id.m_Alias,id.m_subscriber,msg);
+#else
+	H460P_PresenceSubscription & sub = BuildSubscriptionMsg(pid.m_guid,id,msg);
+#endif
 	EnQueuePresence(id.m_subscriber,msg);
 	return *(H460P_PresenceSubscription *)sub.Clone();
 }
@@ -630,6 +778,12 @@ bool GkPresence::RegisterEndpoint(const H225_EndpointIdentifier & ep, const H225
 			continue;
 		}
 
+		// Change the status to Online....
+		if (itx->second.m_Notify.GetSize() > 0) {
+			H460P_PresenceNotification & n = itx->second.m_Notify[0];
+			n.m_presentity.m_state = H460P_PresenceState::e_online;
+		}
+
 		// Queue up any notifications of existing registrations
 		for (PINDEX i=0; i< itx->second.m_Instruction.GetSize(); i++) {
 		   // Queue up info from the datastore to supply to the endpoint
@@ -638,9 +792,14 @@ bool GkPresence::RegisterEndpoint(const H225_EndpointIdentifier & ep, const H225
 			EnQueuePresence(addr[j],msg);
 
 			// look in the local store to see if we can send presence info to people logged in.
-			if ((itx->second.m_Instruction[i].GetTag() == e_subscribe) &&
-										(IsLocalAvailable(itx->second.m_Instruction[i],aliasList)))
-						EnQueueFullNotification(itx->second.m_Instruction[i],addr[j]);
+#if H460P_VER < 2
+			const H225_AliasAddress & alias = itx->second.m_Instruction[i];
+#else
+			const H460P_PresenceAlias & palias = itx->second.m_Instruction[i];
+			const H225_AliasAddress & alias = palias.m_alias;
+#endif
+			if (itx->second.m_Instruction[i].GetTag() == e_subscribe)
+						EnQueueFullNotification(alias,addr[j]);
 		}
 
 		// Check for pending subscriptions
@@ -709,7 +868,7 @@ bool GkPresence::GetSubscriptionIdentifier(const H225_AliasAddress & local, cons
 {
 	 H323PresenceIdMap::const_iterator i = remoteIdmap.find(local);
 	 if (i != remoteIdmap.end()) {
-		 H323PresencePending::const_iterator j = i->second.find(local);
+		 H323PresencePending::const_iterator j = i->second.find(remote);
 		 if (j != i->second.end()) {
 			id = j->second;
 			return true;
@@ -747,20 +906,11 @@ void GkPresence::ProcessPresenceElement(const PASN_OctetString & pdu)
 	PWaitAndSignal m(m_AliasMutex);
 
 	if (!ReceivedPDU(pdu)) {
-		PTRACE(4,"H460P\tError processing PDU");
+		PTRACE(4, "H460P\tError processing PDU");
+		SNMP_TRAP(9, SNMPError, Network, "Error decoding Presense PDU");
 	}
 }
 
-void GkPresence::ProcessPresenceElement(const PASN_OctetString & pdu, const H225_TransportAddress & ip)
-{
-	PWaitAndSignal m(m_AliasMutex);
-
-	if (!ReceivedPDU(pdu)) {
-		PTRACE(4,"H460P\tError processing PDU");
-	}
-}
-
-
 bool GkPresence::BuildPresenceElement(unsigned msgtag, const H225_EndpointIdentifier & ep, PASN_OctetString & pdu)
 {
 	PWaitAndSignal m(m_AliasMutex);
@@ -777,16 +927,34 @@ bool GkPresence::BuildPresenceElement(unsigned msgtag, const H225_TransportAddre
 
 bool GkPresence::EnQueueFullNotification(const H225_AliasAddress & local, const H225_AliasAddress & remote)
 {
-    H323PresenceStore::const_iterator itm = localStore.find(local);
-	if (itm != localStore.end()) {
-		H460P_PresencePDU msg;
-		H460P_PresenceNotification & notification = BuildNotificationMsg(itm->second.m_Notify[0],msg);
-		notification.IncludeOptionalField(H460P_PresenceNotification::e_aliasAddress);
-		notification.m_aliasAddress = local;
-		EnQueuePresence(remote,msg);
+	// Send local status to remote
+    H323PresenceStore::iterator itm = localStore.find(local);
+	if (itm == localStore.end())
+        return false;
+
+	H460P_PresencePDU msg;
+    if (itm->second.m_Notify.GetSize() == 0) 
+		return false;
+
+	H460P_PresenceNotification & notification = BuildNotificationMsg(itm->second.m_Notify[0],msg);
+	notification.IncludeOptionalField(H460P_PresenceNotification::e_aliasAddress);
+	notification.m_aliasAddress = local;
+	EnQueuePresence(remote,msg);
+
+	// Send remote status to local
+	H323PresenceStore::iterator itx = localStore.find(remote);
+	if (itx == localStore.end())
 		return true;
-	}
-	return false;
+
+	H460P_PresencePDU msg2;
+	if (itx->second.m_Notify.GetSize() == 0)
+		return true;
+
+	H460P_PresenceNotification & notification2 = BuildNotificationMsg(itx->second.m_Notify[0],msg2);
+	notification2.IncludeOptionalField(H460P_PresenceNotification::e_aliasAddress);
+	notification2.m_aliasAddress = remote;
+	EnQueuePresence(local,msg2);
+	return true;
 }
 
 bool GkPresence::EnQueuePresence(const H225_AliasAddress & addr, const H460P_PresencePDU & msg)
@@ -840,7 +1008,7 @@ bool GkPresence::EnQueuePresence(const H225_AliasAddress & addr, const H460P_Pre
 	}
 
 	// neither registered remote or locally
-	PTRACE(2,"PRES\tPresence " << PresMsgType(msg.GetTag()) << " to " << addr << " dropped as no destination resolved.");
+	PTRACE(2, "PRES\tPresence " << PresMsgType(msg.GetTag()) << " to " << addr << " dropped as no destination resolved.");
 	return false;
 }
 
@@ -1133,44 +1301,6 @@ PBoolean GkPresence::BuildIdentifiers(bool alive, const H225_TransportAddress &
 	return found;
 }
 
-void UpdateLocalPresence(H460P_PresenceNotification & local, const H460P_PresenceNotification & received)
-{
-	H460P_Presentity & l = local.m_presentity;
-	const H460P_Presentity & r = received.m_presentity;
-
-	l.m_state = r.m_state;
-	if (r.HasOptionalField(H460P_Presentity::e_supportedFeatures)) {
-		l.IncludeOptionalField(H460P_Presentity::e_supportedFeatures);
-		l.m_supportedFeatures = r.m_supportedFeatures;
-	}
-	if (r.HasOptionalField(H460P_Presentity::e_geolocation)) {
-		l.IncludeOptionalField(H460P_Presentity::e_geolocation);
-		l.m_geolocation = r.m_geolocation;
-	}
-	if (r.HasOptionalField(H460P_Presentity::e_display)) {
-		l.IncludeOptionalField(H460P_Presentity::e_display);
-		l.m_display = r.m_display;
-	}
-	if (r.HasOptionalField(H460P_Presentity::e_genericData)) {
-		l.IncludeOptionalField(H460P_Presentity::e_genericData);
-		bool found;
-		for (PINDEX i=0; i < r.m_genericData.GetSize(); ++i) {
-		   found = false;
-			for (PINDEX j=0; i < l.m_genericData.GetSize(); ++j) {
-				if (r.m_genericData[i].m_id == l.m_genericData[j].m_id) {
-					l.m_genericData[j] = r.m_genericData[i];
-					found = true;
-					break;
-				}
-			}
-			if (!found) {
-				int sz = l.m_genericData.GetSize();
-				l.m_genericData.SetSize(sz+1);
-				l.m_genericData[sz] = r.m_genericData[i];
-			}
-		}
-	}
-}
 
 bool GkPresence::HandleStatusUpdates(const H460P_PresenceIdentifier & pid, const H225_AliasAddress & local,
 									 unsigned type, const H225_AliasAddress & remote,
@@ -1186,28 +1316,23 @@ bool GkPresence::HandleStatusUpdates(const H460P_PresenceIdentifier & pid, const
 			switch (type) {
 			  case H460P_PresenceInstruction::e_subscribe:
 				  DatabaseUpdate(type,spid);
-				  UpdateInstruction(inst,it->second.m_Instruction);
 				  break;
 
 			  case H460P_PresenceInstruction::e_unsubscribe:
-				  DatabaseDelete(spid);
-				  RemovePresenceMap(pid, remoteIds, remoteIdmap);
-				  RemoveInstruction(remote,it->second.m_Instruction);
+				  DatabaseUpdate(type,spid);
 				  break;
 
 			  case H460P_PresenceInstruction::e_block:
 				  if (id) {
 					DatabaseAdd(spid,*id);
-					DatabaseUpdate(type,AsPresenceString(pid));
+					DatabaseUpdate(type,spid);
 					AddPresenceMap(pid, *id, remoteIds, remoteIdmap);
 					UpdateInstruction(inst,it->second.m_Instruction);
 				  }
 				  break;
 
 			  case H460P_PresenceInstruction::e_unblock:
-				  DatabaseDelete(spid);
-				  RemovePresenceMap(pid, remoteIds, remoteIdmap);
-  				  RemoveInstruction(remote,it->second.m_Instruction);
+				  DatabaseUpdate(type,spid);
 				  break;
 
 			  case H460P_PresenceInstruction::e_pending:
@@ -1220,7 +1345,7 @@ bool GkPresence::HandleStatusUpdates(const H460P_PresenceIdentifier & pid, const
 			}
 		}
 
-		PTRACE(4,"PRES\tChanged Subscription for " << local << " : " << PresMsgType(type) <<  " " << remote);
+		PTRACE(4, "PRES\tChanged Subscription for " << local << " : " << PresMsgType(type) <<  " " << remote);
 
 		if (!IsLocalAvailable(local,aliasList))
 			return true;
@@ -1236,10 +1361,10 @@ bool GkPresence::HandleForwardPresence(const H460P_PresenceIdentifier & identifi
 		m_Indication.push_back(msg);
 		H323PresenceInd xlist;
 		H225_AliasAddress a;
-		H323SetAliasAddress(PString("Relay"),a);
-		PTRACE(5,"PRES\tRelaying Identifier " << identifier << " to " << itx->second);
-		xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(a,m_Indication));	
-		remoteStore.insert(pair<H225_TransportAddress,H323PresenceInd>(itx->second,xlist));
+		H323SetAliasAddress(PString("Relay"), a);
+		PTRACE(5, "PRES\tRelaying Identifier " << identifier << " to " << itx->second);
+		xlist.insert(pair<H225_AliasAddress,list<H460P_PresencePDU> >(a, m_Indication));	
+		remoteStore.insert(pair<H225_TransportAddress,H323PresenceInd>(itx->second, xlist));
 		return true;
 	}
 	return false;
@@ -1252,7 +1377,8 @@ bool GkPresence::HandleSubscriptionLocal(const H460P_PresenceSubscription & subs
 	H323PresenceIds::const_iterator it = remoteIds.find(subscription.m_identifier);
 	if (it != remoteIds.end()) {
 		if (!subscription.HasOptionalField(H460P_PresenceSubscription::e_approved)) {
-			PTRACE(4,"PRES\tLOGIC ERROR: Received a subscription reply but not subscriber and no approval indication");
+			PTRACE(4, "PRES\tLOGIC ERROR: Received a subscription reply but not subscriber and no approval indication");
+			SNMP_TRAP(7, SNMPError, Authentication, "Received a Presence subscription reply but not subscriber and no approval indication");
 			return false;
 		}
 		approved = subscription.m_approved;
@@ -1271,7 +1397,16 @@ bool GkPresence::HandleSubscriptionLocal(const H460P_PresenceSubscription & subs
 		itx->second.m_Authorize.Add((H323PresenceSubscription &)subscription);
 		H323PresenceID id;
 			id.m_subscriber = subscription.m_subscribe;
+#if H460P_VER < 2
 			id.m_Alias = subscription.m_aliases[0];
+#else
+			const H460P_PresenceAlias & pAlias = subscription.m_aliases[0];
+			id.m_Alias = pAlias.m_alias;
+			if (pAlias.HasOptionalField(H460P_PresenceAlias::e_display)) {
+				const H460P_PresenceDisplay & pDisp = pAlias.m_display;
+				id.m_Display = pDisp.m_display.GetValue();
+			}
+#endif
 			id.m_Active = false;
 			id.m_isSubscriber = false;
 		HandleSubscription(true,subscription.m_identifier,id);
@@ -1286,13 +1421,13 @@ bool GkPresence::RemoveSubscription(unsigned type,const H460P_PresenceIdentifier
 	if (it != remoteIds.end()) {
 		H323PresenceStore::iterator it1 = localStore.find(it->second.m_subscriber);
 		if (it1 != localStore.end())
-			HandleStatusUpdates(pid,it->second.m_subscriber, type, it->second.m_Alias);
+			HandleStatusUpdates(pid,it1->first, type, it->second.m_Alias);
 
 		H323PresenceStore::iterator it2 = localStore.find(it->second.m_Alias);
 		if (it2 != localStore.end())
-			HandleStatusUpdates(pid,it->second.m_Alias,type,it->second.m_subscriber);
+			HandleStatusUpdates(pid,it2->first,type,it->second.m_subscriber);
 
-	   PTRACE(4,"PRES\tRemoved Subscription " << PresMsgType(type) << " : " << it->second.m_subscriber << " to " << it->second.m_Alias);
+	   PTRACE(4, "PRES\tRemoved Subscription " << PresMsgType(type) << " : " << it->second.m_subscriber << " to " << it->second.m_Alias);
 	   return true;
 	}
 	return false;
@@ -1301,30 +1436,44 @@ bool GkPresence::RemoveSubscription(unsigned type,const H460P_PresenceIdentifier
 bool GkPresence::HandleNewInstruction(unsigned tag, const H225_AliasAddress & addr,
 									  const H460P_PresenceInstruction & instruction, H323PresenceInstructions & instructions)
 {
-
+#if H460P_VER < 2
 	const H225_AliasAddress & a = instruction;
+#else
+	const H460P_PresenceAlias & palias = instruction;
+	const H225_AliasAddress & a = palias.m_alias;
+	PString displayName = PString();
+#endif
 
 	if (a == addr)
 		return true;
 
 	// Check to see if we already have this instruction
 	for (PINDEX i=0; i< instructions.GetSize(); i++) {
-		H225_AliasAddress & b = instructions[i];
+#if H460P_VER < 2
+		const H225_AliasAddress & b = instruction;
+#else
+		const H460P_PresenceAlias & pb = instruction;
+		const H225_AliasAddress & b = pb.m_alias;
+		if (pb.HasOptionalField(H460P_PresenceAlias::e_display)) {
+			const H460P_PresenceDisplay & pd = pb.m_display;
+			displayName = pd.m_display.GetValue();
+		}
+#endif
 		if (a == b) {
-			if (instruction.GetTag() == instructions[i].GetTag())
-				return true;
+//			if (tag == instructions[i].GetTag())
+//				return true;
 
-			if (instruction.GetTag() == H460P_PresenceInstruction::e_unsubscribe) {
+			if (tag == H460P_PresenceInstruction::e_unsubscribe) {
 				H460P_PresenceIdentifier pid;
 				if (GetSubscriptionIdentifier(addr, a, pid))
 					return RemoveSubscription(tag,pid);
 			}
-			if (instruction.GetTag() == H460P_PresenceInstruction::e_unblock) {
+			if (tag == H460P_PresenceInstruction::e_unblock) {
 				H460P_PresenceIdentifier pid;
 				if (GetSubscriptionIdentifier(addr, a, pid))
 					return HandleStatusUpdates(pid, addr, e_unblock, a);
 			}
-			PTRACE(4,"PRES\tPresence Instruction : " << instruction << " from " << addr << " not handled!");
+			PTRACE(4, "PRES\tPresence Instruction : " << instruction << " from " << addr << " not handled!");
 			return false;
 		}
 	}
@@ -1339,21 +1488,27 @@ bool GkPresence::HandleNewInstruction(unsigned tag, const H225_AliasAddress & ad
 			idx.m_Updated = PTime();
 			idx.m_Active = false;
 
-			if ((tag == H460P_PresenceInstruction::e_subscribe)) {
+			if (tag == H460P_PresenceInstruction::e_subscribe) {
 				//Build Remote Subscription message
 					idx.m_isSubscriber = false;
 					idx.m_subscriber = a;
 					idx.m_Alias = addr;
+#if H460P_VER >= 2
+					idx.m_Display = displayName;
+#endif
 				HandleSubscription(true,pid,idx);
 
 				//Build local side
 				idx.m_isSubscriber = true;
 				idx.m_subscriber = addr;
 				idx.m_Alias = a;
+#if H460P_VER >= 2
+				idx.m_Display = PString();
+#endif
 				HandleStatusUpdates(pid, addr, e_pending, a, &idx);
 			}
 
-			if ((tag == H460P_PresenceInstruction::e_block)) {
+			if (tag == H460P_PresenceInstruction::e_block) {
 				idx.m_isSubscriber = true;
 				idx.m_subscriber = addr;
 				idx.m_Alias = a;
@@ -1368,17 +1523,24 @@ bool GkPresence::HandleNewInstruction(unsigned tag, const H225_AliasAddress & ad
 
 void GkPresence::OnNotification(MsgType tag, const H460P_PresenceNotification & notify, const H225_AliasAddress & addr)
 {
-	if (tag == e_Status) {
+	if (tag == e_Status || tag ==  e_Notify) {
 		// Update the local Store
 		H323PresenceStore::iterator itx = localStore.find(addr);
 		if (itx != localStore.end()) {
 			H323PresenceEndpoint & ep = itx->second;
-			ep.m_Notify.SetSize(1);
-			UpdateLocalPresence(ep.m_Notify[0], notify);
+			if (ep.m_Notify.GetSize() == 0) {
+				ep.m_Notify.SetSize(1);
+				UpdateLocalPresence(ep.m_Notify[0], notify);
+			}
 
 			for (PINDEX i=0; i < ep.m_Instruction.GetSize(); i++) {
 				if (ep.m_Instruction[i].GetTag() == H460P_PresenceInstruction::e_subscribe) {
-					H225_AliasAddress & a = ep.m_Instruction[i];
+#if H460P_VER < 2
+					const H225_AliasAddress & a = ep.m_Instruction[i];
+#else
+					const H460P_PresenceAlias & palias = ep.m_Instruction[i];
+					const H225_AliasAddress & a = palias.m_alias;
+#endif
 					H460P_PresencePDU msg;
 					H460P_PresenceNotification & notification = BuildNotificationMsg(notify,msg);
 					notification.IncludeOptionalField(H460P_PresenceNotification::e_aliasAddress);
@@ -1409,20 +1571,26 @@ void GkPresence::OnSubscription(MsgType tag, const H460P_PresenceSubscription &
 		H460P_PresencePDU msg;
 		BuildSubscriptionMsg(subscription,msg);
 		if (!HandleForwardPresence(subscription.m_identifier, msg)) {
-			PTRACE(2,"PRES\tSubscription received " << subscription.m_identifier.m_guid << " from " << AsString(addr,0) << " not handled");
+			PTRACE(2, "PRES\tSubscription received " << subscription.m_identifier.m_guid << " from " << AsString(addr,0) << " not handled");
 			return;
 		}
 	}
 
 	// Handle the senders side
 	for (PINDEX i=0; i<subscription.m_aliases.GetSize(); i++) {
-		if (IsLocalPresence(subscription.m_aliases[i],localStore)) {
+#if H460P_VER < 2
+		const H225_AliasAddress & sub = subscription.m_aliases[i];
+#else
+		const H460P_PresenceAlias & palias = subscription.m_aliases[i];
+		const H225_AliasAddress & sub = palias.m_alias;
+#endif
+		if (IsLocalPresence(sub,localStore)) {
 			if (approved) {
-				HandleStatusUpdates(subscription.m_identifier, subscription.m_aliases[i], e_subscribe, subscription.m_subscribe);
-				if (IsLocalAvailable(subscription.m_aliases[i],aliasList))
-					EnQueueFullNotification(subscription.m_aliases[i], subscription.m_subscribe);
+				HandleStatusUpdates(subscription.m_identifier, sub, e_subscribe, subscription.m_subscribe);
+				if (IsLocalAvailable(sub,aliasList))
+					EnQueueFullNotification(sub, subscription.m_subscribe);
 			} else
-				HandleStatusUpdates(subscription.m_identifier, subscription.m_aliases[i], e_unsubscribe, subscription.m_subscribe);
+				HandleStatusUpdates(subscription.m_identifier, sub, e_unsubscribe, subscription.m_subscribe);
 		}
 	}
 }
@@ -1438,7 +1606,7 @@ void GkPresence::OnNotification(MsgType tag, const H460P_PresenceNotification &
 		H323PresenceIds::const_iterator it = remoteIds.find(notify.m_subscribers[i]);
 		if (it != remoteIds.end()) {
 			H225_AliasAddress addr = it->second.m_subscriber;
-			PTRACE(5,"PRES\tReceived Notification " << notify.m_subscribers[i] << " for " << addr << " from " << ip);
+			PTRACE(5, "PRES\tReceived Notification " << notify.m_subscribers[i] << " for " << addr << " from " << ip);
 			H460P_PresencePDU msg;
 			H460P_PresenceNotification & notification = BuildNotificationMsg(notify,msg);
 			notification.RemoveOptionalField(H460P_PresenceNotification::e_subscribers);
@@ -1446,12 +1614,12 @@ void GkPresence::OnNotification(MsgType tag, const H460P_PresenceNotification &
 		} else {
 			// not one of ours see if we are passing on.
 			H460P_PresencePDU msg;
-			H460P_PresenceNotification & notification = BuildNotificationMsg(notify,msg);
+			H460P_PresenceNotification & notification = BuildNotificationMsg(notify, msg);
 			notification.m_subscribers.SetSize(1);
 			notification.m_subscribers[0] = notify.m_subscribers[i];
-            if (!HandleForwardPresence(notification.m_subscribers[0], msg)) {
-		        PTRACE(4,"PRES\tUnknown Notification received " << notification.m_subscribers[0] << " from " << ip << " disgarding.");
-		    }
+			if (!HandleForwardPresence(notification.m_subscribers[0], msg)) {
+				PTRACE(4, "PRES\tUnknown Notification received " << notification.m_subscribers[0] << " from " << ip << " disgarding.");
+			}
 		}
 	}
 }
@@ -1463,7 +1631,7 @@ void GkPresence::OnSubscription(MsgType tag,const H460P_PresenceSubscription & s
 		H460P_PresencePDU msg;
 		BuildSubscriptionMsg(subscription,msg);
 		if (!HandleForwardPresence(subscription.m_identifier, msg)) {
-			PTRACE(4,"PRES\tUnknown Subscription received " << subscription.m_identifier << " from " << ip << " disgarding.");
+			PTRACE(4, "PRES\tUnknown Subscription received " << subscription.m_identifier << " from " << ip << " disgarding.");
 			return;
 		}
 	}
@@ -1478,7 +1646,7 @@ void GkPresence::OnIdentifiers(MsgType tag, const H460P_PresenceIdentifier & ide
 
 		if (tag == e_Alive) {
 			/// keepAlive
-  			PTRACE(5,"PRES\tReceived KeepAlive " << identifier << " for " << id.m_subscriber << " from " << ip);
+  			PTRACE(5, "PRES\tReceived KeepAlive " << identifier << " for " << id.m_subscriber << " from " << ip);
 		}
 
 		if (tag == e_Remove) {
@@ -1491,7 +1659,7 @@ void GkPresence::OnIdentifiers(MsgType tag, const H460P_PresenceIdentifier & ide
 		H460P_PresencePDU msg;
 		BuildIdentifierMsg(identifier,msg);
 		if (!HandleForwardPresence(identifier, msg)) {
-			PTRACE(4,"PRES\tUnknown Identifier received " << identifier << " from " << ip << " disgarding.");
+			PTRACE(4, "PRES\tUnknown Identifier received " << identifier << " from " << ip << " disgarding.");
 		}
 	}
 }
