@@ -42,7 +42,6 @@ public:
 	bool BuildPresenceElement(unsigned msgtag, const H225_EndpointIdentifier & ep, PASN_OctetString & pdu);
 	bool BuildPresenceElement(unsigned msgtag, const H225_TransportAddress & ip, PASN_OctetString & pdu);
 	void ProcessPresenceElement(const PASN_OctetString & pdu);
-	void ProcessPresenceElement(const PASN_OctetString & pdu, const H225_TransportAddress & ip);
 
 	bool GetPendingIdentifiers(list<H225_EndpointIdentifier> & epid);
 	bool GetPendingAddresses(list<H225_TransportAddress> & gkip);
@@ -57,6 +56,7 @@ public:
 	bool GetLocalSubscriptions(const H225_AliasAddress & local,
 							list<H460P_PresenceIdentifier> & id);
 
+	void DatabaseIncrementalUpdate();
 
 protected:
 
@@ -138,7 +138,7 @@ protected:
 	bool RemoveSubscription(unsigned type, const H460P_PresenceIdentifier & pid);
 
   // Database Functions
-	bool DatabaseLoad();
+	bool DatabaseLoad(PBoolean incremental);
 	bool DatabaseAdd(const PString & identifier, const H323PresenceID & id);
 	bool DatabaseDelete(const PString & identifier);
 	bool DatabaseUpdate(unsigned tag, const PString & identifier);
@@ -175,6 +175,8 @@ private:
 
 	// query timeout
 	long					m_timeout;
+	PInt64					m_lastTimeStamp;
+	PBoolean				m_incrementalUpdate;
 #endif
 
 };
