@@ -2,7 +2,7 @@
 //
 // pwlib_compat.h PWLib compatibility header
 //
-// Copyright (c) 2006-2011, Jan Willamowius
+// Copyright (c) 2006-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +13,7 @@
 
 
 #ifndef PWLIB_COMPAT_H
-#define PWLIB_COMPAT_H "@(#) $Id: pwlib_compat.h,v 1.50 2011/09/23 09:28:52 willamowius Exp $"
+#define PWLIB_COMPAT_H "@(#) $Id: pwlib_compat.h,v 1.75 2012/12/28 20:37:05 willamowius Exp $"
 
 #include "ptbuildopts.h"
 #include "openh323buildopts.h"
@@ -26,6 +26,10 @@
 	#endif
 #endif
 
+#ifndef PTRACING
+	#error "Enable PTRACING in PTLIB to compile GnuGk"
+#endif
+
 // check for PConfig support
 #ifndef P_CONFIG_FILE
 	#error "Make sure PTLib has config file support enabled, avoid --disable-configfile or --enable-openh323 or --enable-minsize etc."
@@ -45,12 +49,6 @@
 	#endif
 #endif
 
-// define PWCharArray for PWLib < 2.x.x
-#if PTLIB_MAJOR < 2
-	#define PWCharArray PWORDArray
-#endif
-
-
 #if !defined(P_USE_STANDARD_CXX_BOOL) && !defined(P_USE_INTEGER_BOOL)
 	typedef int PBoolean;
 #endif
@@ -83,17 +81,21 @@
 	#endif
 #endif
 
-#if PWLIB_MAJOR == 1
-	#if PWLIB_MINOR == 12
-		#define hasRDS 1 
-		#define hasSETENUMSERVERS 1    
+#ifdef P_DNS
+	#if PWLIB_MAJOR == 1
+		#if PWLIB_MINOR == 12
+			#define hasRDS 1
+			#define hasSETENUMSERVERS 1
+		#endif
 	#endif
 #endif
 
 // for PTlib v2.x
 #ifdef PTLIB_MAJOR 
-	#define hasRDS 1 
-	#define hasSETENUMSERVERS 1    
+	#ifdef P_DNS
+		#define hasRDS 1
+		#define hasSETENUMSERVERS 1
+	#endif
 	#define hasDeletingSetStream 1
 #endif
 
@@ -106,11 +108,17 @@
 	#if PTLIB_MINOR >= 8
 		#define hasDynaLinkGetLastError 1
 	#endif
+	#if PTLIB_MINOR == 9
+		#define hasPTLibTraceOnShutdownBug	1
+	#endif
 	#if PTLIB_MINOR >= 10
 		#ifdef P_HAS_IPV6
 			// IPv6 support before 2.10.0 is too buggy
 			#define hasIPV6     1
 		#endif
+		#if PTLIB_BUILD >= 6
+			#define hasLDAPStartTLS 1
+		#endif
 	#endif
 	#if PTLIB_MINOR >= 11
 		#define hasLDAPStartTLS 1
@@ -127,6 +135,12 @@
 	#endif
 #endif
 
+#if defined(hasPTRACE2)
+   #define PTRACEX(level, args)  PTRACE2(level,NULL,args)
+#else
+   #define PTRACEX(level, args)  PTRACE(level,args)   
+#endif
+
 #if !defined(PWLIB_MAJOR) && !defined(PTLIB_MAJOR)
 	#if _WIN32
 		#pragma message ("warning: Can't detect PTLib version")
@@ -137,7 +151,7 @@
 	#define hasDeletingSetStream 1
 #endif
 
-// store version numver it PT macros for dispaly later on
+// store version number it PT macros for display later on
 #if !defined(PTLIB_MAJOR)
 	#define PTLIB_MAJOR	PWLIB_MAJOR
 	#define PTLIB_MINOR	PWLIB_MINOR
@@ -148,11 +162,21 @@
 
 // OpenH323/H323Plus version matching
 #ifdef H323_H235
-   #define HAS_H235_MEDIA 1      // H235 Media Encryption Support
+	#define HAS_H235_MEDIA 1      // H.235.6 Media Encryption Support
 #endif
 
 #ifdef H323_H460
 	#define HAS_H460 1    // H460 support
+#else
+	#undef HAS_H46017
+	#undef HAS_H46018
+	#undef HAS_H46023
+#endif
+
+#ifdef HAS_H46017			// config switch H.460.17 must be enabled
+	#ifdef H323_H46026		// feature detection, if library supports H.460.26
+		#define HAS_H46026 1
+	#endif
 #endif
 
 #ifdef HAS_H46023		// config switch if H.460.23 should be enabled
@@ -165,6 +189,13 @@
 
 #ifdef H323_H460P
 	#define HAS_H460P	1  // Presence
+//#ifdef H323_H460P_VER
+	#define H460P_VER 2 // H323_H460P_VER
+	#define OID3 "1.3.6.1.4.1.17090.0.12"  // Presence v2
+//#else
+//	#define H460P_VER 1
+//	#define OID3 "1.3.6.1.4.1.17090.0.3"  // Presence
+//#endif
 #endif
 
 #if OPENH323_MAJOR == 1
@@ -198,10 +229,12 @@
 	#endif	
 #endif
 
-#if defined(hasPTRACE2)
-   #define PTRACEX(level, args)  PTRACE2(level,NULL,args)
-#else
-   #define PTRACEX(level, args)  PTRACE(level,args)   
+#if defined(P_SNMP) || defined(HAS_NETSNMP) || defined(_WIN32)
+#define HAS_SNMP 1
 #endif
 
+#if defined(H323_H450)
+#define HAS_H450 1
 #endif
+
+#endif // PWLIB_COMPAT_H
