@@ -4,7 +4,7 @@
  * RADIUS protocol accounting logger module for GNU Gatekeeper. 
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
- * Copyright (c) 2005-2011, Jan Willamowius
+ * Copyright (c) 2005-2012, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -87,12 +87,12 @@ GkAcctLogger::Status RadAcct::Log(
 		return Next;
 		
 	if (m_radiusClient == NULL) {
-		PTRACE(1,"RADACCT\t"<<GetName()<<" - null RADIUS client instance");
+		PTRACE(1, "RADACCT\t"<<GetName()<<" - null RADIUS client instance");
 		return Fail;
 	}
 
 	if ((evt & (AcctStart | AcctStop | AcctUpdate)) && (!call)) {
-		PTRACE(1,"RADACCT\t"<<GetName()<<" - missing call info for event "<<evt);
+		PTRACE(1, "RADACCT\t"<<GetName()<<" - missing call info for event "<<evt);
 		return Fail;
 	}
 	
@@ -276,16 +276,14 @@ GkAcctLogger::Status RadAcct::Log(
 		
 	// send request and wait for response
 	RadiusPDU * response = NULL;
-	bool result = OnSendPDU(*pdu, evt, call);
-	
+	bool result = true;
+
 	// accounting updates must be fast, so we are just sending
 	// the request to the server and are not waiting for a response
-	if (result) {
-		if (evt & AcctUpdate) {
-			result = m_radiusClient->SendRequest(*pdu);
-		} else {
-			result = m_radiusClient->MakeRequest(*pdu, response) && (response != NULL);
-		}
+	if (evt & AcctUpdate) {
+		result = m_radiusClient->SendRequest(*pdu);
+	} else {
+		result = m_radiusClient->MakeRequest(*pdu, response) && (response != NULL);
 	}
 			
 	delete pdu;
@@ -298,35 +296,16 @@ GkAcctLogger::Status RadAcct::Log(
 	if (response) {
 		// check if Access-Request has been accepted
 		result = (response->GetCode() == RadiusPDU::AccountingResponse);
-		if (result)
-			result = OnReceivedPDU(*response, evt, call);
-		else
+		if (!result) {
 			PTRACE(4, "RADACCT\t" << GetName() << " - received response is not "
 				" an AccountingResponse, event " << evt << ", call no. "
 				<< (call ? call->GetCallNumber() : 0));
+		}
 		delete response;
 	}
 	return result ? Ok : Fail;
 }
 
-bool RadAcct::OnSendPDU(
-	RadiusPDU& /*pdu*/,
-	GkAcctLogger::AcctEvent /*evt*/,
-	const callptr& /*call*/
-	)
-{
-	return true;
-}
-
-bool RadAcct::OnReceivedPDU(
-	RadiusPDU& /*pdu*/,
-	GkAcctLogger::AcctEvent /*evt*/,
-	const callptr& /*call*/
-	)
-{
-	return true;
-}
-
 namespace {
 	// append RADIUS based accounting logger to the global list of loggers
 	GkAcctLoggerCreator<RadAcct> RadAcctCreator("RadAcct");
