@@ -3,7 +3,7 @@
 // yasocket.cxx
 //
 // Copyright (c) Citron Network Inc. 2002-2003
-// Copyright (c) 2004-2011, Jan Willamowius
+// Copyright (c) 2004-2012, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -18,6 +18,7 @@
 #include "rwlock.h"
 #include "yasocket.h"
 #include "Toolkit.h"
+#include "snmp.h"
 #include "gk.h"
 
 using std::mem_fun;
@@ -45,12 +46,12 @@ bool YaSelectList::Select(SelectType t, const PTimeInterval & timeout)
 	const_iterator endIter = fds.end();
 	while (i != endIter)
 		fdset.add((*i++)->GetHandle());
-	
+
 	fd_set *readfds, *writefds;
 	if (t == Read)
-		readfds = fdset, writefds = 0;
+		readfds = fdset, writefds = NULL;
 	else
-		writefds = fdset, readfds = 0;
+		writefds = fdset, readfds = NULL;
 
 	const unsigned long msec = timeout.GetInterval();
 	struct timeval tval;
@@ -367,6 +368,7 @@ bool YaTCPSocket::Listen(const Address & addr, unsigned qs, WORD pt, PSocket::Re
 		os_handle = ::socket(PF_INET6, SOCK_STREAM, 0);
 		if (addr.IsAny() && !SetOption(IPV6_V6ONLY, 0, IPPROTO_IPV6)) {
 			PTRACE(1, "Removing of IPV6_V6ONLY failed");
+			SNMP_TRAP(10, SNMPWarning, Network, "IPv6 error");
 		}
 	}
 	else
@@ -543,6 +545,7 @@ bool YaUDPSocket::Listen(const Address & addr, unsigned, WORD pt, PSocket::Reusa
 		os_handle = ::socket(PF_INET6, SOCK_DGRAM, 0);
 		if (addr.IsAny() && !SetOption(IPV6_V6ONLY, 0, IPPROTO_IPV6)) {
 			PTRACE(1, "Removing of IPV6_V6ONLY failed");
+			SNMP_TRAP(10, SNMPWarning, Network, "IPv6 error");
 		}
 	} else
 #endif
@@ -628,6 +631,11 @@ int YaUDPSocket::os_send(const void * buf, int sz)
 #else // LARGE_FDSET
 
 #ifdef hasIPV6
+
+#ifndef IPV6_V6ONLY
+  #define IPV6_V6ONLY 27
+#endif
+
 bool TCPSocket::DualStackListen(WORD newPort)
 {
 	if (!Toolkit::Instance()->IsIPv6Enabled())
@@ -643,17 +651,20 @@ bool TCPSocket::DualStackListen(WORD newPort)
 	// attempt to create a socket
 	if (!OpenSocket(PF_INET6)) {
 		PTRACE(4, "Socket\tOpenSocket failed");
+		SNMP_TRAP(10, SNMPError, Network, "IPv6 error");
 		return false;
 	}
 
 	// allow IPv4 connects
 	if (!SetOption(IPV6_V6ONLY, 0, IPPROTO_IPV6)) {
 		PTRACE(4, "Socket\tSetOption(IPV6_V6ONLY) failed");
+		SNMP_TRAP(10, SNMPWarning, Network, "IPv6 error");
 	}
 
 	// attempt to listen
 	if (!SetOption(SO_REUSEADDR, 1)) {
 		PTRACE(4, "Socket\tSetOption(SO_REUSEADDR) failed");
+		SNMP_TRAP(10, SNMPWarning, Network, "IPv6 error");
 		os_close();
 		return false;
 	}
@@ -670,6 +681,7 @@ bool TCPSocket::DualStackListen(WORD newPort)
 
 	if (!ConvertOSError(::listen(os_handle, 1))) {
 		PTRACE(4, "Socket\tlisten failed: " << GetErrorText());
+		SNMP_TRAP(10, SNMPError, Network, "DualStack listen failed");
 		os_close();
 		return false;
 	}
@@ -680,6 +692,7 @@ bool TCPSocket::DualStackListen(WORD newPort)
 	socklen_t size = sizeof(sa);
 	if (!ConvertOSError(::getsockname(os_handle, (sockaddr*)&sa, &size))) {
 		PTRACE(4, "Socket\tgetsockname failed: " << GetErrorText());
+		SNMP_TRAP(10, SNMPError, Network, "getsockname() failed");
 		os_close();
 		return false;
 	}
@@ -713,12 +726,14 @@ bool UDPSocket::DualStackListen(const PIPSocket::Address & localAddr, WORD newPo
 	// attempt to create a socket
 	if (!OpenSocket(PF_INET6)) {
 		PTRACE(4, "Socket\tOpenSocket failed");
+		SNMP_TRAP(10, SNMPError, Network, "OpenSocket failed");
 		return false;
 	}
 
 	// allow IPv4 connects
 	if (!SetOption(IPV6_V6ONLY, 0, IPPROTO_IPV6)) {
 		PTRACE(4, "Socket\tSetOption(IPV6_V6ONLY) failed");
+		SNMP_TRAP(10, SNMPWarning, Network, "SetOption failed");
 	}
 
 	sockaddr_in6 sa;
@@ -737,6 +752,7 @@ bool UDPSocket::DualStackListen(const PIPSocket::Address & localAddr, WORD newPo
 	socklen_t size = sizeof(sa);
 	if (!ConvertOSError(::getsockname(os_handle, (sockaddr*)&sa, &size))) {
 		PTRACE(4, "Socket\tgetsockname failed: " << GetErrorText());
+		SNMP_TRAP(10, SNMPError, Network, "getsockname() failed");
 		os_close();
 		return false;
 	}
@@ -832,7 +848,13 @@ bool USocket::WriteData(const BYTE * buf, int len)
 		return false;
 		
 	int remaining = len;
-	if (qsize == 0 && !writeMutex.WillBlock()) {
+	if (qsize == 0 && 
+#if PTLIB_VER < 2120
+		!writeMutex.WillBlock()
+#else
+		writeMutex.Try()
+#endif
+		) {
 		PWaitAndSignal lock(writeMutex);
 		while (remaining > 0) {
 			int sendnow = remaining > MAX_SOCKET_CHUNK
@@ -871,6 +893,7 @@ bool USocket::ErrorHandler(PSocket::ErrorGroup group)
 	{
 		case PSocket::Timeout:
 			PTRACE(4, msg << " Error(" << group << "): Timeout");
+			SNMP_TRAP(10, SNMPError, Network, "Socket timeout: " + Name());
 			break;
 		case PSocket::NoError:
 			if (group == PSocket::LastReadError) {
@@ -881,8 +904,8 @@ bool USocket::ErrorHandler(PSocket::ErrorGroup group)
 		default:
 			PTRACE(3, msg << " Error(" << group << "): "
 				<< PSocket::GetErrorText(e) << " (" << e << ':'
-				<< self->GetErrorNumber(group) << ')'
-				);
+				<< self->GetErrorNumber(group) << ')');
+			SNMP_TRAP(10, SNMPError, Network, "Socket error " + PSocket::GetErrorText(e));
 			CloseSocket();
 			break;
 	}
@@ -903,7 +926,7 @@ bool USocket::InternalWriteData(const BYTE * buf, int len)
 		ErrorHandler(PSocket::LastWriteError);
 	if (IsSocketOpen()) {
 		PTRACE(4, type << '\t' << Name() << " blocked, " << wcount << " bytes written, " << len << " bytes queued");
-		// push_front used intentionally, as InternalWriteData can be called
+		// InternalWriteData can be called
 		// either when flushing the queue (so any remaining unflushed data
 		// should be put back at the queue front) or when the queue is epmty
 		PWaitAndSignal lock(queueMutex);
@@ -988,21 +1011,17 @@ void SocketsReader::CleanUp()
 
 bool SocketsReader::SelectSockets(SocketSelectList & slist)
 {
-#if PTRACING
 	int ss = slist.GetSize();
-#endif
 	ConfigReloadMutex.EndRead();
 	if (!slist.Select(SocketSelectList::Read, m_timeout)) {
 		ConfigReloadMutex.StartRead();
 		return false;
 	}
 	ConfigReloadMutex.StartRead();
-#if PTRACING
 	if (PTrace::CanTrace(6)) {
 		PString msg(PString::Printf, "\t%u sockets selected from %u, total %u/%u", slist.GetSize(), ss, m_socksize, m_rmsize);
 		PTRACE(6, GetName() << msg);
 	}
-#endif
 	return true;
 }
 
@@ -1029,7 +1048,7 @@ void SocketsReader::Exec()
 	SocketSelectList slist(GetName());
 
 	if (BuildSelectList(slist)) {
-		if (SelectSockets(slist)) {
+		if (SelectSockets(slist)) {	// SelectSockets() will unlock ConfigReloadMutex while waiting
 			int ss = slist.GetSize();
 			for (int i = 0; i < ss; ++i)
 #ifdef LARGE_FDSET
@@ -1042,7 +1061,7 @@ void SocketsReader::Exec()
 	} else {
 		CleanUp();
 		ConfigReloadMutex.EndRead();
-		PTRACE(6, GetName() << " waiting...");
+		//PTRACE(6, GetName() << " waiting...");
 		Wait(SOCKETSREADER_IDLE_TIMEOUT);
 		ConfigReloadMutex.StartRead();
 	}
@@ -1105,7 +1124,11 @@ bool TCPServer::CloseListener(TCPListenSocket * socket)
 
 void TCPServer::ReadSocket(IPSocket * socket)
 {
+#if PTLIB_VER < 2120
 	if (ShutdownMutex.WillBlock()) {
+#else
+	if (!ShutdownMutex.Wait(0)) {
+#endif
 		PTRACE(4, GetName() << "\tShutdown: Rejecting call on " << socket->GetName());
 		int rej = ::accept(socket->GetHandle(), NULL, NULL);
 		::shutdown(rej, 2 /* SHUT_RDWR */ );
